@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,8 +18,8 @@
  */
 
 #include <infinoted/infinoted-dh-params.h>
-#include <infinoted/infinoted-creds.h>
 #include <infinoted/infinoted-util.h>
+#include <libinfinity/common/inf-cert-util.h>
 #include <libinfinity/inf-i18n.h>
 
 #include <glib/gstdio.h>
@@ -27,6 +27,7 @@
 
 /**
  * infinoted_dh_params_ensure:
+ * @log: A #InfinotedLog, or %NULL.
  * @credentials: A #InfCertificateCredentials.
  * @dh_params: A pointer to a gnutils_dh_params_t structure.
  * @error: Location to store error information, if any.
@@ -39,10 +40,15 @@
  * disk cache and sets them into *@dh_params. If generation fails, the
  * function returns %FALSE and @error is set.
  *
+ * @log is used to write a log message about the parameters being generated
+ * if they are not cached, so the user knows what's going on during this
+ * lengthy operation.
+ *
  * Returns: %TRUE on success or %FALSE on error.
  */
 gboolean
-infinoted_dh_params_ensure(InfCertificateCredentials* credentials,
+infinoted_dh_params_ensure(InfinotedLog* log,
+                           InfCertificateCredentials* credentials,
                            gnutls_dh_params_t* dh_params,
                            GError** error)
 {
@@ -64,17 +70,20 @@ infinoted_dh_params_ensure(InfCertificateCredentials* credentials,
   {
     /* DH params expire every week */
     /*if(st.st_mtime + 60 * 60 * 24 * 7 > time(NULL))*/
-      *dh_params = infinoted_creds_read_dh_params(filename, NULL);
+      *dh_params = inf_cert_util_read_dh_params(filename, NULL);
   }
 
   if(*dh_params == NULL)
   {
     infinoted_util_create_dirname(filename, NULL);
 
-    infinoted_util_log_info(
-      _("Generating 2048 bit Diffie-Hellman parameters..."));
+    if(log != NULL)
+    {
+      infinoted_log_info(
+        log, _("Generating 2048 bit Diffie-Hellman parameters..."));
+    }
 
-    *dh_params = infinoted_creds_create_dh_params(error);
+    *dh_params = inf_cert_util_create_dh_params(error);
 
     if(*dh_params == NULL)
     {
@@ -82,7 +91,7 @@ infinoted_dh_params_ensure(InfCertificateCredentials* credentials,
       return FALSE;
     }
 
-    infinoted_creds_write_dh_params(*dh_params, filename, NULL);
+    inf_cert_util_write_dh_params(*dh_params, filename, NULL);
   }
 
   g_free(filename);
