@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -73,7 +73,7 @@ infinoted_util_get_pidfile_path_system(void) {
 
 /**
  * infinoted_util_create_dirname:
- * @path: The filename to create a path to.
+ * @path: (type filename): The filename to create a path to.
  * @error: Location to store error information, if any.
  *
  * Creates directories leading to the given path. Does not create a directory
@@ -100,7 +100,7 @@ infinoted_util_create_dirname(const gchar* path,
  * infinoted_util_set_errno_error:
  * @error: A pointer to a #GError pointer, or %NULL.
  * @save_errno: An errno variable.
- * @prefix: A prefix string, or %NULL.
+ * @prefix: (allow-none): A prefix string, or %NULL.
  *
  * Sets @error to @save_errno with domain ERRNO_ERROR. If @prefix is
  * non-%NULL, @prefix is prefixed to @error's message, obtained by strerror().
@@ -123,11 +123,10 @@ infinoted_util_set_errno_error(GError** error,
   }
   else
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       g_quark_from_static_string("ERRNO_ERROR"),
       save_errno,
-      "%s",
       strerror(save_errno)
     );
   }
@@ -165,8 +164,9 @@ infinoted_util_daemon_set_local_pid_file_proc(void)
  * infinoted_util_daemon_pid_file_kill:
  * @sig: The signal to send to the daemon process.
  *
- * This is a thin wrapper for daemon_pid_file_kill() which uses
- * daemon_pid_file_kill_wait() if available with a timeout of 5 seconds.
+ * This is a thin wrapper for <function>daemon_pid_file_kill()</function>
+ * which uses <function>daemon_pid_file_kill_wait()</function> if available
+ * with a timeout of 5 seconds.
  *
  * Returns: 0 if the signal was sent or nonzero otherwise.
  */
