@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -47,6 +47,8 @@ struct _InfinotedPluginCertificateAuth {
   gnutls_x509_privkey_t ca_key;
   guint ca_key_index;
 
+  gint verify_flags;
+
   InfAclAccountId super_id;
   InfRequest* set_acl_request;
 };
@@ -116,10 +118,6 @@ infinoted_plugin_certificate_auth_certificate_func(InfXmppConnection* xmpp,
 
   if(chain != NULL)
   {
-    /* Note that we don't require client certificates to be signed by a CA.
-     * We only require them to be signed by one of the certificates in our
-     * list, but we don't care whether that's a CA or not. A common use case
-     * is to sign client certificates with our own server certificate. */
     res = gnutls_x509_crt_list_verify(
       inf_certificate_chain_get_raw(chain),
       inf_certificate_chain_get_n_certificates(chain),
@@ -127,7 +125,7 @@ infinoted_plugin_certificate_auth_certificate_func(InfXmppConnection* xmpp,
       plugin->n_cas,
       NULL,
       0,
-      GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT | GNUTLS_VERIFY_DISABLE_CA_SIGN,
+      plugin->verify_flags,
       &verify_result
     );
 
@@ -176,6 +174,14 @@ infinoted_plugin_certificate_auth_info_initialize(gpointer plugin_info)
   plugin->ca_key = NULL;
   plugin->ca_key_index = G_MAXUINT;
 
+  /* Note that we don't require client certificates to be signed by a CA,
+   * by default. We only require them to be signed by one of the certificates
+   * in our list, but we don't care whether that's a CA or not. A common use
+   * case is to sign client certificates with our own server certificate. */
+  plugin->verify_flags =
+    GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT |
+    GNUTLS_VERIFY_DISABLE_CA_SIGN;
+
   plugin->super_id = 0;
   plugin->set_acl_request = NULL;
 }
@@ -213,11 +219,10 @@ infinoted_plugin_certificate_auth_initialize(InfinotedPluginManager* manager,
   creds = infinoted_plugin_manager_get_credentials(manager);
   if(creds == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_plugin_certificate_auth_error_quark(),
       INFINOTED_PLUGIN_CERTIFICATE_AUTH_ERROR_NO_CREDENTIALS,
-      "%s",
       _("The certificate-auth plugin can only be used when TLS is enabled "
         "and a server certificate has been set.")
     );
@@ -276,11 +281,10 @@ infinoted_plugin_certificate_auth_initialize(InfinotedPluginManager* manager,
       gnutls_x509_privkey_deinit(plugin->ca_key);
       plugin->ca_key = NULL;
 
-      g_set_error(
+      g_set_error_literal(
         error,
         infinoted_plugin_certificate_auth_error_quark(),
         INFINOTED_PLUGIN_CERTIFICATE_AUTH_ERROR_NO_CA_FOR_KEY,
-        "%s",
         _("The given CA key does not match with any of the CA certificates")
       );
 
@@ -310,11 +314,10 @@ infinoted_plugin_certificate_auth_initialize(InfinotedPluginManager* manager,
   {
     if(plugin->ca_key == NULL)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         infinoted_plugin_certificate_auth_error_quark(),
         INFINOTED_PLUGIN_CERTIFICATE_AUTH_ERROR_NO_CA_KEY,
-        "%s",
         _("Cannot generate a superuser certificate without CA key")
       );
 
@@ -514,7 +517,8 @@ infinoted_plugin_certificate_auth_connection_added(InfXmlConnection* conn,
       xmpp,
       cert_req,
       infinoted_plugin_certificate_auth_certificate_func,
-      plugin
+      plugin,
+      NULL
     );
   }
 }
@@ -537,11 +541,53 @@ infinoted_plugin_certificate_auth_connection_removed(InfXmlConnection* conn,
       xmpp,
       GNUTLS_CERT_IGNORE,
       NULL,
+      NULL,
       NULL
     );
   }
 }
 
+static const GFlagsValue INFINOTED_PLUGIN_CERTIFICATE_AUTH_VERIFY_FLAGS[] = {
+  {
+    GNUTLS_VERIFY_DISABLE_CA_SIGN,
+    "GNUTLS_VERIFY_DISABLE_CA_SIGN",
+    "disable-ca-sign"
+  }, {
+    GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT,
+    "GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT",
+    "allow-v1-ca-certificate"
+  }, {
+    GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD2,
+    "GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD2",
+    "allow-md2"
+  }, {
+    GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD2,
+    "GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD2",
+    "allow-md5"
+  }, {
+    GNUTLS_VERIFY_DISABLE_TIME_CHECKS,
+    "GNUTLS_VERIFY_DISABLE_TIME_CHECKS",
+    "disable-time-checks"
+  }, {
+    0,
+    NULL,
+    NULL
+  }
+};
+
+static gboolean
+infinoted_plugin_certificate_auth_convert_verify_flags(gpointer in,
+                                                       gpointer out,
+                                                       GError** error)
+{
+  return infinoted_parameter_convert_flags(
+    in,
+    out,
+    INFINOTED_PLUGIN_CERTIFICATE_AUTH_VERIFY_FLAGS,
+    error
+  );
+}
+
 static const InfinotedParameterInfo INFINOTED_PLUGIN_CERTIFICATE_AUTH_OPTIONS[] = {
   { "ca-list",
     INFINOTED_PARAMETER_STRING,
@@ -589,6 +635,18 @@ static const InfinotedParameterInfo INFINOTED_PLUGIN_CERTIFICATE_AUTH_OPTIONS[]
        "or the plugin is re-loaded. This option can only be given when "
        "the \"ca-key\" parameter is set."),
     N_("FILENAME")
+  }, {
+    "verification-flags",
+    INFINOTED_PARAMETER_STRING_LIST,
+    0,
+    offsetof(InfinotedPluginCertificateAuth, verify_flags),
+    infinoted_plugin_certificate_auth_convert_verify_flags,
+    0,
+    N_("Flags to be used when verifying a client certificate. Each of these "
+       "flags weakens the security, and so should be set only when "
+       "absolutely necessary, and it should be done with care. "
+       "[Default: disable-ca-sign]"),
+    N_("flag1;flag2;[...]")
   }, {
     NULL,
     0,
