@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,6 @@
 #include <libinfgtk/inf-gtk-browser-store.h>
 #include <libinfgtk/inf-gtk-browser-model.h>
 #include <libinfinity/client/infc-browser.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-i18n.h>
 #include <libinfinity/inf-signals.h>
 
@@ -93,7 +92,12 @@ enum {
 
 #define INF_GTK_BROWSER_STORE_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_GTK_TYPE_BROWSER_STORE, InfGtkBrowserStorePrivate))
 
-static GObjectClass* parent_class;
+static void inf_gtk_browser_store_tree_model_iface_init(GtkTreeModelIface* iface);
+static void inf_gtk_browser_store_browser_model_iface_init(InfGtkBrowserModelInterface* iface);
+G_DEFINE_TYPE_WITH_CODE(InfGtkBrowserStore, inf_gtk_browser_store, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfGtkBrowserStore)
+  G_IMPLEMENT_INTERFACE(GTK_TYPE_TREE_MODEL, inf_gtk_browser_store_tree_model_iface_init)
+  G_IMPLEMENT_INTERFACE(INF_GTK_TYPE_BROWSER_MODEL, inf_gtk_browser_store_browser_model_iface_init))
 
 /*
  * Utility functions
@@ -903,6 +907,7 @@ inf_gtk_browser_store_begin_request_subscribe_session_cb(InfBrowser* browser,
 static void
 inf_gtk_browser_store_resolv_complete_func(InfDiscoveryInfo* info,
                                            InfXmlConnection* connection,
+                                           const GError* error,
                                            gpointer user_data)
 {
   InfGtkBrowserStore* store;
@@ -926,8 +931,6 @@ inf_gtk_browser_store_resolv_complete_func(InfDiscoveryInfo* info,
   store = INF_GTK_BROWSER_STORE(user_data);
   priv = INF_GTK_BROWSER_STORE_PRIVATE(store);
   new_item = inf_gtk_browser_store_find_item_by_discovery_info(store, info);
-  old_item = inf_gtk_browser_store_find_item_by_connection(store, connection);
-
   g_assert(new_item != NULL);
   g_assert(new_item->status == INF_GTK_BROWSER_MODEL_RESOLVING);
 
@@ -936,158 +939,144 @@ inf_gtk_browser_store_resolv_complete_func(InfDiscoveryInfo* info,
   tree_iter.user_data2 = GUINT_TO_POINTER(0);
   tree_iter.user_data3 = NULL;
 
-  if(old_item != NULL)
+  if(error != NULL)
   {
-    g_assert(old_item != new_item);
+    new_item->status = INF_GTK_BROWSER_MODEL_ERROR;
+    new_item->error = g_error_copy(error);
 
-    /* There is already an item with the same connection. This is perhaps from
-     * another discovery or was inserted directly. We remove the current item
-     * and move the existing one to the place of it. */
-
-    count = 0;
-    prev = NULL;
-
-    for(cur = priv->first_item; cur != NULL; cur = cur->next)
-    {
-      if(cur == old_item) { old_pos = count; prev_old = prev; }
-      if(cur == new_item) { new_pos = count; prev_new = prev; }
-      ++ count;
-      prev = cur;
-    }
-
-    inf_gtk_browser_store_remove_item(store, new_item);
-    if(old_pos > new_pos) -- old_pos;
-    else -- new_pos;
-    -- count;
-
-    /* Reorder list if the two items were not adjacent */
-    if(new_pos != old_pos)
-    {
-      /* old item is last element, but it is moved elsewhere */
-      if(old_item->next == NULL)
-        priv->last_item = prev_old;
-
-      /* Unlink old_item */
-      if(prev_old != NULL)
-        prev_old->next = old_item->next;
-      else
-        priv->first_item = old_item->next;
-
-      /* Relink */
-      old_item->next = prev_new->next;
-
-      if(prev_new != NULL)
-        prev_new->next = old_item;
-      else
-        priv->first_item = old_item;
-
-      /* old_item has been moved to end of list */
-      if(old_item->next == NULL)
-        priv->last_item = old_item;
-
-      order = g_malloc(sizeof(gint) * count);
-      if(new_pos < old_pos)
-      {
-        for(i = 0; i < new_pos; ++ i)
-          order[i] = i;
-        order[new_pos] = old_pos;
-        for(i = new_pos + 1; i <= old_pos; ++ i)
-          order[i] = i - 1;
-        for(i = old_pos + 1; i < count; ++ i)
-          order[i] = i;
-      }
-      else
-      {
-        for(i = 0; i < old_pos; ++ i)
-          order[i] = i;
-        for(i = old_pos; i < new_pos; ++ i)
-          order[i] = i + 1;
-        order[new_pos] = old_pos;
-        for(i = new_pos + 1; i < count; ++ i)
-          order[i] = i;
-      }
-
-      path = gtk_tree_path_new();
-      gtk_tree_model_rows_reordered(GTK_TREE_MODEL(store), path, NULL, order);
-      gtk_tree_path_free(path);
-
-      /* TODO: Perhaps we should emit a signal so that the view can
-       * highlight and scroll to the existing item. And also so that it can
-       * over initial root exploration. */
-
-      g_free(order);
-    }
+    path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &tree_iter);
+    gtk_tree_model_row_changed(GTK_TREE_MODEL(store), path, &tree_iter);
+    gtk_tree_path_free(path);
   }
   else
   {
-    path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &tree_iter);
+    old_item =
+      inf_gtk_browser_store_find_item_by_connection(store, connection);
 
-    browser = infc_browser_new(
-      priv->io,
-      priv->communication_manager,
-      connection
-    );
-
-    /* The connection is not set if the browser could not find a "central"
-     * method for the connection's network. */
-    /* TODO: Set error */
-    if(infc_browser_get_connection(browser) != NULL)
+    if(old_item != NULL)
     {
-      inf_gtk_browser_store_item_set_browser(
-        store,
-        new_item,
-        path,
-        INF_BROWSER(browser)
-      );
+      g_assert(old_item != new_item);
+
+      /* There is already an item with the same connection. This is perhaps
+       * from another discovery or was inserted directly. We remove the
+       * current item and move the existing one to the place of it. */
+
+      count = 0;
+      prev = NULL;
+
+      for(cur = priv->first_item; cur != NULL; cur = cur->next)
+      {
+        if(cur == old_item) { old_pos = count; prev_old = prev; }
+        if(cur == new_item) { new_pos = count; prev_new = prev; }
+        ++ count;
+        prev = cur;
+      }
+
+      inf_gtk_browser_store_remove_item(store, new_item);
+      if(old_pos > new_pos) -- old_pos;
+      else -- new_pos;
+      -- count;
+
+      /* Reorder list if the two items were not adjacent */
+      if(new_pos != old_pos)
+      {
+        /* old item is last element, but it is moved elsewhere */
+        if(old_item->next == NULL)
+          priv->last_item = prev_old;
+
+        /* Unlink old_item */
+        if(prev_old != NULL)
+          prev_old->next = old_item->next;
+        else
+          priv->first_item = old_item->next;
+
+        /* Relink */
+        old_item->next = prev_new->next;
+
+        if(prev_new != NULL)
+          prev_new->next = old_item;
+        else
+          priv->first_item = old_item;
+
+        /* old_item has been moved to end of list */
+        if(old_item->next == NULL)
+          priv->last_item = old_item;
+
+        order = g_malloc(sizeof(gint) * count);
+        if(new_pos < old_pos)
+        {
+          for(i = 0; i < new_pos; ++ i)
+            order[i] = i;
+          order[new_pos] = old_pos;
+          for(i = new_pos + 1; i <= old_pos; ++ i)
+            order[i] = i - 1;
+          for(i = old_pos + 1; i < count; ++ i)
+            order[i] = i;
+        }
+        else
+        {
+          for(i = 0; i < old_pos; ++ i)
+            order[i] = i;
+          for(i = old_pos; i < new_pos; ++ i)
+            order[i] = i + 1;
+          order[new_pos] = old_pos;
+          for(i = new_pos + 1; i < count; ++ i)
+            order[i] = i;
+        }
+
+        path = gtk_tree_path_new();
+        gtk_tree_model_rows_reordered(
+          GTK_TREE_MODEL(store),
+          path,
+          NULL,
+          order
+        );
+        gtk_tree_path_free(path);
+
+        /* TODO: Perhaps we should emit a signal so that the view can
+         * highlight and scroll to the existing item. And also so that it can
+         * over initial root exploration. */
+
+        g_free(order);
+      }
     }
+    else
+    {
+      path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &tree_iter);
 
-    g_object_unref(G_OBJECT(browser));
-    gtk_tree_path_free(path);
+      browser = infc_browser_new(
+        priv->io,
+        priv->communication_manager,
+        connection
+      );
+
+      /* The connection is not set if the browser could not find a "central"
+       * method for the connection's network. */
+      /* TODO: Set error */
+      if(infc_browser_get_connection(browser) != NULL)
+      {
+        inf_gtk_browser_store_item_set_browser(
+          store,
+          new_item,
+          path,
+          INF_BROWSER(browser)
+        );
+      }
+
+      g_object_unref(G_OBJECT(browser));
+      gtk_tree_path_free(path);
+    }
   }
 }
 
-static void
-inf_gtk_browser_store_resolv_error_func(InfDiscoveryInfo* info,
-                                        const GError* error,
-                                        gpointer user_data)
-{
-  InfGtkBrowserStore* store;
-  InfGtkBrowserStorePrivate* priv;
-  InfGtkBrowserStoreItem* item;
-  GtkTreeIter tree_iter;
-  GtkTreePath* path;
-
-  store = INF_GTK_BROWSER_STORE(user_data);
-  priv = INF_GTK_BROWSER_STORE_PRIVATE(store);
-  item = inf_gtk_browser_store_find_item_by_discovery_info(store, info);
-
-  g_assert(item != NULL);
-  g_assert(item->status == INF_GTK_BROWSER_MODEL_RESOLVING);
-  item->status = INF_GTK_BROWSER_MODEL_ERROR;
-  item->error = g_error_copy(error);
-
-  tree_iter.stamp = priv->stamp;
-  tree_iter.user_data = item;
-  tree_iter.user_data2 = GUINT_TO_POINTER(0);
-  tree_iter.user_data3 = NULL;
-
-  path = gtk_tree_model_get_path(GTK_TREE_MODEL(store), &tree_iter);
-  gtk_tree_model_row_changed(GTK_TREE_MODEL(store), path, &tree_iter);
-  gtk_tree_path_free(path);
-}
-
 /*
  * GObject overrides
  */
 
 static void
-inf_gtk_browser_store_init(GTypeInstance* instance,
-                           gpointer g_class)
+inf_gtk_browser_store_init(InfGtkBrowserStore* store)
 {
-  InfGtkBrowserStore* store;
   InfGtkBrowserStorePrivate* priv;
-
-  store = INF_GTK_BROWSER_STORE(instance);
   priv = INF_GTK_BROWSER_STORE_PRIVATE(store);
 
   priv->stamp = g_random_int();
@@ -1144,7 +1133,7 @@ inf_gtk_browser_store_dispose(GObject* object)
     priv->io = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_gtk_browser_store_parent_class)->dispose(object);
 }
 
 static void
@@ -2182,7 +2171,6 @@ inf_gtk_browser_store_browser_model_resolve(InfGtkBrowserModel* model,
     discovery,
     info,
     inf_gtk_browser_store_resolv_complete_func,
-    inf_gtk_browser_store_resolv_error_func,
     model
   );
 }
@@ -2229,17 +2217,10 @@ inf_gtk_browser_store_browser_iter_to_tree_iter(InfGtkBrowserModel* model,
  */
 
 static void
-inf_gtk_browser_store_class_init(gpointer g_class,
-                                 gpointer class_data)
+inf_gtk_browser_store_class_init(InfGtkBrowserStoreClass* browser_store_class)
 {
   GObjectClass* object_class;
-  InfGtkBrowserStoreClass* browser_store_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  browser_store_class = INF_GTK_BROWSER_STORE_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfGtkBrowserStorePrivate));
+  object_class = G_OBJECT_CLASS(browser_store_class);
 
   object_class->dispose = inf_gtk_browser_store_dispose;
   object_class->set_property = inf_gtk_browser_store_set_property;
@@ -2271,12 +2252,8 @@ inf_gtk_browser_store_class_init(gpointer g_class,
 }
 
 static void
-inf_gtk_browser_store_tree_model_init(gpointer g_iface,
-                                      gpointer iface_data)
+inf_gtk_browser_store_tree_model_iface_init(GtkTreeModelIface* iface)
 {
-  GtkTreeModelIface* iface;
-  iface = (GtkTreeModelIface*)g_iface;
-
   iface->get_flags = inf_gtk_browser_store_tree_model_get_flags;
   iface->get_n_columns = inf_gtk_browser_store_tree_model_get_n_columns;
   iface->get_column_type = inf_gtk_browser_store_tree_model_get_column_type;
@@ -2292,12 +2269,9 @@ inf_gtk_browser_store_tree_model_init(gpointer g_iface,
 }
 
 static void
-inf_gtk_browser_store_browser_model_init(gpointer g_iface,
-                                         gpointer iface_data)
+inf_gtk_browser_store_browser_model_iface_init(
+  InfGtkBrowserModelInterface* iface)
 {
-  InfGtkBrowserModelIface* iface;
-  iface = (InfGtkBrowserModelIface*)g_iface;
-
   iface->set_browser = inf_gtk_browser_store_browser_model_set_browser;
   iface->resolve = inf_gtk_browser_store_browser_model_resolve;
   /* inf_gtk_browser_store_browser_model_browser_iter_to_tree_iter would be
@@ -2306,74 +2280,19 @@ inf_gtk_browser_store_browser_model_init(gpointer g_iface,
     inf_gtk_browser_store_browser_iter_to_tree_iter;
 }
 
-GType
-inf_gtk_browser_store_get_type(void)
-{
-  static GType browser_store_type = 0;
-
-  if(!browser_store_type)
-  {
-    static const GTypeInfo browser_store_type_info = {
-      sizeof(InfGtkBrowserStoreClass),    /* class_size */
-      NULL,                               /* base_init */
-      NULL,                               /* base_finalize */
-      inf_gtk_browser_store_class_init,   /* class_init */
-      NULL,                               /* class_finalize */
-      NULL,                               /* class_data */
-      sizeof(InfGtkBrowserStore),         /* instance_size */
-      0,                                  /* n_preallocs */
-      inf_gtk_browser_store_init,         /* instance_init */
-      NULL                                /* value_table */
-    };
-
-    static const GInterfaceInfo tree_model_info = {
-      inf_gtk_browser_store_tree_model_init,
-      NULL,
-      NULL
-    };
-
-    static const GInterfaceInfo browser_model_info = {
-      inf_gtk_browser_store_browser_model_init,
-      NULL,
-      NULL
-    };
-
-    browser_store_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfGtkBrowserStore",
-      &browser_store_type_info,
-      0
-    );
-
-    g_type_add_interface_static(
-      browser_store_type,
-      GTK_TYPE_TREE_MODEL,
-      &tree_model_info
-    );
-
-    g_type_add_interface_static(
-      browser_store_type,
-      INF_GTK_TYPE_BROWSER_MODEL,
-      &browser_model_info
-    );
-  }
-
-  return browser_store_type;
-}
-
 /*
  * Public API.
  */
 
 /**
- * inf_gtk_browser_store_new:
+ * inf_gtk_browser_store_new: (constructor)
  * @io: A #InfIo object for the created #InfcBrowser to schedule timeouts.
  * @comm_manager: The #InfCommunicationManager with which to explore
  * remote directories.
  *
  * Creates a new #InfGtkBrowserStore.
  *
- * Return Value: A new #InfGtkBrowserStore.
+ * Returns: (transfer full): A new #InfGtkBrowserStore.
  **/
 InfGtkBrowserStore*
 inf_gtk_browser_store_new(InfIo* io,
@@ -2453,7 +2372,7 @@ inf_gtk_browser_store_add_discovery(InfGtkBrowserStore* store,
  * inf_gtk_browser_store_add_connection:
  * @store: A #InfGtkBrowserStore.
  * @connection: A #InfXmlConnection.
- * @name: Name for the item, or %NULL.
+ * @name: (allow-none): Name for the item, or %NULL.
  *
  * This function adds a connection to the @store. @store will show up
  * an item for the connection if there is not already one. This allows to
@@ -2464,7 +2383,7 @@ inf_gtk_browser_store_add_discovery(InfGtkBrowserStore* store,
  * The return value is the #InfBrowser for the added connection, or an
  * existing browser for the connection, if there is any.
  *
- * Returns: An #InfBrowser for the new connection.
+ * Returns: (transfer none): An #InfBrowser for the new connection.
  **/
 InfBrowser*
 inf_gtk_browser_store_add_connection(InfGtkBrowserStore* store,
