@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,11 +17,6 @@
  * MA 02110-1301, USA.
  */
 
-#include <libinfinity/adopted/inf-adopted-request-log.h>
-#include <libinfinity/inf-marshal.h>
-
-#include <string.h> /* For (g_)memmove */
-
 /**
  * SECTION:inf-adopted-request-log
  * @title: InfAdoptedRequestLog
@@ -41,6 +36,10 @@
  * or Redo requests do not refer to some request that is about to be removed.
  */
 
+#include <libinfinity/adopted/inf-adopted-request-log.h>
+
+#include <string.h> /* For (g_)memmove */
+
 typedef struct _InfAdoptedRequestLogCleanupCacheData
   InfAdoptedRequestLogCleanupCacheData;
 struct _InfAdoptedRequestLogCleanupCacheData {
@@ -98,10 +97,12 @@ enum {
 #define INF_ADOPTED_REQUEST_LOG_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_ADOPTED_TYPE_REQUEST_LOG, InfAdoptedRequestLogPrivate))
 #define INF_ADOPTED_REQUEST_LOG_PRIVATE(obj)     ((InfAdoptedRequestLogPrivate*)(obj)->priv)
 
-static GObjectClass* parent_class;
 static const guint INF_ADOPTED_REQUEST_LOG_INC = 0x80;
 static guint request_log_signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE_WITH_CODE(InfAdoptedRequestLog, inf_adopted_request_log, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfAdoptedRequestLog))
+
 #ifdef INF_ADOPTED_REQUEST_LOG_CHECK_RELATED
 static void
 inf_adopted_request_log_verify_related(InfAdoptedRequestLog* log)
@@ -285,13 +286,9 @@ inf_adopted_request_log_find_associated(InfAdoptedRequestLog* log,
  */
 
 static void
-inf_adopted_request_log_init(GTypeInstance* instance,
-                             gpointer g_class)
+inf_adopted_request_log_init(InfAdoptedRequestLog* log)
 {
-  InfAdoptedRequestLog* log;
   InfAdoptedRequestLogPrivate* priv;
-
-  log = INF_ADOPTED_REQUEST_LOG(instance);
   log->priv = INF_ADOPTED_REQUEST_LOG_GET_PRIVATE(log);
   priv = INF_ADOPTED_REQUEST_LOG_PRIVATE(log);
 
@@ -331,7 +328,7 @@ inf_adopted_request_log_dispose(GObject* object)
   priv->end = 0;
   priv->offset = 0;
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_adopted_request_log_parent_class)->dispose(object);
 }
 
 static void
@@ -345,7 +342,7 @@ inf_adopted_request_log_finalize(GObject* object)
 
   g_free(priv->entries);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_adopted_request_log_parent_class)->finalize(object);
 }
 
 static void
@@ -630,17 +627,11 @@ inf_adopted_request_log_add_request_handler(InfAdoptedRequestLog* log,
 }
 
 static void
-inf_adopted_request_log_class_init(gpointer g_class,
-                                   gpointer class_data)
+inf_adopted_request_log_class_init(
+  InfAdoptedRequestLogClass* request_log_class)
 {
   GObjectClass* object_class;
-  InfAdoptedRequestLogClass* request_log_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  request_log_class = INF_ADOPTED_REQUEST_LOG_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfAdoptedRequestLogPrivate));
+  object_class = G_OBJECT_CLASS(request_log_class);
 
   object_class->dispose = inf_adopted_request_log_dispose;
   object_class->finalize = inf_adopted_request_log_finalize;
@@ -717,7 +708,7 @@ inf_adopted_request_log_class_init(gpointer g_class,
 
   /**
    * InfAdoptedRequestLog::add-request:
-   * @request_log: The #InfAdoptedRequestLog to which a new request is added.
+   * @log: The #InfAdoptedRequestLog to which a new request is added.
    * @request: The new request being added.
    *
    * This signal is emitted whenever a new request is added to the request log
@@ -729,56 +720,25 @@ inf_adopted_request_log_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfAdoptedRequestLogClass, add_request),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_ADOPTED_TYPE_REQUEST
   );
 }
 
-GType
-inf_adopted_request_log_get_type(void)
-{
-  static GType request_log_type = 0;
-
-  if(!request_log_type)
-  {
-    static const GTypeInfo request_log_type_info = {
-      sizeof(InfAdoptedRequestLogClass),    /* class_size */
-      NULL,                                 /* base_init */
-      NULL,                                 /* base_finalize */
-      inf_adopted_request_log_class_init,   /* class_init */
-      NULL,                                 /* class_finalize */
-      NULL,                                 /* class_data */
-      sizeof(InfAdoptedRequestLog),         /* instance_size */
-      0,                                    /* n_preallocs */
-      inf_adopted_request_log_init,         /* instance_init */
-      NULL                                  /* value_table */
-    };
-
-    request_log_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfAdoptedRequestLog",
-      &request_log_type_info,
-      0
-    );
-  }
-
-  return request_log_type;
-}
-
 /*
  * Public API
  */
 
 /**
- * inf_adopted_request_log_new:
+ * inf_adopted_request_log_new: (constructor)
  * @user_id: The ID of the #InfAdoptedUser to create a request log for. The
  * request log only contains requests of that particular user.
  *
  * Creates a new #InfAdoptedRequestLog for the user with the given ID.
  *
- * Returns: A new #InfAdoptedRequestLog.
+ * Returns: (transfer full): A new #InfAdoptedRequestLog.
  **/
 InfAdoptedRequestLog*
 inf_adopted_request_log_new(guint user_id)
@@ -802,7 +762,7 @@ inf_adopted_request_log_new(guint user_id)
  *
  * Returns the ID of the user whose requests @log contains.
  *
- * Return Value: The log's user ID.
+ * Returns: The log's user ID.
  **/
 guint
 inf_adopted_request_log_get_user_id(InfAdoptedRequestLog* log)
@@ -817,7 +777,7 @@ inf_adopted_request_log_get_user_id(InfAdoptedRequestLog* log)
  *
  * Returns the first index (i.e. the index of the oldest request) in the log.
  *
- * Return Value: The first index in the log.
+ * Returns: The first index in the log.
  **/
 guint
 inf_adopted_request_log_get_begin(InfAdoptedRequestLog* log)
@@ -835,7 +795,7 @@ inf_adopted_request_log_get_begin(InfAdoptedRequestLog* log)
  * inf_adopted_request_log_get_end() - inf_adopted_request_log_get_begin()
  * reveals the number of requests in the log.
  *
- * Return Value: The index of the next request in the log.
+ * Returns: The index of the next request in the log.
  **/
 guint
 inf_adopted_request_log_get_end(InfAdoptedRequestLog* log)
@@ -917,8 +877,8 @@ inf_adopted_request_log_set_begin(InfAdoptedRequestLog* log,
  * Returns the request with the given index. Such a request must exist in
  * @log.
  *
- * Return Value: A #InfAdoptedRequest. The request is owned by the request
- * log, you do not need to free it.
+ * Returns: (transfer none): A #InfAdoptedRequest. The request is owned by
+ * the request log, you do not need to free it.
  **/
 InfAdoptedRequest*
 inf_adopted_request_log_get_request(InfAdoptedRequestLog* log,
@@ -1060,7 +1020,8 @@ inf_adopted_request_log_remove_requests(InfAdoptedRequestLog* log,
  * request, if any. If @request is a %INF_ADOPTED_REQUEST UNDO request, this
  * returns a request that redoes @request, if any.
  *
- * Return Value: The next associated request of @request, or %NULL.
+ * Returns: (transfer none) (allow-none): The next associated request of
+ * @request, or %NULL.
  **/
 InfAdoptedRequest*
 inf_adopted_request_log_next_associated(InfAdoptedRequestLog* log,
@@ -1102,7 +1063,8 @@ inf_adopted_request_log_next_associated(InfAdoptedRequestLog* log,
  * of its own user must be equivalent to inf_adopted_request_log_get_end(),
  * in which case @request is treated as it if was the newest request in @log.
  *
- * Return Value: The previous associated request of @request, or %NULL.
+ * Returns: (transfer none) (allow-none): The previous associated request of
+ * @request, or %NULL.
  **/
 InfAdoptedRequest*
 inf_adopted_request_log_prev_associated(InfAdoptedRequestLog* log,
@@ -1170,8 +1132,8 @@ inf_adopted_request_log_prev_associated(InfAdoptedRequestLog* log,
  * of its own user must be equivalent to inf_adopted_request_log_get_end(),
  * in which case @request is treated as it if was the newest request in @log.
  *
- * Return Value: The original request of @request. This function never
- * returns %NULL.
+ * Returns: (transfer none): The original request of @request.
+ * This function never returns %NULL.
  **/
 InfAdoptedRequest*
 inf_adopted_request_log_original_request(InfAdoptedRequestLog* log,
@@ -1241,7 +1203,8 @@ inf_adopted_request_log_original_request(InfAdoptedRequestLog* log,
  * Returns the request that would be undone if a undo request was added to
  * the request log.
  *
- * Return Value: The next request to be undone, or %NULL.
+ * Returns: (transfer none) (allow-none): The next request to be undone, or
+ * %NULL.
  **/
 InfAdoptedRequest*
 inf_adopted_request_log_next_undo(InfAdoptedRequestLog* log)
@@ -1263,7 +1226,8 @@ inf_adopted_request_log_next_undo(InfAdoptedRequestLog* log)
  * Returns the request that would be redone if a redo request was added to
  * the request log.
  *
- * Return Value: The next request to be redone, or %NULL.
+ * Returns: (transfer none) (allow-none): The next request to be redone, or
+ * %NULL.
  **/
 InfAdoptedRequest*
 inf_adopted_request_log_next_redo(InfAdoptedRequestLog* log)
@@ -1290,8 +1254,8 @@ inf_adopted_request_log_next_redo(InfAdoptedRequestLog* log)
  * Note that the sets of related requests within a request log are
  * disjoint.
  *
- * Return Value: The newest request in @log being related to the @n<!-- -->th
- * request.
+ * Returns: (transfer none): The newest request in @log being related to the
+ * @n<!-- -->th request.
  **/
 InfAdoptedRequest*
 inf_adopted_request_log_upper_related(InfAdoptedRequestLog* log,
@@ -1324,8 +1288,8 @@ inf_adopted_request_log_upper_related(InfAdoptedRequestLog* log,
  * Note that the sets of related requests within a request log are
  * disjoint.
  *
- * Return Value: The oldest request in @log being related to the @n<!-- -->th
- * request.
+ * Returns: (transfer none): The oldest request in @log being related to the
+ * @n<!-- -->th request.
  **/
 InfAdoptedRequest*
 inf_adopted_request_log_lower_related(InfAdoptedRequestLog* log,
@@ -1414,7 +1378,8 @@ inf_adopted_request_log_add_cached_request(InfAdoptedRequestLog* log,
  * See inf_adopted_request_log_add_cached_request() for an explanation of
  * the request cache.
  *
- * Returns: The cached #InfAdoptedRequest according to @vec, or %NULL.
+ * Returns: (transfer none) (allow-none): The cached #InfAdoptedRequest
+ * according to @vec, or %NULL.
  */
 InfAdoptedRequest*
 inf_adopted_request_log_lookup_cached_request(InfAdoptedRequestLog* log,
