@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,8 +17,6 @@
  * MA 02110-1301, USA.
  */
 
-#include <libinfinity/adopted/inf-adopted-request.h>
-
 /**
  * SECTION:inf-adopted-request
  * @title: InfAdoptedRequest
@@ -37,12 +35,37 @@
  * buffer and the user ID of the #InfAdoptedUser having generated the request.
  */
 
+#include <libinfinity/adopted/inf-adopted-request.h>
+#include <libinfinity/inf-define-enum.h>
+
+static const GEnumValue inf_adopted_request_type_values[] = {
+  {
+    INF_ADOPTED_REQUEST_DO,
+    "INF_ADOPTED_REQUEST_DO",
+    "do",
+  }, {
+    INF_ADOPTED_REQUEST_UNDO,
+    "INF_ADOPTED_REQUEST_UNDO",
+    "undo",
+  }, {
+    INF_ADOPTED_REQUEST_REDO,
+    "INF_ADOPTED_REQUEST_REDO",
+    "redo"
+  }, {
+    0,
+    NULL,
+    NULL
+  }
+};
+
 typedef struct _InfAdoptedRequestPrivate InfAdoptedRequestPrivate;
 struct _InfAdoptedRequestPrivate {
   InfAdoptedRequestType type;
   InfAdoptedStateVector* vector;
   guint user_id;
   InfAdoptedOperation* operation;
+  gint64 received;
+  gint64 executed;
 };
 
 enum {
@@ -52,22 +75,24 @@ enum {
   PROP_TYPE,
   PROP_VECTOR,
   PROP_USER_ID,
-  PROP_OPERATION
+  PROP_OPERATION,
+  PROP_RECEIVED,
+
+  /* read/write */
+  PROP_EXECUTED
 };
 
 #define INF_ADOPTED_REQUEST_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_ADOPTED_TYPE_REQUEST, InfAdoptedRequestPrivate))
 #define INF_ADOPTED_REQUEST_PRIVATE(obj)     ((InfAdoptedRequestPrivate*)(obj)->priv)
 
-static GObjectClass* parent_class;
+INF_DEFINE_ENUM_TYPE(InfAdoptedRequestType, inf_adopted_request_type, inf_adopted_request_type_values)
+G_DEFINE_TYPE_WITH_CODE(InfAdoptedRequest, inf_adopted_request, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfAdoptedRequest))
 
 static void
-inf_adopted_request_init(GTypeInstance* instance,
-                         gpointer g_class)
+inf_adopted_request_init(InfAdoptedRequest* request)
 {
-  InfAdoptedRequest* request;
   InfAdoptedRequestPrivate* priv;
-
-  request = INF_ADOPTED_REQUEST(instance);
   request->priv = INF_ADOPTED_REQUEST_GET_PRIVATE(request);
   priv = INF_ADOPTED_REQUEST_PRIVATE(request);
 
@@ -75,6 +100,9 @@ inf_adopted_request_init(GTypeInstance* instance,
   priv->vector = NULL;
   priv->user_id = 0;
   priv->operation = NULL;
+
+  priv->received = 0;
+  priv->executed = 0;
 }
 
 static void
@@ -92,7 +120,7 @@ inf_adopted_request_dispose(GObject* object)
     priv->operation = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_adopted_request_parent_class)->dispose(object);
 }
 
 static void
@@ -107,7 +135,7 @@ inf_adopted_request_finalize(GObject* object)
  if(priv->vector != NULL)
   inf_adopted_state_vector_free(priv->vector);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_adopted_request_parent_class)->finalize(object);
 }
 
 static void
@@ -140,6 +168,13 @@ inf_adopted_request_set_property(GObject* object,
     g_assert(priv->operation == NULL); /* construct only */
     priv->operation = INF_ADOPTED_OPERATION(g_value_dup_object(value));
     break;
+  case PROP_RECEIVED:
+    g_assert(priv->received == 0); /* construct only */
+    priv->received = g_value_get_int64(value);
+    break;
+  case PROP_EXECUTED:
+    priv->executed = g_value_get_int64(value);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -172,6 +207,12 @@ inf_adopted_request_get_property(GObject* object,
   case PROP_OPERATION:
     g_value_set_object(value, G_OBJECT(priv->operation));
     break;
+  case PROP_RECEIVED:
+    g_value_set_int64(value, priv->received);
+    break;
+  case PROP_EXECUTED:
+    g_value_set_int64(value, priv->executed);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -179,14 +220,10 @@ inf_adopted_request_get_property(GObject* object,
 }
 
 static void
-inf_adopted_request_class_init(gpointer g_class,
-                               gpointer class_data)
+inf_adopted_request_class_init(InfAdoptedRequestClass* request_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfAdoptedRequestPrivate));
+  object_class = G_OBJECT_CLASS(request_class);
 
   object_class->dispose = inf_adopted_request_dispose;
   object_class->finalize = inf_adopted_request_finalize;
@@ -243,89 +280,52 @@ inf_adopted_request_class_init(gpointer g_class,
       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY
     )
   );
-}
 
-GType
-inf_adopted_request_type_get_type(void)
-{
-  static GType request_type_type = 0;
+  g_object_class_install_property(
+    object_class,
+    PROP_RECEIVED,
+    g_param_spec_int64(
+      "received",
+      "Received",
+      "Time the request was received, in microseconds",
+      G_MININT64,
+      G_MAXINT64,
+      0,
+      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY
+    )
+  );
 
-  if(!request_type_type)
-  {
-    static const GEnumValue request_type_type_values[] = {
-      {
-        INF_ADOPTED_REQUEST_DO,
-        "INF_ADOPTED_REQUEST_DO",
-        "do",
-      }, {
-        INF_ADOPTED_REQUEST_UNDO,
-        "INF_ADOPTED_REQUEST_UNDO",
-        "undo",
-      }, {
-        INF_ADOPTED_REQUEST_REDO,
-        "INF_ADOPTED_REQUEST_REDO",
-        "redo"
-      }, {
-        0,
-        NULL,
-        NULL
-      }
-    };
-
-    request_type_type = g_enum_register_static(
-      "InfAdoptedRequestType",
-      request_type_type_values
-    );
-  }
-
-  return request_type_type;
-}
-
-GType
-inf_adopted_request_get_type(void)
-{
-  static GType request_type = 0;
-
-  if(!request_type)
-  {
-    static const GTypeInfo request_type_info = {
-      sizeof(InfAdoptedRequestClass),   /* class_size */
-      NULL,                             /* base_init */
-      NULL,                             /* base_finalize */
-      inf_adopted_request_class_init,   /* class_init */
-      NULL,                             /* class_finalize */
-      NULL,                             /* class_data */
-      sizeof(InfAdoptedRequest),        /* instance_size */
-      0,                                /* n_preallocs */
-      inf_adopted_request_init,         /* instance_init */
-      NULL                              /* value_table */
-    };
-
-    request_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfAdoptedRequest",
-      &request_type_info,
-      0
-    );
-  }
-
-  return request_type;
+  g_object_class_install_property(
+    object_class,
+    PROP_EXECUTED,
+    g_param_spec_int64(
+      "executed",
+      "Executed",
+      "Time the request was executed, in microseconds",
+      G_MININT64,
+      G_MAXINT64,
+      0,
+      G_PARAM_READWRITE
+    )
+  );
 }
 
 /**
- * inf_adopted_request_new_do:
+ * inf_adopted_request_new_do: (constructor)
  * @vector: The vector time at which the request was made.
  * @user_id: The ID of the user that made the request.
  * @operation: The operation the user performed.
+ * @received: Time the request was received, in microseconds since the epoch.
  *
  * Creates a new #InfAdoptedRequest with type %INF_ADOPTED_REQUEST_DO.
  *
- * Return Value: A new DO request.
+ * Returns: (transfer full): A new DO request.
  **/
 InfAdoptedRequest*
 inf_adopted_request_new_do(InfAdoptedStateVector* vector,
                            guint user_id,
-                           InfAdoptedOperation* operation)
+                           InfAdoptedOperation* operation,
+                           gint64 received)
 {
   GObject* object;
 
@@ -339,6 +339,7 @@ inf_adopted_request_new_do(InfAdoptedStateVector* vector,
     "vector", vector,
     "user-id", user_id,
     "operation", operation,
+    "received", received,
     NULL
   );
 
@@ -346,20 +347,22 @@ inf_adopted_request_new_do(InfAdoptedStateVector* vector,
 }
 
 /**
- * inf_adopted_request_new_undo:
+ * inf_adopted_request_new_undo: (constructor)
  * @vector: The vector time at which the request was made.
  * @user_id: The ID of the user that made the request.
+ * @received: Time the request was received, in microseconds since the epoch.
  *
  * Creates a new #InfAdoptedRequest with type %INF_ADOPTED_REQUEST_UNDO.
  * The operation performed is implicitely defined by reverting the operation
  * of the associated DO or REDO request, but must still be computed by
  * #InfAdoptedAlgorithm.
  *
- * Return Value: A new UNDO request.
+ * Returns: (transfer full): A new UNDO request.
  **/
 InfAdoptedRequest*
 inf_adopted_request_new_undo(InfAdoptedStateVector* vector,
-                             guint user_id)
+                             guint user_id,
+                             gint64 received)
 {
   GObject* object;
 
@@ -371,6 +374,7 @@ inf_adopted_request_new_undo(InfAdoptedStateVector* vector,
     "type", INF_ADOPTED_REQUEST_UNDO,
     "vector", vector,
     "user-id", user_id,
+    "received", received,
     NULL
   );
   
@@ -378,20 +382,22 @@ inf_adopted_request_new_undo(InfAdoptedStateVector* vector,
 }
 
 /**
- * inf_adopted_request_new_redo:
+ * inf_adopted_request_new_redo: (constructor)
  * @vector: The vector time at which the request was made.
  * @user_id: The ID of the user that made the request.
+ * @received: Time the request was received, in microseconds since the epoch.
  *
  * Creates a new #InfAdoptedRequest with type %INF_ADOPTED_REQUEST_REDO. The
  * operation performed is implicitely defined by reverting the operation of
  * the associated UNDO request, but must still be computed by
  * #InfAdoptedAlgorithm.
  *
- * Return Value: A new REDO request.
+ * Returns: (transfer full): A new REDO request.
  **/
 InfAdoptedRequest*
 inf_adopted_request_new_redo(InfAdoptedStateVector* vector,
-                             guint user_id)
+                             guint user_id,
+                             gint64 received)
 {
   GObject* object;
   
@@ -403,6 +409,7 @@ inf_adopted_request_new_redo(InfAdoptedStateVector* vector,
     "type", INF_ADOPTED_REQUEST_REDO,
     "vector", vector,
     "user-id", user_id,
+    "received", received,
     NULL
   );
   
@@ -415,12 +422,13 @@ inf_adopted_request_new_redo(InfAdoptedStateVector* vector,
  *
  * Creates a copy of @request with an initial reference count of 1.
  *
- * Return Value: A new #InfAdoptedRequest.
+ * Returns: (transfer full): A new #InfAdoptedRequest.
  **/
 InfAdoptedRequest*
 inf_adopted_request_copy(InfAdoptedRequest* request)
 {
   InfAdoptedRequestPrivate* priv;
+  InfAdoptedRequestPrivate* new_priv;
   GObject* object;
   
   g_return_val_if_fail(INF_ADOPTED_IS_REQUEST(request), NULL);
@@ -434,6 +442,7 @@ inf_adopted_request_copy(InfAdoptedRequest* request)
       "vector", priv->vector,
       "user-id", priv->user_id,
       "operation", priv->operation,
+      "received", priv->received,
       NULL
     );
   }
@@ -444,10 +453,13 @@ inf_adopted_request_copy(InfAdoptedRequest* request)
       "type", priv->type,
       "vector", priv->vector,
       "user-id", priv->user_id,
+      "received", priv->received,
       NULL
     );
   }
 
+  new_priv = INF_ADOPTED_REQUEST_PRIVATE(INF_ADOPTED_REQUEST(object));
+  new_priv->executed = priv->executed;
   return INF_ADOPTED_REQUEST(object);
 }
 
@@ -457,7 +469,7 @@ inf_adopted_request_copy(InfAdoptedRequest* request)
  *
  * Returns the request type of @request.
  *
- * Return Value: The type of @request.
+ * Returns: The type of @request.
  **/
 InfAdoptedRequestType
 inf_adopted_request_get_request_type(InfAdoptedRequest* request)
@@ -477,8 +489,8 @@ inf_adopted_request_get_request_type(InfAdoptedRequest* request)
  * Returns the vector time the request was made i.e. its operation can be
  * applied to the buffer.
  *
- * Return Value: The state vector of @request. The returned value should
- * not be freed, it is owned by the #InfAdoptedRequest.
+ * Returns: (transfer none): The state vector of @request. The returned value
+ * should not be freed, it is owned by the #InfAdoptedRequest.
  **/
 InfAdoptedStateVector*
 inf_adopted_request_get_vector(InfAdoptedRequest* request)
@@ -493,7 +505,7 @@ inf_adopted_request_get_vector(InfAdoptedRequest* request)
  *
  * Returns the user ID of the user that issued @request.
  *
- * Return Value: The request's user ID.
+ * Returns: The request's user ID.
  **/
 guint
 inf_adopted_request_get_user_id(InfAdoptedRequest* request)
@@ -509,7 +521,7 @@ inf_adopted_request_get_user_id(InfAdoptedRequest* request)
  * Returns the operation carried by the request. This can only be called if
  * the request's type is %INF_ADOPTED_REQUEST_DO.
  *
- * Return Value: The request's operation.
+ * Returns: (transfer none): The request's operation.
  **/
 InfAdoptedOperation*
 inf_adopted_request_get_operation(InfAdoptedRequest* request)
@@ -517,7 +529,7 @@ inf_adopted_request_get_operation(InfAdoptedRequest* request)
   InfAdoptedRequestPrivate* priv;
 
   g_return_val_if_fail(INF_ADOPTED_IS_REQUEST(request), NULL);
-  
+
   priv = INF_ADOPTED_REQUEST_PRIVATE(request);
   g_return_val_if_fail(priv->operation != NULL, NULL);
 
@@ -538,23 +550,90 @@ inf_adopted_request_get_index(InfAdoptedRequest* request)
 {
   InfAdoptedRequestPrivate* priv;
 
-  g_return_val_if_fail(INF_ADOPTED_IS_REQUEST(request), NULL);
+  g_return_val_if_fail(INF_ADOPTED_IS_REQUEST(request), 0);
 
   priv = INF_ADOPTED_REQUEST_PRIVATE(request);
   return inf_adopted_state_vector_get(priv->vector, priv->user_id);
 }
 
+/**
+ * inf_adopted_request_get_receive_time:
+ * @request: A #InfAdoptedRequest.
+ *
+ * Returns the time when the request was received, or, if it's a local
+ * request, generated. The time is given in microseconds since January 1,
+ * 1970.
+ *
+ * Returns: Time when the request was received.
+ */
+gint64
+inf_adopted_request_get_receive_time(InfAdoptedRequest* request)
+{
+  InfAdoptedRequestPrivate* priv;
+
+  g_return_val_if_fail(INF_ADOPTED_IS_REQUEST(request), 0);
+
+  priv = INF_ADOPTED_REQUEST_PRIVATE(request);
+  return priv->received;
+}
+
+/**
+ * inf_adopted_request_get_execute_time:
+ * @request: A #InfAdoptedRequest.
+ *
+ * Returns the time when the request was executed by an #InfAdoptedAlgorithm,
+ * see inf_adopted_algorithm_execute_request(). The time is
+ * given in microseconds since January 1, 1970. If the request was not yet
+ * executed, the function returns 0.
+ *
+ * Returns: The time when the function was executed, or 0.
+ */
+gint64
+inf_adopted_request_get_execute_time(InfAdoptedRequest* request)
+{
+  InfAdoptedRequestPrivate* priv;
+
+  g_return_val_if_fail(INF_ADOPTED_IS_REQUEST(request), 0);
+
+  priv = INF_ADOPTED_REQUEST_PRIVATE(request);
+  return priv->executed;
+}
+
+/**
+ * inf_adopted_request_set_execute_time:
+ * @request: A #InfAdoptedRequest.
+ * @time: A time in microseconds since January 1, 1970.
+ *
+ * Sets the time when @request was executed. Usually this is called by
+ * #InfAdoptedAlgorithm when it executes a request, i.e. translates it to the
+ * current state of the document.
+ */
+void
+inf_adopted_request_set_execute_time(InfAdoptedRequest* request,
+                                     gint64 time)
+{
+  InfAdoptedRequestPrivate* priv;
+
+  g_return_if_fail(INF_ADOPTED_IS_REQUEST(request));
+
+  priv = INF_ADOPTED_REQUEST_PRIVATE(request);
+
+  if(priv->executed != time)
+  {
+    priv->executed = time;
+    g_object_notify(G_OBJECT(request), "executed");
+  }
+}
+
 /**
  * inf_adopted_request_need_concurrency_id:
  * @request: The request to transform.
  * @against: The request to transform against.
  *
  * Returns whether transforming @request against @against requires a
- * concurrency ID. You can still call inf_adopted_request_transform() with
- * a concurrency ID of %INF_ADOPTED_CONCURRENCY_NONE even if this function
- * returns %TRUE if you don't have another possibility to find a
- * concurrency ID in which case user IDs are used to determine which request
- * to transform.
+ * concurrency ID. If this function returns %TRUE, you must provide the
+ * @request_lcs and @against_lcs parameters when calling
+ * inf_adopted_request_transform().
  *
  * Both request need to be of type %INF_ADOPTED_REQUEST_DO, and their state
  * vectors must be the same.
@@ -593,116 +672,89 @@ inf_adopted_request_need_concurrency_id(InfAdoptedRequest* request,
   );
 }
 
-/**
- * inf_adopted_request_get_concurrency_id:
- * @request: The request to transform.
- * @against: The request to transform against.
- *
- * Returns a concurrency ID for transformation of @operation against @against.
- * It always returns %INF_ADOPTED_CONCURRENCY_NONE when
- * inf_adopted_request_need_concurrency_id() returns %TRUE (but that's not
- * necessarily true the other way around), since it is not possible to decide
- * which operation to transform without any additional information.
- *
- * However, the function can be called on the same requests in a previous
- * state. In some cases, a decision can be made based on these previous
- * requests. This can be used as a concurrency ID for a call to
- * inf_adopted_request_transform(). If this does not yield a decision, it is
- * still possible to call inf_adopted_request_transform() with
- * %INF_ADOPTED_CONCURRENCY_NONE as concurrency ID in which case an arbitrary
- * request will be transformed, based on the user IDs of the requests.
- *
- * Both requests must be of type %INF_ADOPTED_REQUEST_DO, and their state
- * vectors must be the same.
- *
- * Returns: A concurrency ID between @operation and @against. Can be
- * %INF_ADOPTED_CONCURRENCY_NONE in case no decision can be made.
- */
-InfAdoptedConcurrencyId
-inf_adopted_request_get_concurrency_id(InfAdoptedRequest* request,
-                                       InfAdoptedRequest* against)
-{
-  InfAdoptedRequestPrivate* request_priv;
-  InfAdoptedRequestPrivate* against_priv;
-  
-  g_return_val_if_fail(
-    INF_ADOPTED_IS_REQUEST(request),
-    INF_ADOPTED_CONCURRENCY_NONE
-  );
-  g_return_val_if_fail(
-    INF_ADOPTED_IS_REQUEST(against),
-    INF_ADOPTED_CONCURRENCY_NONE
-  );
-  
-  request_priv = INF_ADOPTED_REQUEST_PRIVATE(request);
-  against_priv = INF_ADOPTED_REQUEST_PRIVATE(against);
-  
-  g_return_val_if_fail(
-    request_priv->type == INF_ADOPTED_REQUEST_DO,
-    INF_ADOPTED_CONCURRENCY_NONE
-  );
-  g_return_val_if_fail(
-    against_priv->type == INF_ADOPTED_REQUEST_DO,
-    INF_ADOPTED_CONCURRENCY_NONE
-  );
-  g_return_val_if_fail(
-    request_priv->user_id != against_priv->user_id,
-    INF_ADOPTED_CONCURRENCY_NONE
-  );
-  
-  g_return_val_if_fail(
-    inf_adopted_state_vector_compare(
-      request_priv->vector,
-      against_priv->vector
-    ) == 0,
-    INF_ADOPTED_CONCURRENCY_NONE
-  );
-
-  return inf_adopted_operation_get_concurrency_id(
-    request_priv->operation,
-    against_priv->operation
-  );
-}
-
 /**
  * inf_adopted_request_transform:
  * @request: The request to transform.
  * @against: The request to transform against.
- * @concurrency_id: A concurrency ID for the transformation.
+ * @request_lcs: The request to transform in a previous state, or %NULL.
+ * @against_lcs: The request to transform against in a previous state, or
+ * %NULL.
  *
  * Transforms the operation of @request against the operation of @against.
  * Both requests must be of type %INF_ADOPTED_REQUEST_DO, and their state
  * vectors must be the same.
  *
- * @concurrency_id can be %INF_ADOPTED_CONCURRENCY_NONE even if the
- * transformation requires a concurrency ID (see
- * inf_adopted_request_need_concurrency_id()). In that case, it is assumed
- * that it does not matter which operation to transform, and user IDs are
- * used to determine a concurrency ID for the transformation.
+ * If the function inf_adopted_request_need_concurrency_id() returns %TRUE,
+ * @request_lcs and @against_lcs must not be %NULL.
  *
- * Returns: A new #InfAdoptedRequest, the result of the transformation.
+ * Returns: (transfer full): A new #InfAdoptedRequest, the result of the
+ * transformation.
  **/
 InfAdoptedRequest*
 inf_adopted_request_transform(InfAdoptedRequest* request,
                               InfAdoptedRequest* against,
-                              InfAdoptedConcurrencyId concurrency_id)
+                              InfAdoptedRequest* request_lcs,
+                              InfAdoptedRequest* against_lcs)
 {
   InfAdoptedRequestPrivate* request_priv;
   InfAdoptedRequestPrivate* against_priv;
+  InfAdoptedRequestPrivate* request_lcs_priv;
+  InfAdoptedRequestPrivate* against_lcs_priv;
+  InfAdoptedRequestPrivate* new_priv;
   InfAdoptedOperation* new_operation;
   InfAdoptedStateVector* new_vector;
   InfAdoptedRequest* new_request;
-  
+
   g_return_val_if_fail(INF_ADOPTED_IS_REQUEST(request), NULL);
   g_return_val_if_fail(INF_ADOPTED_IS_REQUEST(against), NULL);
-  
+
   request_priv = INF_ADOPTED_REQUEST_PRIVATE(request);
   against_priv = INF_ADOPTED_REQUEST_PRIVATE(against);
   
+  if(request_lcs != NULL)
+    request_lcs_priv = INF_ADOPTED_REQUEST_PRIVATE(request_lcs);
+  if(against_lcs != NULL)
+    against_lcs_priv = INF_ADOPTED_REQUEST_PRIVATE(against_lcs);
+
   g_return_val_if_fail(request_priv->type == INF_ADOPTED_REQUEST_DO, NULL);
   g_return_val_if_fail(against_priv->type == INF_ADOPTED_REQUEST_DO, NULL);
+  g_return_val_if_fail(
+    request_lcs == NULL || request_lcs_priv->type == INF_ADOPTED_REQUEST_DO,
+    NULL
+  );
+  g_return_val_if_fail(
+    against_lcs == NULL || against_lcs_priv->type == INF_ADOPTED_REQUEST_DO,
+    NULL
+  );
+
   g_return_val_if_fail(request_priv->user_id != against_priv->user_id, NULL);
-  
+  g_return_val_if_fail(
+    request_lcs == NULL || request_priv->user_id == request_lcs_priv->user_id,
+    NULL
+  );
+  g_return_val_if_fail(
+    against_lcs == NULL || against_priv->user_id == against_lcs_priv->user_id,
+    NULL
+  );
+
+  g_return_val_if_fail(
+    request_lcs == NULL ||
+    inf_adopted_state_vector_causally_before(
+      request_lcs_priv->vector,
+      request_priv->vector
+    ),
+    NULL
+  );
+
+  g_return_val_if_fail(
+    against_lcs == NULL ||
+    inf_adopted_state_vector_causally_before(
+      against_lcs_priv->vector,
+      against_priv->vector
+    ),
+    NULL
+  );
+
   g_return_val_if_fail(
     inf_adopted_state_vector_compare(
       request_priv->vector,
@@ -710,19 +762,21 @@ inf_adopted_request_transform(InfAdoptedRequest* request,
     ) == 0, NULL
   );
 
-  if(concurrency_id != INF_ADOPTED_CONCURRENCY_NONE)
-  {
-    new_operation = inf_adopted_operation_transform(
+  g_return_val_if_fail(
+    inf_adopted_operation_need_concurrency_id(
       request_priv->operation,
-      against_priv->operation,
-      concurrency_id
-    );
-  }
-  else if(request_priv->user_id > against_priv->user_id)
+      against_priv->operation
+    ) == FALSE || (request_lcs != NULL && against_lcs != NULL),
+    NULL
+  );
+
+  if(request_priv->user_id > against_priv->user_id)
   {
     new_operation = inf_adopted_operation_transform(
       request_priv->operation,
       against_priv->operation,
+      request_lcs == NULL ? NULL : request_lcs_priv->operation,
+      against_lcs == NULL ? NULL : against_lcs_priv->operation,
       INF_ADOPTED_CONCURRENCY_OTHER
     );
   }
@@ -731,6 +785,8 @@ inf_adopted_request_transform(InfAdoptedRequest* request,
     new_operation = inf_adopted_operation_transform(
       request_priv->operation,
       against_priv->operation,
+      request_lcs == NULL ? NULL : request_lcs_priv->operation,
+      against_lcs == NULL ? NULL : against_lcs_priv->operation,
       INF_ADOPTED_CONCURRENCY_SELF
     );
   }
@@ -741,9 +797,13 @@ inf_adopted_request_transform(InfAdoptedRequest* request,
   new_request = inf_adopted_request_new_do(
     new_vector,
     request_priv->user_id,
-    new_operation
+    new_operation,
+    request_priv->received
   );
 
+  new_priv = INF_ADOPTED_REQUEST_PRIVATE(new_request);
+  new_priv->executed = request_priv->executed;
+
   g_object_unref(new_operation);
   inf_adopted_state_vector_free(new_vector);
   return new_request;
@@ -765,13 +825,14 @@ inf_adopted_request_transform(InfAdoptedRequest* request,
  * @request must be of type %INF_ADOPTED_REQUEST_DO and its operation must
  * be reversible.
  *
- * Returns: The mirrored request as a new #InfAdoptedRequest.
+ * Returns: (transfer full): The mirrored request as a new #InfAdoptedRequest.
  **/
 InfAdoptedRequest*
 inf_adopted_request_mirror(InfAdoptedRequest* request,
                            guint by)
 {
   InfAdoptedRequestPrivate* priv;
+  InfAdoptedRequestPrivate* new_priv;
   InfAdoptedOperation* new_operation;
   InfAdoptedStateVector* new_vector;
   InfAdoptedRequest* new_request;
@@ -793,9 +854,13 @@ inf_adopted_request_mirror(InfAdoptedRequest* request,
   new_request = inf_adopted_request_new_do(
     new_vector,
     priv->user_id,
-    new_operation
+    new_operation,
+    priv->received
   );
 
+  new_priv = INF_ADOPTED_REQUEST_PRIVATE(new_request);
+  new_priv->executed = priv->executed;
+
   g_object_unref(new_operation);
   inf_adopted_state_vector_free(new_vector);
   return new_request;
@@ -816,7 +881,7 @@ inf_adopted_request_mirror(InfAdoptedRequest* request,
  *
  * @into must not be the same user as the one that issued @request.
  *
- * Returns: The folded request as a new #InfAdoptedRequest.
+ * Returns: (transfer full): The folded request as a new #InfAdoptedRequest.
  **/
 InfAdoptedRequest*
 inf_adopted_request_fold(InfAdoptedRequest* request,
@@ -824,6 +889,7 @@ inf_adopted_request_fold(InfAdoptedRequest* request,
                          guint by)
 {
   InfAdoptedRequestPrivate* priv;
+  InfAdoptedRequestPrivate* new_priv;
   InfAdoptedStateVector* new_vector;
   InfAdoptedRequest* new_request;
 
@@ -846,6 +912,7 @@ inf_adopted_request_fold(InfAdoptedRequest* request,
         "operation", priv->operation,
         "vector", new_vector,
         "user-id", priv->user_id,
+        "received", priv->received,
         NULL
       )
     );
@@ -858,11 +925,15 @@ inf_adopted_request_fold(InfAdoptedRequest* request,
         "type", priv->type,
         "vector", new_vector,
         "user-id", priv->user_id,
+        "received", priv->received,
         NULL
       )
     );
   }
 
+  new_priv = INF_ADOPTED_REQUEST_PRIVATE(new_request);
+  new_priv->executed = priv->executed;
+
   inf_adopted_state_vector_free(new_vector);
   return new_request;
 }
