@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,16 +17,6 @@
  * MA 02110-1301, USA.
  */
 
-#include <libinfinity/adopted/inf-adopted-session-replay.h>
-#include <libinfinity/common/inf-simulated-connection.h>
-#include <libinfinity/common/inf-standalone-io.h>
-#include <libinfinity/common/inf-xml-util.h>
-#include <libinfinity/inf-i18n.h>
-
-#include <libxml/xmlreader.h>
-
-#include <string.h>
-
 /**
  * SECTION:inf-adopted-session-replay
  * @title: InfAdoptedSessionReplay
@@ -45,6 +35,16 @@
  * the replayed session.
  */
 
+#include <libinfinity/adopted/inf-adopted-session-replay.h>
+#include <libinfinity/common/inf-simulated-connection.h>
+#include <libinfinity/common/inf-standalone-io.h>
+#include <libinfinity/common/inf-xml-util.h>
+#include <libinfinity/inf-i18n.h>
+
+#include <libxml/xmlreader.h>
+
+#include <string.h>
+
 /* cf.
  * http://www.gnu.org/software/dotgnu/pnetlib-doc/System/Xml/XmlNodeType.html
  */
@@ -79,9 +79,11 @@ enum {
 
 #define INF_ADOPTED_SESSION_REPLAY_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_ADOPTED_TYPE_SESSION_REPLAY, InfAdoptedSessionReplayPrivate))
 
-static GObjectClass* parent_class;
 static GQuark session_replay_error_quark;
 
+G_DEFINE_TYPE_WITH_CODE(InfAdoptedSessionReplay, inf_adopted_session_replay, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfAdoptedSessionReplay))
+
 static xmlNodePtr
 inf_adopted_session_replay_read_current(xmlTextReaderPtr reader,
                                         GError** error)
@@ -94,11 +96,10 @@ inf_adopted_session_replay_read_current(xmlTextReaderPtr reader,
   {
     xml_error = xmlGetLastError();
 
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_XML,
-      "%s",
       xml_error->message
     );
 
@@ -119,11 +120,10 @@ inf_adopted_session_replay_handle_advance_result(int result,
   case -1:
     xml_error = xmlGetLastError();
 
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_XML,
-      "%s",
       xml_error->message
     );
 
@@ -148,21 +148,19 @@ inf_adopted_session_replay_handle_advance_required_result(int result,
   case -1:
     xml_error = xmlGetLastError();
 
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_XML,
-      "%s",
       xml_error->message
     );
 
     return FALSE;
   case 0:
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_UNEXPECTED_EOF,
-      "%s",
       _("Unexpected end of recording")
     );
 
@@ -351,11 +349,10 @@ inf_adopted_session_replay_play_initial(InfAdoptedSessionReplay* replay,
   name = xmlTextReaderConstName(reader);
   if(strcmp((const char*)name, "infinote-adopted-session-record") != 0)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_DOCUMENT,
-      "%s",
       _("Document is not a session recording")
     );
 
@@ -367,11 +364,10 @@ inf_adopted_session_replay_play_initial(InfAdoptedSessionReplay* replay,
   {
     xmlFree(value);
 
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_SESSION_TYPE,
-      "%s",
       _("Session type of the recording does not match")
     );
 
@@ -388,11 +384,10 @@ inf_adopted_session_replay_play_initial(InfAdoptedSessionReplay* replay,
   name = xmlTextReaderConstName(reader);
   if(strcmp((const char*)name, "initial") != 0)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_FORMAT,
-      "%s",
       _("Initial session state missing in recording")
     );
 
@@ -463,11 +458,10 @@ inf_adopted_session_replay_play_initial(InfAdoptedSessionReplay* replay,
     case INF_SESSION_RUNNING:
       g_signal_handler_disconnect(priv->session, handler);
 
-      g_set_error(
+      g_set_error_literal(
         error,
         session_replay_error_quark,
         INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_FORMAT,
-        "%s",
         _("Session switched to running without having finished playing "
           "the initial")
       );
@@ -484,11 +478,10 @@ inf_adopted_session_replay_play_initial(InfAdoptedSessionReplay* replay,
 
   if(xmlTextReaderNodeType(reader) != XML_READER_TYPE_END_ELEMENT)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_FORMAT,
-      "%s",
       _("Superfluous XML in initial session section")
     );
 
@@ -498,11 +491,10 @@ inf_adopted_session_replay_play_initial(InfAdoptedSessionReplay* replay,
   if(inf_session_get_status(INF_SESSION(priv->session)) ==
      INF_SESSION_SYNCHRONIZING)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_FORMAT,
-      "%s",
       _("Session is still in synchronizing state after having "
         "played the initial")
     );
@@ -526,14 +518,10 @@ inf_adopted_session_replay_play_initial(InfAdoptedSessionReplay* replay,
  */
 
 static void
-inf_adopted_session_replay_init(GTypeInstance* instance,
-                                gpointer g_class)
+inf_adopted_session_replay_init(InfAdoptedSessionReplay* replay)
 {
-  InfAdoptedSessionReplay* record;
   InfAdoptedSessionReplayPrivate* priv;
-
-  record = INF_ADOPTED_SESSION_REPLAY(instance);
-  priv = INF_ADOPTED_SESSION_REPLAY_PRIVATE(record);
+  priv = INF_ADOPTED_SESSION_REPLAY_PRIVATE(replay);
 
   priv->filename = NULL;
   priv->reader = NULL;
@@ -561,7 +549,7 @@ inf_adopted_session_replay_dispose(GObject* object)
 
   inf_adopted_session_replay_clear(replay);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_adopted_session_replay_parent_class)->dispose(object);
 }
 
 static void
@@ -575,7 +563,7 @@ inf_adopted_session_replay_finalize(GObject* object)
 
   g_assert(priv->filename == NULL);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_adopted_session_replay_parent_class)->finalize(object);
 }
 
 static void
@@ -632,14 +620,11 @@ inf_adopted_session_replay_get_property(GObject* object,
  */
 
 static void
-inf_adopted_session_replay_class_init(gpointer g_class,
-                                      gpointer class_data)
+inf_adopted_session_replay_class_init(
+  InfAdoptedSessionReplayClass* replay_class)
 {
   GObjectClass* object_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfAdoptedSessionReplayPrivate));
+  object_class = G_OBJECT_CLASS(replay_class);
 
   object_class->dispose = inf_adopted_session_replay_dispose;
   object_class->finalize = inf_adopted_session_replay_finalize;
@@ -674,51 +659,20 @@ inf_adopted_session_replay_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_adopted_session_replay_get_type(void)
-{
-  static GType session_replay_type = 0;
-
-  if(!session_replay_type)
-  {
-    static const GTypeInfo session_replay_type_info = {
-      sizeof(InfAdoptedSessionReplayClass),   /* class_size */
-      NULL,                                   /* base_init */
-      NULL,                                   /* base_finalize */
-      inf_adopted_session_replay_class_init,  /* class_init */
-      NULL,                                   /* class_finalize */
-      NULL,                                   /* class_data */
-      sizeof(InfAdoptedSessionReplay),        /* instance_size */
-      0,                                      /* n_preallocs */
-      inf_adopted_session_replay_init,        /* instance_init */
-      NULL                                    /* value_table */
-    };
-
-    session_replay_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfAdoptedSessionReplay",
-      &session_replay_type_info,
-      0
-    );
-  }
-
-  return session_replay_type;
-}
-
 /*
  * Public API.
  */
 
 /**
- * inf_adopted_session_replay_new:
+ * inf_adopted_session_replay_new: (constructor)
  *
  * Creates a new #InfAdoptedSessionReplay. Use
  * inf_adopted_session_replay_set_record() to start the recording, and
  * inf_adopted_session_replay_play_next() or
  * inf_adopted_session_replay_play_to_end() to play it.
  *
- * Return Value: A new #InfAdoptedSessionReplay. Free with g_object_unref()
- * when no longer in use.
+ * Returns: (transfer full): A new #InfAdoptedSessionReplay. Free with
+ * g_object_unref() when no longer in use.
  **/
 InfAdoptedSessionReplay*
 inf_adopted_session_replay_new(void)
@@ -731,7 +685,7 @@ inf_adopted_session_replay_new(void)
 /**
  * inf_adopted_session_replay_set_record:
  * @replay: A #InfAdoptedSessionReplay.
- * @filename: Path to the record file to play.
+ * @filename: (type filename): Path to the record file to play.
  * @plugin: A #InfcNotePlugin for the note type of the recorded session.
  * @error: Location to store error information, if any.
  *
@@ -770,11 +724,10 @@ inf_adopted_session_replay_set_record(InfAdoptedSessionReplay* replay,
   {
     xml_error = xmlGetLastError();
 
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_FILE,
-      "%s",
       xml_error->message
     );
 
@@ -831,8 +784,9 @@ inf_adopted_session_replay_set_record(InfAdoptedSessionReplay* replay,
       io,
       priv->client_manager,
       INF_SESSION_SYNCHRONIZING,
-      priv->client_group,
+      INF_COMMUNICATION_GROUP(priv->client_group),
       INF_XML_CONNECTION(priv->client_conn),
+      NULL,
       plugin->user_data
     )
   );
@@ -871,7 +825,7 @@ inf_adopted_session_replay_set_record(InfAdoptedSessionReplay* replay,
  * Returns the played back session, or %NULL if
  * inf_adopted_session_replay_set_record() was not yet called.
  *
- * Returns: A #InfAdoptedSessionReplay, or %NULL.
+ * Returns: (transfer none): A #InfAdoptedSessionReplay, or %NULL.
  */
 InfAdoptedSession*
 inf_adopted_session_replay_get_session(InfAdoptedSessionReplay* replay)
@@ -914,6 +868,7 @@ inf_adopted_session_replay_play_next(InfAdoptedSessionReplay* replay,
   InfSessionClass* session_class;
   GArray* user_props;
   GParameter* param;
+  gboolean result;
   guint i;
 
   g_return_val_if_fail(INF_ADOPTED_IS_SESSION_REPLAY(replay), FALSE);
@@ -930,11 +885,10 @@ inf_adopted_session_replay_play_next(InfAdoptedSessionReplay* replay,
 
   if(type != XML_READER_TYPE_ELEMENT)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       session_replay_error_quark,
       INF_ADOPTED_SESSION_REPLAY_ERROR_BAD_FORMAT,
-      "%s",
       _("Superfluous XML in requests section")
     );
 
@@ -1006,13 +960,23 @@ inf_adopted_session_replay_play_next(InfAdoptedSessionReplay* replay,
       g_value_set_object(&param->value, G_OBJECT(priv->client_conn));
     }
 
-    user = inf_session_add_user(
+    result = session_class->validate_user_props(
       INF_SESSION(priv->session),
       (const GParameter*)user_props->data,
       user_props->len,
+      NULL,
       error
     );
 
+    if(result == TRUE)
+    {
+      user = inf_session_add_user(
+        INF_SESSION(priv->session),
+        (const GParameter*)user_props->data,
+        user_props->len
+      );
+    }
+
     for(i = 0; i < user_props->len; ++i)
       g_value_unset(&g_array_index(user_props, GParameter, i).value);
     g_array_free(user_props, TRUE);
