@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -46,6 +46,7 @@
 #include <libinfinity/common/inf-acl.h>
 #include <libinfinity/common/inf-xml-util.h>
 #include <libinfinity/common/inf-error.h>
+#include <libinfinity/inf-define-enum.h>
 #include <libinfinity/inf-i18n.h>
 
 #include <string.h>
@@ -109,153 +110,75 @@ const InfAclMask INF_ACL_MASK_LEAF = {
   MAKE_FULL_MASK(MAKE_LEAF_MASK)
 };
 
-GType
-inf_acl_account_get_type(void)
-{
-  static GType acl_account_type = 0;
-
-  if(!acl_account_type)
+static const GEnumValue inf_acl_setting_values[] = {
   {
-    acl_account_type = g_boxed_type_register_static(
-      "InfAclAccount",
-      (GBoxedCopyFunc)inf_acl_account_copy,
-      (GBoxedFreeFunc)inf_acl_account_free
-    );
+    INF_ACL_CAN_ADD_SUBDIRECTORY,
+    "INF_ACL_CAN_ADD_SUBDIRECTORY",
+    "can-add-subdirectory"
+  }, {
+    INF_ACL_CAN_ADD_DOCUMENT,
+    "INF_ACL_CAN_ADD_DOCUMENT",
+    "can-add-document"
+  }, {
+    INF_ACL_CAN_SYNC_IN,
+    "INF_ACL_CAN_SYNC_IN",
+    "can-sync-in"
+  }, {
+    INF_ACL_CAN_REMOVE_NODE,
+    "INF_ACL_CAN_REMOVE_NODE",
+    "can-remove-node"
+  }, {
+    INF_ACL_CAN_EXPLORE_NODE,
+    "INF_ACL_CAN_EXPLORE_NODE",
+    "can-explore-node"
+  }, {
+    INF_ACL_CAN_SUBSCRIBE_CHAT,
+    "INF_ACL_CAN_SUBSCRIBE_CHAT",
+    "can-subscribe-chat"
+  }, {
+    INF_ACL_CAN_SUBSCRIBE_SESSION,
+    "INF_ACL_CAN_SUBSCRIBE_SESSION",
+    "can-subscribe-session"
+  }, {
+    INF_ACL_CAN_JOIN_USER,
+    "INF_ACL_CAN_JOIN_USER",
+    "can-join-user"
+  }, {
+    INF_ACL_CAN_QUERY_ACCOUNT_LIST,
+    "INF_ACL_CAN_QUERY_ACCOUNT_LIST",
+    "can-query-account-list"
+  }, {
+    INF_ACL_CAN_CREATE_ACCOUNT,
+    "INF_ACL_CAN_CREATE_ACCOUNT",
+    "can-create-account"
+  }, {
+    INF_ACL_CAN_OVERRIDE_ACCOUNT,
+    "INF_ACL_CAN_OVERRIDE_ACCOUNT",
+    "can-override-account"
+  }, {
+    INF_ACL_CAN_REMOVE_ACCOUNT,
+    "INF_ACL_CAN_REMOVE_ACCOUNT",
+    "can-remove-account"
+  }, {
+    INF_ACL_CAN_QUERY_ACL,
+    "INF_ACL_CAN_QUERY_ACL",
+    "can-query-acl"
+  }, {
+    INF_ACL_CAN_SET_ACL,
+    "INF_ACL_CAN_SET_ACL",
+    "can-set-acl"
+  }, {
+    0,
+    NULL,
+    NULL
   }
+};
 
-  return acl_account_type;
-}
-
-GType
-inf_acl_setting_get_type(void)
-{
-  static GType acl_setting_type = 0;
-
-  if(!acl_setting_type)
-  {
-    static const GEnumValue acl_setting_values[] = {
-      {
-        INF_ACL_CAN_ADD_SUBDIRECTORY,
-        "INF_ACL_CAN_ADD_SUBDIRECTORY",
-        "can-add-subdirectory"
-      }, {
-        INF_ACL_CAN_ADD_DOCUMENT,
-        "INF_ACL_CAN_ADD_DOCUMENT",
-        "can-add-document"
-      }, {
-        INF_ACL_CAN_SYNC_IN,
-        "INF_ACL_CAN_SYNC_IN",
-        "can-sync-in"
-      }, {
-        INF_ACL_CAN_REMOVE_NODE,
-        "INF_ACL_CAN_REMOVE_NODE",
-        "can-remove-node"
-      }, {
-        INF_ACL_CAN_EXPLORE_NODE,
-        "INF_ACL_CAN_EXPLORE_NODE",
-        "can-explore-node"
-      }, {
-        INF_ACL_CAN_SUBSCRIBE_CHAT,
-        "INF_ACL_CAN_SUBSCRIBE_CHAT",
-        "can-subscribe-chat"
-      }, {
-        INF_ACL_CAN_SUBSCRIBE_SESSION,
-        "INF_ACL_CAN_SUBSCRIBE_SESSION",
-        "can-subscribe-session"
-      }, {
-        INF_ACL_CAN_JOIN_USER,
-        "INF_ACL_CAN_JOIN_USER",
-        "can-join-user"
-      }, {
-        INF_ACL_CAN_QUERY_ACCOUNT_LIST,
-        "INF_ACL_CAN_QUERY_ACCOUNT_LIST",
-        "can-query-account-list"
-      }, {
-        INF_ACL_CAN_CREATE_ACCOUNT,
-        "INF_ACL_CAN_CREATE_ACCOUNT",
-        "can-create-account"
-      }, {
-        INF_ACL_CAN_OVERRIDE_ACCOUNT,
-        "INF_ACL_CAN_OVERRIDE_ACCOUNT",
-        "can-override-account"
-      }, {
-        INF_ACL_CAN_REMOVE_ACCOUNT,
-        "INF_ACL_CAN_REMOVE_ACCOUNT",
-        "can-remove-account"
-      }, {
-        INF_ACL_CAN_QUERY_ACL,
-        "INF_ACL_CAN_QUERY_ACL",
-        "can-query-acl"
-      }, {
-        INF_ACL_CAN_SET_ACL,
-        "INF_ACL_CAN_SET_ACL",
-        "can-set-acl"
-      }, {
-        0,
-        NULL,
-        NULL
-      }
-    };
-
-    acl_setting_type = g_enum_register_static(
-      "InfAclSetting",
-      acl_setting_values
-    );
-  }
-
-  return acl_setting_type;
-}
-
-GType
-inf_acl_mask_get_type(void)
-{
-  static GType acl_mask_type = 0;
-
-  if(!acl_mask_type)
-  {
-    acl_mask_type = g_boxed_type_register_static(
-      "InfAclMask",
-      (GBoxedCopyFunc)inf_acl_mask_copy,
-      (GBoxedFreeFunc)inf_acl_mask_free
-    );
-  }
-
-  return acl_mask_type;
-}
-
-GType
-inf_acl_sheet_get_type(void)
-{
-  static GType acl_sheet_type = 0;
-
-  if(!acl_sheet_type)
-  {
-    acl_sheet_type = g_boxed_type_register_static(
-      "InfAclSheet",
-      (GBoxedCopyFunc)inf_acl_sheet_copy,
-      (GBoxedFreeFunc)inf_acl_sheet_free
-    );
-  }
-
-  return acl_sheet_type;
-}
-
-GType
-inf_acl_sheet_set_get_type(void)
-{
-  static GType acl_sheet_set_type = 0;
-
-  if(!acl_sheet_set_type)
-  {
-    acl_sheet_set_type = g_boxed_type_register_static(
-      "InfAclSheetSet",
-      (GBoxedCopyFunc)inf_acl_sheet_set_copy,
-      (GBoxedFreeFunc)inf_acl_sheet_set_free
-    );
-  }
-
-  return acl_sheet_set_type;
-}
+G_DEFINE_BOXED_TYPE(InfAclAccount, inf_acl_account, inf_acl_account_copy, inf_acl_account_free)
+INF_DEFINE_ENUM_TYPE(InfAclSetting, inf_acl_setting, inf_acl_setting_values)
+G_DEFINE_BOXED_TYPE(InfAclMask, inf_acl_mask, inf_acl_mask_copy, inf_acl_mask_free)
+G_DEFINE_BOXED_TYPE(InfAclSheet, inf_acl_sheet, inf_acl_sheet_copy, inf_acl_sheet_free)
+G_DEFINE_BOXED_TYPE(InfAclSheetSet, inf_acl_sheet_set, inf_acl_sheet_set_copy, inf_acl_sheet_set_free)
 
 /**
  * inf_acl_account_id_to_string:
@@ -279,7 +202,8 @@ inf_acl_account_id_to_string(InfAclAccountId account)
  * Converts the given string into a unique account identifier which can be
  * used with the rest of the ACL API.
  *
- * Returns: The account ID which is equivalent to the given string.
+ * Returns: (transfer full): The account ID which is equivalent to the given
+ * string.
  */
 InfAclAccountId
 inf_acl_account_id_from_string(const gchar* id)
@@ -290,11 +214,11 @@ inf_acl_account_id_from_string(const gchar* id)
 /**
  * inf_acl_account_new:
  * @id: The unique ID of the new account.
- * @name: The human-readable name of the new account.
+ * @name: (allow-none): The human-readable name of the new account.
  *
  * Creates a new #InfAclAccount.
  *
- * Returns: A new #InfAclAccount object.
+ * Returns: (transfer full): A new #InfAclAccount object.
  */
 InfAclAccount*
 inf_acl_account_new(const InfAclAccountId id,
@@ -316,8 +240,8 @@ inf_acl_account_new(const InfAclAccountId id,
  *
  * Creates a copy of @account.
  *
- * Returns: A new #InfAclAccount. Free with inf_acl_account_free() when no longer
- * needed.
+ * Returns: (transfer full): A new #InfAclAccount. Free with
+ * inf_acl_account_free() when no longer needed.
  */
 InfAclAccount*
 inf_acl_account_copy(const InfAclAccount* account)
@@ -347,7 +271,7 @@ inf_acl_account_free(InfAclAccount* account)
 
 /**
  * inf_acl_account_array_free:
- * @accounts: An array of #InfAclAccount objects.
+ * @accounts: (array length=n_accounts): An array of #InfAclAccount objects.
  * @n_accounts: The number of elements in the array.
  *
  * Releases all resources allocated by an array of #InfAclAccount<!-- -->s.
@@ -372,8 +296,8 @@ void inf_acl_account_array_free(InfAclAccount* accounts,
  * mandatory fields being missing, the function returns %NULL and @error is
  * set.
  *
- * Returns: A new #InfAclAccount on success, or %NULL on failure. Free with
- * inf_acl_account_free() when no longer needed.
+ * Returns: (transfer full): A new #InfAclAccount on success, or %NULL on
+ * failure. Free with inf_acl_account_free() when no longer needed.
  */
 InfAclAccount*
 inf_acl_account_from_xml(xmlNodePtr xml,
@@ -431,8 +355,8 @@ inf_acl_account_to_xml(const InfAclAccount* account,
  * by value. This function is mainly meant for the boxed type definition and
  * for language bindings.
  *
- * Returns: A new #InfAclMask. Free with inf_acl_mask_free() when no longer
- * needed.
+ * Returns: (transfer full): A new #InfAclMask. Free with inf_acl_mask_free()
+ * when no longer needed.
  */
 InfAclMask*
 inf_acl_mask_copy(const InfAclMask* mask)
@@ -527,7 +451,7 @@ inf_acl_mask_equal(const InfAclMask* lhs,
  * Initializes @mask such that all permissions are off except the one
  * corresponding to @setting.
  *
- * Returns: The mask itself.
+ * Returns: (transfer none): The mask itself.
  */
 InfAclMask*
 inf_acl_mask_set1(InfAclMask* mask,
@@ -548,13 +472,14 @@ inf_acl_mask_set1(InfAclMask* mask,
 /**
  * inf_acl_mask_setv:
  * @mask: The #InfAclMask to initialize.
- * @settings: An array of #InfAclSetting<!-- -->s to set.
+ * @settings: (array length=n_settings): An array of
+ * #InfAclSetting<!-- -->s to set.
  * @n_settings: The number of settings.
  *
  * Initializes @mask such that all permissions are off except the ones
  * specified in the @settings array.
  *
- * Returns: The mask itself.
+ * Returns: (transfer none): The mask itself.
  */
 InfAclMask*
 inf_acl_mask_setv(InfAclMask* mask,
@@ -584,12 +509,12 @@ inf_acl_mask_setv(InfAclMask* mask,
  * inf_acl_mask_and:
  * @lhs: First mask.
  * @rhs: Second mask.
- * @out: Output mask.
+ * @out: (out): Output mask.
  *
  * Computes the bitwise AND of @lhs and @rhs and writes the result to @out.
  * @out is allowed to be equivalent to @lhs and/or @rhs.
  *
- * Returns: The output mask.
+ * Returns: (transfer none): The output mask.
  */
 InfAclMask*
 inf_acl_mask_and(const InfAclMask* lhs,
@@ -616,7 +541,7 @@ inf_acl_mask_and(const InfAclMask* lhs,
  * Disables the bit corresponding to setting in @mask, leaving all other bits
  * alone.
  *
- * Returns: The mask itself.
+ * Returns: (transfer none): The mask itself.
  */
 InfAclMask*
 inf_acl_mask_and1(InfAclMask* mask,
@@ -634,12 +559,12 @@ inf_acl_mask_and1(InfAclMask* mask,
  * inf_acl_mask_or:
  * @lhs: First mask.
  * @rhs: Second mask.
- * @out: Output mask.
+ * @out: (out): Output mask.
  *
  * Computes the bitwise OR of @lhs and @rhs and writes the result to @out.
  * @out is allowed to be equivalent to @lhs and/or @rhs.
  *
- * Returns: The output mask.
+ * Returns: (transfer none): The output mask.
  */
 InfAclMask*
 inf_acl_mask_or(const InfAclMask* lhs,
@@ -666,7 +591,7 @@ inf_acl_mask_or(const InfAclMask* lhs,
  * Enables the bit corresponding to setting in @mask, leaving all other bits
  * alone.
  *
- * Returns: The mask itself.
+ * Returns: (transfer none): The mask itself.
  */
 InfAclMask*
 inf_acl_mask_or1(InfAclMask* mask,
@@ -683,12 +608,12 @@ inf_acl_mask_or1(InfAclMask* mask,
 /**
  * inf_acl_mask_neg:
  * @mask: The mask to negate.
- * @out: The output mask.
+ * @out: (out): The output mask.
  *
  * Negates the given mask bitwise and writes the result to @out. The output
  * mask is allowed to be equivalent to @mask itself.
  *
- * Returns: The output mask.
+ * Returns: (transfer full): The output mask.
  */
 InfAclMask*
 inf_acl_mask_neg(const InfAclMask* mask,
@@ -734,8 +659,8 @@ inf_acl_mask_has(const InfAclMask* mask,
  * for properties and bindings. The new sheet will hold permissions for the
  * given account. The permissions will initially be all masked out.
  *
- * Returns: A new #InfAclSheet. Free with inf_acl_sheet_free() when no longer
- * in use.
+ * Returns: (transfer full): A new #InfAclSheet. Free with
+ * inf_acl_sheet_free() when no longer in use.
  */
 InfAclSheet*
 inf_acl_sheet_new(InfAclAccountId account)
@@ -756,7 +681,7 @@ inf_acl_sheet_new(InfAclAccountId account)
  * applications because you can copy the structs by value, but it is useful
  * for properties and bindings. 
  *
- * Returns: A newly-allocated copy of @sheet. Free with
+ * Returns: (transfer full): A newly-allocated copy of @sheet. Free with
  * inf_acl_sheet_free() when no longer in use.
  */
 InfAclSheet*
@@ -785,8 +710,8 @@ inf_acl_sheet_free(InfAclSheet* sheet)
 /**
  * inf_acl_sheet_perms_from_xml:
  * @xml: The XML node to read from.
- * @mask: Output parameter to write the permission mask to.
- * @perms: Output parameter to write the permissions to.
+ * @mask: (out): Output parameter to write the permission mask to.
+ * @perms: (out): Output parameter to write the permissions to.
  * @error: Location to store error information, if any.
  *
  * This function extracts the permission mask and the permission flags from
@@ -831,11 +756,10 @@ inf_acl_sheet_perms_from_xml(xmlNodePtr xml,
       }
       else
       {
-        g_set_error(
+        g_set_error_literal(
           error,
           inf_request_error_quark(),
           INF_REQUEST_ERROR_INVALID_ATTRIBUTE,
-          "%s",
           _("ACL field must be either \"yes\" or \"no\"")
         );
       }
@@ -892,7 +816,8 @@ inf_acl_sheet_perms_to_xml(const InfAclMask* mask,
  * Creates a new #InfAclSheetSet. Add sheets with
  * inf_acl_sheet_set_add_sheet().
  *
- * Returns: A new #InfAclSheetSet. Free with inf_acl_sheet_set_free().
+ * Returns: (transfer full): A new #InfAclSheetSet. Free with
+ * inf_acl_sheet_set_free().
  */
 InfAclSheetSet*
 inf_acl_sheet_set_new(void)
@@ -907,7 +832,7 @@ inf_acl_sheet_set_new(void)
 
 /**
  * inf_acl_sheet_set_new_external:
- * @sheets: An array of #InfAclSheet<!-- -->s
+ * @sheets: (array length=n_sheets): An array of #InfAclSheet<!-- -->s
  * @n_sheets: Number of elements in @sheets.
  *
  * Creates a new #InfAclSheetSet refererencing the given ACL sheets. The
@@ -918,8 +843,8 @@ inf_acl_sheet_set_new(void)
  * inf_acl_sheet_set_add_sheet(), or removed with
  * inf_acl_sheet_set_remove_sheet().
  *
- * Returns: A new #InfAclSheetSet. Free with inf_acl_sheet_set_free() when
- * no longer needed.
+ * Returns: (transfer full): A new #InfAclSheetSet. Free with
+ * inf_acl_sheet_set_free() when no longer needed.
  */
 InfAclSheetSet*
 inf_acl_sheet_set_new_external(const InfAclSheet* sheets,
@@ -944,8 +869,8 @@ inf_acl_sheet_set_new_external(const InfAclSheet* sheets,
  * inf_acl_sheet_set_new_external(), the copied sheet set will also only hold
  * a reference to the external sheets, and the same restrictions apply.
  *
- * Returns: A new #InfAclSheetSet. Free with inf_acl_sheet_set_free() when
- * no longer needed.
+ * Returns: (transfer full): A new #InfAclSheetSet. Free with
+ * inf_acl_sheet_set_free() when no longer needed.
  */
 InfAclSheetSet*
 inf_acl_sheet_set_copy(const InfAclSheetSet* sheet_set)
@@ -1034,7 +959,7 @@ inf_acl_sheet_set_free(InfAclSheetSet* sheet_set)
  * This function can only be used if the sheet set has not been created with
  * the inf_acl_sheet_set_new_external() function.
  *
- * Returns: A #InfAclSheet for the new account.
+ * Returns: (transfer none): A #InfAclSheet for the new account.
  */
 InfAclSheet*
 inf_acl_sheet_set_add_sheet(InfAclSheetSet* sheet_set,
@@ -1106,7 +1031,7 @@ inf_acl_sheet_set_remove_sheet(InfAclSheetSet* sheet_set,
 
 /**
  * inf_acl_sheet_set_merge_sheets:
- * @sheet_set: A #InfAclSheetSet, or %NULL.
+ * @sheet_set: (allow-none) (transfer full): A #InfAclSheetSet, or %NULL.
  * @other: The sheet set to merge.
  *
  * Replaces all sheets that are present in @other in @sheet_set with the ones
@@ -1118,8 +1043,8 @@ inf_acl_sheet_set_remove_sheet(InfAclSheetSet* sheet_set,
  * created and returned, unless @other is empty. If the merged sheet set
  * ends up empty, it is freed and the function returns %NULL.
  *
- * Returns: The merged sheet set, or %NULL when the merged sheet set would
- * be empty.
+ * Returns: (allow-none) (transfer full): The merged sheet set, or %NULL
+ * when the merged sheet set would be empty.
  */
 InfAclSheetSet*
 inf_acl_sheet_set_merge_sheets(InfAclSheetSet* sheet_set,
@@ -1178,8 +1103,8 @@ inf_acl_sheet_set_merge_sheets(InfAclSheetSet* sheet_set,
  * the merge, the returned sheet set can be modified. This allows to replace
  * the current permissions with new ones atomically.
  *
- * Returns: A new #InfAclSheetSet. Free with inf_acl_sheet_set_free() when
- * no longer needed.
+ * Returns: (transfer full): A new #InfAclSheetSet. Free with
+ * inf_acl_sheet_set_free() when no longer needed.
  */
 InfAclSheetSet*
 inf_acl_sheet_set_get_clear_sheets(const InfAclSheetSet* sheet_set)
@@ -1229,7 +1154,7 @@ inf_acl_sheet_set_get_clear_sheets(const InfAclSheetSet* sheet_set)
  * This function can only be used if the sheet set has not been created with
  * the inf_acl_sheet_set_new_external() function.
  *
- * Returns: A #InfAclSheet for @account, or %NULL.
+ * Returns: (transfer none): A #InfAclSheet for @account, or %NULL.
  */
 InfAclSheet*
 inf_acl_sheet_set_find_sheet(InfAclSheetSet* sheet_set,
@@ -1266,7 +1191,7 @@ inf_acl_sheet_set_find_sheet(InfAclSheetSet* sheet_set,
  * that cannot be modified, but it can also be used on a sheet set created
  * with the inf_acl_sheet_set_new_external() function.
  *
- * Returns: A #InfAclSheet for @account, or %NULL.
+ * Returns: (transfer none): A #InfAclSheet for @account, or %NULL.
  */
 const InfAclSheet*
 inf_acl_sheet_set_find_const_sheet(const InfAclSheetSet* sheet_set,
@@ -1294,8 +1219,8 @@ inf_acl_sheet_set_find_const_sheet(const InfAclSheetSet* sheet_set,
  * %NULL and @error is set. If there is no ACL stored in @xml, the function
  * returns %NULL without setting @error.
  *
- * Returns: A #InfAclSheetSet, or %NULL. Free with inf_acl_sheet_set_free()
- * when no longer needed.
+ * Returns: (transfer full): A #InfAclSheetSet, or %NULL. Free with
+ * inf_acl_sheet_set_free() when no longer needed.
  */
 InfAclSheetSet*
 inf_acl_sheet_set_from_xml(xmlNodePtr xml,
