@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,6 +33,8 @@
 
 #include <gnutls/x509.h>
 
+G_DEFINE_BOXED_TYPE(InfCertificateChain, inf_certificate_chain, inf_certificate_chain_ref, inf_certificate_chain_unref)
+
 struct _InfCertificateChain {
   guint ref_count;
 
@@ -40,33 +42,16 @@ struct _InfCertificateChain {
   guint n_certs;
 };
 
-GType
-inf_certificate_chain_get_type(void)
-{
-  static GType certificate_chain_type = 0;
-
-  if(!certificate_chain_type)
-  {
-    certificate_chain_type = g_boxed_type_register_static(
-      "InfCertificateChain",
-      (GBoxedCopyFunc)inf_certificate_chain_ref,
-      (GBoxedFreeFunc)inf_certificate_chain_unref
-    );
-  }
-
-  return certificate_chain_type;
-}
-
 /**
  * inf_certificate_chain_new:
- * @certs: Array of certificates.
+ * @certs: (array length=n_certs): Array of certificates.
  * @n_certs: Number of elements in @certs.
  *
  * Creates a new #InfCertificateChain with the given certificates. The @certs
  * array needs to be allocated with g_malloc. This function takes ownership
  * of @certs.
  *
- * Return Value: A new #InfCertificateChain.
+ * Return Value: (transfer full): A new #InfCertificateChain.
  **/
 InfCertificateChain*
 inf_certificate_chain_new(gnutls_x509_crt_t* certs,
