@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,6 +30,7 @@
  **/
 
 #include <libinfinity/common/inf-ip-address.h>
+#include <libinfinity/inf-define-enum.h>
 
 #ifdef G_OS_WIN32
 # include <winsock2.h>
@@ -44,6 +45,22 @@
 
 #include <string.h>
 
+static const GEnumValue inf_ip_address_family_values[] = {
+  {
+    INF_IP_ADDRESS_IPV4,
+    "INF_IP_ADDRESS_IPV4",
+    "IPv4"
+  }, {
+    INF_IP_ADDRESS_IPV6,
+    "INF_IP_ADDRESS_IPV6",
+    "IPv6"
+  }, {
+    0,
+    NULL,
+    NULL
+  }
+};
+
 struct _InfIpAddress {
   InfIpAddressFamily family;
 
@@ -53,6 +70,9 @@ struct _InfIpAddress {
   } shared;
 };
 
+INF_DEFINE_ENUM_TYPE(InfIpAddressFamily, inf_ip_address_family, inf_ip_address_family_values)
+G_DEFINE_BOXED_TYPE(InfIpAddress, inf_ip_address, inf_ip_address_copy, inf_ip_address_free)
+
 static InfIpAddress*
 inf_ip_address_new_common(InfIpAddressFamily family)
 {
@@ -62,62 +82,13 @@ inf_ip_address_new_common(InfIpAddressFamily family)
   return address;
 }
 
-GType
-inf_ip_address_family_get_type(void)
-{
-  static GType ip_address_family_type = 0;
-
-  if(!ip_address_family_type)
-  {
-    static const GEnumValue ip_address_family_values[] = {
-      {
-        INF_IP_ADDRESS_IPV4,
-        "INF_IP_ADDRESS_IPV4",
-        "IPv4"
-      }, {
-        INF_IP_ADDRESS_IPV6,
-        "INF_IP_ADDRESS_IPV6",
-        "IPv6"
-      }, {
-        0,
-        NULL,
-        NULL
-      }
-    };
-
-    ip_address_family_type = g_enum_register_static(
-      "InfIpAddressFamily",
-      ip_address_family_values
-    );
-  }
-
-  return ip_address_family_type;
-}
-
-GType
-inf_ip_address_get_type(void)
-{
-  static GType ip_address_type = 0;
-
-  if(!ip_address_type)
-  {
-    ip_address_type = g_boxed_type_register_static(
-      "InfIpAddress",
-      (GBoxedCopyFunc)inf_ip_address_copy,
-      (GBoxedFreeFunc)inf_ip_address_free
-    );
-  }
-
-  return ip_address_type;
-}
-
 /**
- * inf_ip_address_new_raw4:
+ * inf_ip_address_new_raw4: (constructor)
  * @address: An IPv4 address in network byte order.
  *
  * Creates a new IPv4 address.
  *
- * Return Value: A new #InfIpAddress.
+ * Returns: (transfer full): A new #InfIpAddress.
  **/
 InfIpAddress*
 inf_ip_address_new_raw4(guint32 address)
@@ -129,12 +100,12 @@ inf_ip_address_new_raw4(guint32 address)
 }
 
 /**
- * inf_ip_address_new_loopback4:
+ * inf_ip_address_new_loopback4: (constructor)
  *
  * Creates a new IPv4 address that contains the local host's IP address
  * ("127.0.0.1").
  *
- * Return Value: A new #InfIpAddress.
+ * Returns: (transfer full): A new #InfIpAddress.
  **/
 InfIpAddress*
 inf_ip_address_new_loopback4(void)
@@ -143,12 +114,12 @@ inf_ip_address_new_loopback4(void)
 }
 
 /**
- * inf_ip_address_new_raw6:
+ * inf_ip_address_new_raw6: (constructor)
  * @address: An IPv6 address in network bype order.
  *
  * Creates a new IPv6 address.
  *
- * Return Value: A new #InfIpAddress.
+ * Returns: (transfer full): A new #InfIpAddress.
  **/
 InfIpAddress*
 inf_ip_address_new_raw6(const guint8 address[16])
@@ -160,12 +131,12 @@ inf_ip_address_new_raw6(const guint8 address[16])
 }
 
 /**
- * inf_ip_address_new_loopback6:
+ * inf_ip_address_new_loopback6: (constructor)
  *
  * Creates a new IPv6 address that contains the local host's IP address
  * ("::1").
  *
- * Return Value: A new #InfIpAddress.
+ * Returns: (transfer full): A new #InfIpAddress.
  **/
 InfIpAddress*
 inf_ip_address_new_loopback6(void)
@@ -174,13 +145,13 @@ inf_ip_address_new_loopback6(void)
 }
 
 /**
- * inf_ip_address_new_from_string:
+ * inf_ip_address_new_from_string: (constructor)
  * @str: A string containing an IPv4 or IPv6 address in standard dots
  * notation.
  *
  * Creates a new IP address (either IPv4 or IPv6) from the given string.
  *
- * Return Value: A new #InfIpAddress, or %NULL.
+ * Returns: (transfer full): A new #InfIpAddress, or %NULL.
  **/
 InfIpAddress*
 inf_ip_address_new_from_string(const gchar* str)
@@ -263,7 +234,7 @@ inf_ip_address_new_from_string(const gchar* str)
  *
  * Creates a new #InfIpAddress that contains the same address as @address.
  *
- * Return Value: A new #InfIpAddress.
+ * Returns: A new #InfIpAddress.
  **/
 InfIpAddress*
 inf_ip_address_copy(const InfIpAddress* address)
@@ -298,7 +269,7 @@ inf_ip_address_free(InfIpAddress* address)
  *
  * Returns the address family of @address.
  *
- * Return Value: A #InfIpAddressFamily.
+ * Returns: A #InfIpAddressFamily.
  **/
 InfIpAddressFamily
 inf_ip_address_get_family(const InfIpAddress* address)
@@ -313,7 +284,7 @@ inf_ip_address_get_family(const InfIpAddress* address)
  * Returns either 32 bit (IPv4) or 128 bit (IPv6) raw address data in host
  * byte order of @address.
  *
- * Return Value: The raw address.
+ * Returns: (transfer none): The raw address.
  **/
 gconstpointer
 inf_ip_address_get_raw(const InfIpAddress* address)
@@ -337,7 +308,7 @@ inf_ip_address_get_raw(const InfIpAddress* address)
  * Returns a string representation of @address in standard dots format (like
  * "192.168.0.1" or "::1").
  *
- * Return Value: A newly-allocated string. Free with g_free().
+ * Returns: (transfer full): A newly-allocated string. Free with g_free().
  **/
 gchar*
 inf_ip_address_to_string(const InfIpAddress* address)
@@ -424,7 +395,7 @@ inf_ip_address_to_string(const InfIpAddress* address)
  *
  * Compares the two addresses for sorting.
  *
- * Return Value: -1 if @address1 compares before, 0 if it compares equal or
+ * Returns: -1 if @address1 compares before, 0 if it compares equal or
  * 1 if it compares after @address.
  **/
 int
