@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,7 +38,6 @@
 
 #include <libinfinity/common/inf-name-resolver.h>
 #include <libinfinity/common/inf-async-operation.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-i18n.h>
 
 /* For getaddrinfo */
@@ -126,9 +125,11 @@ enum {
 
 #define INF_NAME_RESOLVER_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TYPE_NAME_RESOLVER, InfNameResolverPrivate))
 
-static GObjectClass* parent_class;
 static guint name_resolver_signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE_WITH_CODE(InfNameResolver, inf_name_resolver, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfNameResolver))
+
 static void
 inf_name_resolver_result_nullify(InfNameResolverResult* result)
 {
@@ -174,11 +175,10 @@ static void
 inf_name_resolver_set_herrno_error(GError** error,
                                    int code)
 {
-  g_set_error(
+  g_set_error_literal(
     error,
     g_quark_from_static_string("INF_NAME_RESOLVER_HERRNO_ERROR"),
     code,
-    "%s",
     hstrerror(code)
   );
 }
@@ -187,11 +187,10 @@ static void
 inf_name_resolver_set_errno_error(GError** error,
                                   int code)
 {
-  g_set_error(
+  g_set_error_literal(
     error,
     g_quark_from_static_string("INF_NAME_RESOLVER_ERRNO_ERROR"),
     code,
-    "%s",
     strerror(code)
   );
 }
@@ -199,11 +198,10 @@ inf_name_resolver_set_errno_error(GError** error,
 static void
 inf_name_resolver_set_incomplete_error(GError** error)
 {
-  g_set_error(
+  g_set_error_literal(
     error,
     g_quark_from_static_string("INF_NAME_RESOLVER_INCOMPLETE_ERROR"),
     0,
-    "%s",
     _("The reply from the DNS server is incomplete")
   );
 }
@@ -430,11 +428,10 @@ inf_name_resolver_lookup_srv(const gchar* query,
   {
     str = g_win32_error_message(status);
 
-    g_set_error(
+    g_set_error_literal(
       error,
       g_quark_from_static_string("WIN32_ERROR"),
       status,
-      "%s",
       str
     );
 
@@ -478,10 +475,10 @@ inf_name_resolver_lookup_srv(const gchar* query,
 
   /* libresolv uses a global struct for its operation, and is not threadsafe.
    * Therefore, we protect this function callby a mutex. */
-  static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+  static GMutex mutex;
 
   /* Make the DNS query */
-  g_static_mutex_lock(&mutex);
+  g_mutex_lock(&mutex);
 
   *n_srvs = 0;
   h_errno = 0;
@@ -492,13 +489,13 @@ inf_name_resolver_lookup_srv(const gchar* query,
     /* If the host was not found, simply return 0 SRV entries */
     if(h_errno != HOST_NOT_FOUND)
       inf_name_resolver_set_herrno_error(error, h_errno);
-    g_static_mutex_unlock(&mutex);
+    g_mutex_unlock(&mutex);
     return NULL;
   }
 
   /* At this point we have the DNS answer, and the rest of the function
    * is thread-safe. */
-  g_static_mutex_unlock(&mutex);
+  g_mutex_unlock(&mutex);
 
   if(len < sizeof(HEADER))
   {
@@ -599,11 +596,10 @@ inf_name_resolver_lookup_a_aaaa(const gchar* hostname,
 
   if(err != 0)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       g_quark_from_static_string("GETADDRINFO_ERROR"),
       err,
-      "%s",
       gai_strerror(err)
     );
 
@@ -933,14 +929,10 @@ inf_name_resolver_backup_done_func(gpointer run_data,
 }
 
 static void
-inf_name_resolver_init(GTypeInstance* instance,
-                       gpointer g_class)
+inf_name_resolver_init(InfNameResolver* resolver)
 {
-  InfNameResolver* connection;
   InfNameResolverPrivate* priv;
-
-  connection = INF_NAME_RESOLVER(instance);
-  priv = INF_NAME_RESOLVER_PRIVATE(connection);
+  priv = INF_NAME_RESOLVER_PRIVATE(resolver);
 
   priv->io = NULL;
   priv->hostname = NULL;
@@ -955,11 +947,11 @@ inf_name_resolver_init(GTypeInstance* instance,
 static void
 inf_name_resolver_dispose(GObject* object)
 {
-  InfNameResolver* connection;
+  InfNameResolver* resolver;
   InfNameResolverPrivate* priv;
 
-  connection = INF_NAME_RESOLVER(object);
-  priv = INF_NAME_RESOLVER_PRIVATE(connection);
+  resolver = INF_NAME_RESOLVER(object);
+  priv = INF_NAME_RESOLVER_PRIVATE(resolver);
 
   if(priv->operation != NULL)
   {
@@ -973,17 +965,17 @@ inf_name_resolver_dispose(GObject* object)
     priv->io = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_name_resolver_parent_class)->dispose(object);
 }
 
 static void
 inf_name_resolver_finalize(GObject* object)
 {
-  InfNameResolver* connection;
+  InfNameResolver* resolver;
   InfNameResolverPrivate* priv;
 
-  connection = INF_NAME_RESOLVER(object);
-  priv = INF_NAME_RESOLVER_PRIVATE(connection);
+  resolver = INF_NAME_RESOLVER(object);
+  priv = INF_NAME_RESOLVER_PRIVATE(resolver);
 
   inf_name_resolver_result_cleanup(&priv->result);
 
@@ -991,7 +983,7 @@ inf_name_resolver_finalize(GObject* object)
   g_free(priv->service);
   g_free(priv->srv);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_name_resolver_parent_class)->finalize(object);
 }
 
 static void
@@ -1071,17 +1063,10 @@ inf_name_resolver_get_property(GObject* object,
 }
 
 static void
-inf_name_resolver_class_init(gpointer g_class,
-                             gpointer class_data)
+inf_name_resolver_class_init(InfNameResolverClass* name_resolver_class)
 {
   GObjectClass* object_class;
-  InfNameResolverClass* name_resolver_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  name_resolver_class = INF_NAME_RESOLVER_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfNameResolverPrivate));
+  object_class = G_OBJECT_CLASS(name_resolver_class);
 
   object_class->dispose = inf_name_resolver_dispose;
   object_class->finalize = inf_name_resolver_finalize;
@@ -1153,50 +1138,19 @@ inf_name_resolver_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfNameResolverClass, resolved),
     NULL, NULL,
-    inf_marshal_VOID__POINTER,
+    g_cclosure_marshal_VOID__BOXED,
     G_TYPE_NONE,
     1,
-    G_TYPE_POINTER /* GError */
+    G_TYPE_ERROR
   );
 }
 
-GType
-inf_name_resolver_get_type(void)
-{
-  static GType name_resolver_type = 0;
-
-  if(!name_resolver_type)
-  {
-    static const GTypeInfo name_resolver_type_info = {
-      sizeof(InfNameResolverClass),  /* class_size */
-      NULL,                          /* base_init */
-      NULL,                          /* base_finalize */
-      inf_name_resolver_class_init,  /* class_init */
-      NULL,                          /* class_finalize */
-      NULL,                          /* class_data */
-      sizeof(InfNameResolver),       /* instance_size */
-      0,                             /* n_preallocs */
-      inf_name_resolver_init,        /* instance_init */
-      NULL                           /* value_table */
-    };
-
-    name_resolver_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfNameResolver",
-      &name_resolver_type_info,
-      0
-    );
-  }
-
-  return name_resolver_type;
-}
-
 /**
- * inf_name_resolver_new:
+ * inf_name_resolver_new: (constructor)
  * @io: A #InfIo object used to schedule events in the main thread.
  * @hostname: The hostname to look up.
- * @service: The name of the service to look up, or %NULL.
- * @srv: The SRV record to look up, or %NULL.
+ * @service: (allow-none): The name of the service to look up, or %NULL.
+ * @srv: (allow-none): The SRV record to look up, or %NULL.
  *
  * Creates a new #InfNameResolver. Use inf_name_resolver_start() to start
  * resolving the hostname.
@@ -1215,7 +1169,8 @@ inf_name_resolver_get_type(void)
  * inf_name_resolver_get_port() is then different from 0, then use that
  * port number, otherwise the default port number for the service.
  *
- * Returns: A new #InfNameResolver. Free with g_object_unref().
+ * Returns: (transfer full): A new #InfNameResolver. Free with
+ * g_object_unref().
  **/
 InfNameResolver*
 inf_name_resolver_new(InfIo* io,
@@ -1247,7 +1202,7 @@ inf_name_resolver_new(InfIo* io,
  *
  * Returns the currently configured hostname that @resolver will look up.
  *
- * Returns: The currently configured hostname.
+ * Returns: (allow-none): The currently configured hostname.
  */
 const gchar*
 inf_name_resolver_get_hostname(InfNameResolver* resolver)
@@ -1262,7 +1217,7 @@ inf_name_resolver_get_hostname(InfNameResolver* resolver)
  *
  * Returns the currently configured service that @resolver will look up.
  *
- * Returns: The currently configured service.
+ * Returns: (allow-none): The currently configured service.
  */
 const gchar*
 inf_name_resolver_get_service(InfNameResolver* resolver)
@@ -1277,7 +1232,7 @@ inf_name_resolver_get_service(InfNameResolver* resolver)
  *
  * Returns the currently configured SRV record that @resolver will look up.
  *
- * Returns: The currently configured SRV record.
+ * Returns: (allow-none): The currently configured SRV record.
  */
 const gchar*
 inf_name_resolver_get_srv(InfNameResolver* resolver)
@@ -1445,7 +1400,7 @@ inf_name_resolver_get_n_addresses(InfNameResolver* resolver)
  * Returns the @index<!-- -->th address looked up in the last hostname
  * resolution operation.
  *
- * Returns: The looked up #InfIpAddress.
+ * Returns: (transfer none): The looked up #InfIpAddress.
  */
 const InfIpAddress*
 inf_name_resolver_get_address(InfNameResolver* resolver,
