@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -45,7 +45,6 @@
 #include <libinfinity/communication/inf-communication-manager.h>
 #include <libinfinity/communication/inf-communication-registry.h>
 #include <libinfinity/communication/inf-communication-group-private.h>
-#include <libinfinity/inf-marshal.h>
 
 #include <string.h>
 
@@ -80,9 +79,11 @@ enum {
 
 #define INF_COMMUNICATION_GROUP_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_COMMUNICATION_TYPE_GROUP, InfCommunicationGroupPrivate))
 
-static GObjectClass* parent_class;
 static guint group_signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE_EXTENDED(InfCommunicationGroup, inf_communication_group, G_TYPE_OBJECT, G_TYPE_FLAG_ABSTRACT,
+  G_ADD_PRIVATE(InfCommunicationGroup))
+
 /*
  * Signal handlers
  */
@@ -318,13 +319,9 @@ inf_communication_group_set_registry(InfCommunicationGroup* group,
  */
 
 static void
-inf_communication_group_init(GTypeInstance* instance,
-                             gpointer g_class)
+inf_communication_group_init(InfCommunicationGroup* group)
 {
-  InfCommunicationGroup* group;
   InfCommunicationGroupPrivate* priv;
-
-  group = INF_COMMUNICATION_GROUP(instance);
   priv = INF_COMMUNICATION_GROUP_PRIVATE(group);
 
   priv->communication_manager = NULL;
@@ -356,7 +353,7 @@ inf_communication_group_dispose(GObject* object)
   inf_communication_group_set_manager(group, NULL);
   inf_communication_group_set_target(group, NULL);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_communication_group_parent_class)->dispose(object);
 }
 
 static void
@@ -370,7 +367,7 @@ inf_communication_group_finalize(GObject* object)
 
   g_free(priv->name);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_communication_group_parent_class)->finalize(object);
 }
 
 static void
@@ -461,17 +458,10 @@ inf_communication_group_get_property(GObject* object,
  */
 
 static void
-inf_communication_group_class_init(gpointer g_class,
-                                   gpointer class_data)
+inf_communication_group_class_init(InfCommunicationGroupClass* group_class)
 {
   GObjectClass* object_class;
-  InfCommunicationGroupClass* group_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  group_class = INF_COMMUNICATION_GROUP_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfCommunicationGroupPrivate));
+  object_class = G_OBJECT_CLASS(group_class);
 
   object_class->dispose = inf_communication_group_dispose;
   object_class->finalize = inf_communication_group_finalize;
@@ -544,7 +534,7 @@ inf_communication_group_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfCommunicationGroupClass, member_added),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_XML_CONNECTION
@@ -563,44 +553,13 @@ inf_communication_group_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfCommunicationGroupClass, member_removed),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_XML_CONNECTION
   );
 }
 
-GType
-inf_communication_group_get_type(void)
-{
-  static GType group_type = 0;
-
-  if(!group_type)
-  {
-    static const GTypeInfo group_type_info = {
-      sizeof(InfCommunicationGroupClass),  /* class_size */
-      NULL,                                /* base_init */
-      NULL,                                /* base_finalize */
-      inf_communication_group_class_init,  /* class_init */
-      NULL,                                /* class_finalize */
-      NULL,                                /* class_data */
-      sizeof(InfCommunicationGroup),       /* instance_size */
-      0,                                   /* n_preallocs */
-      inf_communication_group_init,        /* instance_init */
-      NULL                                 /* value_table */
-    };
-
-    group_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfCommunicationGroup",
-      &group_type_info,
-      G_TYPE_FLAG_ABSTRACT
-    );
-  }
-
-  return group_type;
-}
-
 /*
  * Public API.
  */
@@ -628,7 +587,7 @@ inf_communication_group_get_name(InfCommunicationGroup* group)
  * Returns the group's target. The target of a group is the
  * #InfCommunicationObject to which received and sent messages are reported.
  *
- * Returns: A #InfCommunicationGroup, or %NULL.
+ * Returns: (transfer none) (allow-none): A #InfCommunicationGroup, or %NULL.
  */
 InfCommunicationObject*
 inf_communication_group_get_target(InfCommunicationGroup* group)
@@ -721,7 +680,7 @@ inf_communication_group_is_member(InfCommunicationGroup* group,
  * inf_communication_group_send_message:
  * @group: A #InfCommunicationGroup.
  * @connection: The #InfXmlConnection to which to send the message.
- * @xml: The message to send.
+ * @xml: (transfer full): The message to send.
  *
  * Sends a message @connection which must be a member of @group. @connection
  * needs to be a member of this group. This function takes ownership of @xml.
@@ -750,7 +709,7 @@ inf_communication_group_send_message(InfCommunicationGroup* group,
 /**
  * inf_communication_group_send_group_message:
  * @group: A #InfCommunicationGroup.
- * @xml: The message to send.
+ * @xml: (transfer full): The message to send.
  *
  * Sends a message to all members of @group. This function takes ownership
  * of @xml.
@@ -892,7 +851,7 @@ inf_communication_group_get_method_for_connection(InfCommunicationGroup* grp,
  * local ID, otherwise the remote ID of the connection to the publisher on
  * @for_connection's network is returned.
  *
- * Returns: The publisher's host ID. Free with g_free().
+ * Returns: (transfer full): The publisher's host ID. Free with g_free().
  */
 gchar*
 inf_communication_group_get_publisher_id(InfCommunicationGroup* group,
