@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -56,7 +56,8 @@ enum {
 
 #define INF_COMMUNICATION_JOINED_GROUP_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_COMMUNICATION_TYPE_JOINED_GROUP, InfCommunicationJoinedGroupPrivate))
 
-static GObjectClass* parent_class;
+G_DEFINE_TYPE_WITH_CODE(InfCommunicationJoinedGroup, inf_communication_joined_group, INF_COMMUNICATION_TYPE_GROUP,
+  G_ADD_PRIVATE(InfCommunicationJoinedGroup))
 
 /* Required by inf_communication_joined_group_publisher_notify_status_cb() */
 static void
@@ -150,13 +151,9 @@ inf_communication_joined_group_get_publisher_id(InfCommunicationGroup* group,
  */
 
 static void
-inf_communication_joined_group_init(GTypeInstance* instance,
-                                    gpointer g_class)
+inf_communication_joined_group_init(InfCommunicationJoinedGroup* group)
 {
-  InfCommunicationJoinedGroup* group;
   InfCommunicationJoinedGroupPrivate* priv;
-
-  group = INF_COMMUNICATION_JOINED_GROUP(instance);
   priv = INF_COMMUNICATION_JOINED_GROUP_PRIVATE(group);
 
   priv->publisher_conn = NULL;
@@ -164,21 +161,16 @@ inf_communication_joined_group_init(GTypeInstance* instance,
   priv->method = NULL;
 }
 
-static GObject*
-inf_communication_joined_group_constructor(GType type,
-                                           guint n_construct_properties,
-                                           GObjectConstructParam* properties)
+static void
+inf_communication_joined_group_constructed(GObject* object)
 {
-  GObject* object;
   InfCommunicationJoinedGroupPrivate* priv;
+  priv = INF_COMMUNICATION_JOINED_GROUP_PRIVATE(object);
 
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_construct_properties,
-    properties
+  G_OBJECT_CLASS(inf_communication_joined_group_parent_class)->constructed(
+    object
   );
 
-  priv = INF_COMMUNICATION_JOINED_GROUP_PRIVATE(object);
   g_assert(priv->publisher_conn != NULL);
   g_assert(priv->method != NULL);
 
@@ -195,8 +187,6 @@ inf_communication_joined_group_constructor(GType type,
     INF_COMMUNICATION_GROUP(object),
     priv->publisher_conn
   );
-
-  return object;
 }
 
 static void
@@ -210,7 +200,9 @@ inf_communication_joined_group_dispose(GObject* object)
 
   inf_communication_joined_group_set_publisher(group, NULL);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_communication_joined_group_parent_class)->dispose(
+    object
+  );
 }
 
 static void
@@ -225,7 +217,9 @@ inf_communication_joined_group_finalize(GObject* object)
   g_free(priv->publisher_id);
   g_free(priv->method);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_communication_joined_group_parent_class)->finalize(
+    object
+  );
 }
 
 static void
@@ -292,23 +286,16 @@ inf_communication_joined_group_get_property(GObject* object,
  */
 
 static void
-inf_communication_joined_group_class_init(gpointer g_class,
-                                          gpointer class_data)
+inf_communication_joined_group_class_init(
+  InfCommunicationJoinedGroupClass* joined_group_class)
 {
   GObjectClass* object_class;
   InfCommunicationGroupClass* group_class;
 
-  object_class = G_OBJECT_CLASS(g_class);
-  group_class = INF_COMMUNICATION_GROUP_CLASS(g_class);
+  object_class = G_OBJECT_CLASS(joined_group_class);
+  group_class = INF_COMMUNICATION_GROUP_CLASS(joined_group_class);
 
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-
-  g_type_class_add_private(
-    g_class,
-    sizeof(InfCommunicationJoinedGroupPrivate)
-  );
-
-  object_class->constructor = inf_communication_joined_group_constructor;
+  object_class->constructed = inf_communication_joined_group_constructed;
   object_class->dispose = inf_communication_joined_group_dispose;
   object_class->finalize = inf_communication_joined_group_finalize;
   object_class->set_property = inf_communication_joined_group_set_property;
@@ -343,37 +330,6 @@ inf_communication_joined_group_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_communication_joined_group_get_type(void)
-{
-  static GType joined_group_type = 0;
-
-  if(!joined_group_type)
-  {
-    static const GTypeInfo joined_group_type_info = {
-      sizeof(InfCommunicationJoinedGroupClass),   /* class_size */
-      NULL,                                       /* base_init */
-      NULL,                                       /* base_finalize */
-      inf_communication_joined_group_class_init,  /* class_init */
-      NULL,                                       /* class_finalize */
-      NULL,                                       /* class_data */
-      sizeof(InfCommunicationJoinedGroup),        /* instance_size */
-      0,                                          /* n_preallocs */
-      inf_communication_joined_group_init,        /* instance_init */
-      NULL                                        /* value_table */
-    };
-
-    joined_group_type = g_type_register_static(
-      INF_COMMUNICATION_TYPE_GROUP,
-      "InfCommunicationJoinedGroup",
-      &joined_group_type_info,
-      0
-    );
-  }
-
-  return joined_group_type;
-}
-
 /*
  * Public API.
  */
@@ -413,7 +369,7 @@ inf_communication_joined_group_remove_member(InfCommunicationJoinedGroup* grp,
  * Returns a #InfXmlConnection to the group's publisher, or %NULL if the
  * publisher is no longer a group member.
  *
- * Returns: A #InfXmlConnection, or %NULL.
+ * Returns: (transfer none) (allow-none): A #InfXmlConnection, or %NULL.
  */
 InfXmlConnection*
 inf_communication_joined_group_get_publisher(InfCommunicationJoinedGroup* g)
