@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -57,7 +57,8 @@ struct _InfCommunicationManagerPrivate {
 
 #define INF_COMMUNICATION_MANAGER_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_COMMUNICATION_TYPE_MANAGER, InfCommunicationManagerPrivate))
 
-static GObjectClass* parent_class;
+G_DEFINE_TYPE_WITH_CODE(InfCommunicationManager, inf_communication_manager, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfCommunicationManager))
 
 static void
 inf_communication_manager_joined_key_free(gpointer key_)
@@ -175,13 +176,9 @@ inf_communication_manager_joined_group_unrefed(gpointer data,
  */
 
 static void
-inf_communication_manager_init(GTypeInstance* instance,
-                               gpointer g_class)
+inf_communication_manager_init(InfCommunicationManager* manager)
 {
-  InfCommunicationManager* manager;
   InfCommunicationManagerPrivate* priv;
-
-  manager = INF_COMMUNICATION_MANAGER(instance);
   priv = INF_COMMUNICATION_MANAGER_PRIVATE(manager);
 
   priv->registry = g_object_new(INF_COMMUNICATION_TYPE_REGISTRY, NULL);
@@ -235,67 +232,25 @@ inf_communication_manager_dispose(GObject* object)
     priv->registry = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_communication_manager_parent_class)->dispose(object);
 }
 
-/*
- * GType registration.
- */
-
 static void
-inf_communication_manager_class_init(gpointer g_class,
-                                     gpointer class_data)
+inf_communication_manager_class_init(
+  InfCommunicationManagerClass* manager_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfCommunicationManagerPrivate));
+  object_class = G_OBJECT_CLASS(manager_class);
 
   object_class->dispose = inf_communication_manager_dispose;
 }
 
-GType
-inf_communication_manager_get_type(void)
-{
-  static GType manager_type = 0;
-
-  if(!manager_type)
-  {
-    static const GTypeInfo manager_type_info = {
-      sizeof(InfCommunicationManagerClass),  /* class_size */
-      NULL,                                  /* base_init */
-      NULL,                                  /* base_finalize */
-      inf_communication_manager_class_init,  /* class_init */
-      NULL,                                  /* class_finalize */
-      NULL,                                  /* class_data */
-      sizeof(InfCommunicationManager),       /* instance_size */
-      0,                                     /* n_preallocs */
-      inf_communication_manager_init,        /* instance_init */
-      NULL                                   /* value_table */
-    };
-
-    manager_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfCommunicationManager",
-      &manager_type_info,
-      0
-    );
-  }
-
-  return manager_type;
-}
-
-/*
- * Public API.
- */
-
 /**
- * inf_communication_manager_new:
+ * inf_communication_manager_new: (constructor)
  *
  * Creates a new #InfCommunicationManager.
  *
- * Returns: A new #InfCommunicationManager.
+ * Returns: (transfer full): A new #InfCommunicationManager.
  */
 InfCommunicationManager*
 inf_communication_manager_new(void)
@@ -307,7 +262,8 @@ inf_communication_manager_new(void)
  * inf_communication_manager_open_group:
  * @manager: A #InfCommunicationManager.
  * @group_name: A name for the new group.
- * @methods: Methods to support, or %NULL.
+ * @methods: (allow-none) (array zero-terminated=1): Methods to support, or
+ * %NULL.
  *
  * Opens a new communication group published by the local host. @group_name
  * is an identifier for the group via which other hosts can join the group
@@ -320,8 +276,8 @@ inf_communication_manager_new(void)
  * "central" method will be used, which is guaranteed to be supported for
  * all networks.
  *
- * Returns: A #InfCommunicationHostedGroup. Free with g_object_unref() to
- * leave the group.
+ * Returns: (transfer full): A #InfCommunicationHostedGroup. Free with
+ * g_object_unref() to leave the group.
  */
 InfCommunicationHostedGroup*
 inf_communication_manager_open_group(InfCommunicationManager* manager,
@@ -390,8 +346,8 @@ inf_communication_manager_open_group(InfCommunicationManager* manager,
  * inf_communication_manager_get_factory_for() for @publisher_conn's network
  * and @method returns %NULL).
  *
- * Returns: A new #InfCommunicationJoinedGroup, or %NULL. Free with
- * g_object_unref() to leave the group.
+ * Returns: (transfer full) (allow-none): A new #InfCommunicationJoinedGroup,
+ * or %NULL. Free with g_object_unref() to leave the group.
  */
 InfCommunicationJoinedGroup*
 inf_communication_manager_join_group(InfCommunicationManager* manager,
@@ -507,7 +463,8 @@ inf_communication_manager_add_factory(InfCommunicationManager* manager,
  * a #InfCommunicationMethod for @method_name on @network, or %NULL if the
  * network/method combination is not supported.
  *
- * Returns: A #InfCommunicationFactory, or %NULL.
+ * Returns: (transfer none) (allow-none): A #InfCommunicationFactory,
+ * or %NULL.
  */
 InfCommunicationFactory*
 inf_communication_manager_get_factory_for(InfCommunicationManager* manager,
