@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -91,7 +91,8 @@ struct _InfCommunicationRegistryPrivate {
 
 #define INF_COMMUNICATION_REGISTRY_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_COMMUNICATION_TYPE_REGISTRY, InfCommunicationRegistryPrivate))
 
-static GObjectClass* parent_class;
+G_DEFINE_TYPE_WITH_CODE(InfCommunicationRegistry, inf_communication_registry, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfCommunicationRegistry))
 
 /* Maximum number of messages enqueued at the same time */
 static const guint INF_COMMUNICATION_REGISTRY_INNER_QUEUE_LIMIT = 5;
@@ -100,6 +101,9 @@ static void
 inf_communication_registry_send_real(InfCommunicationRegistryEntry* entry,
                                      guint num_messages)
 {
+  InfXmlConnection* connection;
+  InfXmlConnectionStatus status;
+
   xmlNodePtr container;
   xmlNodePtr child;
   xmlNodePtr xml;
@@ -140,6 +144,12 @@ inf_communication_registry_send_real(InfCommunicationRegistryEntry* entry,
     entry->enqueued_list = container;
     child = container;
 
+    connection = entry->key.connection;
+    g_object_ref(connection);
+
+    g_object_get(G_OBJECT(connection), "status", &status, NULL);
+    g_assert(status == INF_XML_CONNECTION_OPEN);
+
     while(child != NULL)
     {
       /* TODO: The group could be unset at this point if called from
@@ -151,11 +161,7 @@ inf_communication_registry_send_real(InfCommunicationRegistryEntry* entry,
       {
         for(xml = child->children; xml != NULL; xml = xml->next)
         {
-          inf_communication_method_enqueued(
-            entry->method,
-            entry->key.connection,
-            xml
-          );
+          inf_communication_method_enqueued(entry->method, connection, xml);
         }
       }
 
@@ -174,8 +180,15 @@ inf_communication_registry_send_real(InfCommunicationRegistryEntry* entry,
        * will simply append to entry->enqueued_list, and we will enqueue and
        * send the messages within the next iteration(s).
        */
-      inf_xml_connection_send(entry->key.connection, xml);
+      inf_xml_connection_send(connection, xml);
+
+      /* Break if sending the data lead to connection closure */
+      g_object_get(G_OBJECT(connection), "status", &status, NULL);
+      if(status != INF_XML_CONNECTION_OPEN)
+        break;
     }
+
+    g_object_unref(connection);
   }
 }
 
@@ -690,13 +703,9 @@ inf_communication_registry_group_unrefed(gpointer user_data,
  */
 
 static void
-inf_communication_registry_init(GTypeInstance* instance,
-                                gpointer g_class)
+inf_communication_registry_init(InfCommunicationRegistry* registry)
 {
-  InfCommunicationRegistry* registry;
   InfCommunicationRegistryPrivate* priv;
-
-  registry = INF_COMMUNICATION_REGISTRY(instance);
   priv = INF_COMMUNICATION_REGISTRY_PRIVATE(registry);
 
   priv->connections = g_hash_table_new(NULL, NULL);
@@ -758,61 +767,19 @@ inf_communication_registry_dispose(GObject* object)
   g_hash_table_unref(priv->connections);
   g_hash_table_unref(priv->entries);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_communication_registry_parent_class)->dispose(object);
 }
 
-/*
- * GType registration.
- */
-
 static void
-inf_communication_registry_class_init(gpointer g_class,
-                                     gpointer class_data)
+inf_communication_registry_class_init(
+  InfCommunicationRegistryClass* registry_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfCommunicationRegistryPrivate));
+  object_class = G_OBJECT_CLASS(registry_class);
 
   object_class->dispose = inf_communication_registry_dispose;
 }
 
-GType
-inf_communication_registry_get_type(void)
-{
-  static GType registry_type = 0;
-
-  if(!registry_type)
-  {
-    static const GTypeInfo registry_type_info = {
-      sizeof(InfCommunicationRegistryClass),  /* class_size */
-      NULL,                                   /* base_init */
-      NULL,                                   /* base_finalize */
-      inf_communication_registry_class_init,  /* class_init */
-      NULL,                                   /* class_finalize */
-      NULL,                                   /* class_data */
-      sizeof(InfCommunicationRegistry),       /* instance_size */
-      0,                                      /* n_preallocs */
-      inf_communication_registry_init,        /* instance_init */
-      NULL                                    /* value_table */
-    };
-
-    registry_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfCommunicationRegistry",
-      &registry_type_info,
-      0
-    );
-  }
-
-  return registry_type;
-}
-
-/*
- * Public API.
- */
-
 /**
  * inf_communication_registry_register:
  * @registry: A #InfCommunicationRegistry.
@@ -1013,7 +980,7 @@ inf_communication_registry_is_registered(InfCommunicationRegistry* registry,
  * @registry: A #InfCommunicationRegistry.
  * @group: The group for which to send the message #InfCommunicationGroup.
  * @connection: A registered #InfXmlConnection.
- * @xml: The message to send.
+ * @xml: (transfer full): The message to send.
  *
  * Sends an XML message to @connection. @connection must have been registered
  * with inf_communication_registry_register() before. If the message has been
