@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,9 +54,13 @@ enum {
 
 #define INFD_FILESYSTEM_STORAGE_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INFD_TYPE_FILESYSTEM_STORAGE, InfdFilesystemStoragePrivate))
 
-static GObjectClass* parent_class;
 static GQuark infd_filesystem_storage_error_quark;
 
+static void infd_filesystem_storage_storage_iface_init(InfdStorageInterface* iface);
+G_DEFINE_TYPE_WITH_CODE(InfdFilesystemStorage, infd_filesystem_storage, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfdFilesystemStorage)
+  G_IMPLEMENT_INTERFACE(INFD_TYPE_STORAGE, infd_filesystem_storage_storage_iface_init))
+
 /* Checks whether path is valid, and sets error if not */
 static gboolean
 infd_filesystem_storage_verify_path(const gchar* path,
@@ -65,17 +69,26 @@ infd_filesystem_storage_verify_path(const gchar* path,
   gchar** components;
   gchar** component;
 
-  components = g_strsplit(path, "/", 0);
+  if(path[0] != '/')
+  {
+    g_set_error_literal(
+      error,
+      infd_filesystem_storage_error_quark,
+      INFD_FILESYSTEM_STORAGE_ERROR_INVALID_PATH,
+      _("The path does not start with \"/\"")
+    );
+  }
+
+  components = g_strsplit(path + 1, "/", 0);
   for(component = components; *component != NULL; ++ component)
   {
-    if(*component == '\0' ||
+    if(**component == '\0' ||
        strcmp(*component, ".") == 0 || strcmp(*component, "..") == 0)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         infd_filesystem_storage_error_quark,
         INFD_FILESYSTEM_STORAGE_ERROR_INVALID_PATH,
-        "%s",
         _("The path contains invalid components")
       );
 
@@ -114,6 +127,14 @@ infd_filesystem_storage_set_root_directory(InfdFilesystemStorage* storage,
   }
   else
   {
+    if (!g_path_is_absolute(converted))
+    {
+      gchar* cwd = g_get_current_dir();
+      gchar* full_path = g_build_filename(cwd, converted, NULL);
+      g_free(cwd);
+      g_free(converted);
+      converted = full_path;
+    }
     if(!inf_file_util_create_directory(converted, 0755, &error))
     {
       g_warning(
@@ -134,11 +155,10 @@ static void
 infd_filesystem_storage_system_error(int code,
                                      GError** error)
 {
-  g_set_error(
+  g_set_error_literal(
     error,
     G_FILE_ERROR,
     g_file_error_from_errno(code),
-    "%s",
     g_strerror(code)
   );
 }
@@ -298,11 +318,10 @@ infd_filesystem_storage_write_xml_file_impl(InfdFilesystemStorage* storage,
     fclose(file);
     /* TODO: unlink? */
 
-    g_set_error(
+    g_set_error_literal(
       error,
       g_quark_from_static_string("LIBXML2_OUTPUT_ERROR"),
       xmlerror->code,
-      "%s",
       xmlerror->message
     );
 
@@ -334,20 +353,16 @@ infd_filesystem_storage_get_acl_path(InfdFilesystemStorage* storage,
   else
   {
     full_path =
-      infd_filesystem_storage_get_path(storage, "xml", "global-acl", error);
+      infd_filesystem_storage_get_path(storage, "xml", "/global-acl", error);
   }
 
   return full_path;
 }
 
 static void
-infd_filesystem_storage_init(GTypeInstance* instance,
-                             gpointer g_class)
+infd_filesystem_storage_init(InfdFilesystemStorage* storage)
 {
-  InfdFilesystemStorage* storage;
   InfdFilesystemStoragePrivate* priv;
-
-  storage = INFD_FILESYSTEM_STORAGE(instance);
   priv = INFD_FILESYSTEM_STORAGE_PRIVATE(storage);
 
   priv->root_directory = NULL;
@@ -364,7 +379,7 @@ infd_filesystem_storage_finalize(GObject* object)
 
   g_free(priv->root_directory);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(infd_filesystem_storage_parent_class)->finalize(object);
 }
 
 static void
@@ -785,17 +800,11 @@ infd_filesystem_storage_storage_write_acl(InfdStorage* storage,
 }
 
 static void
-infd_filesystem_storage_class_init(gpointer g_class,
-                                   gpointer class_data)
+infd_filesystem_storage_class_init(
+  InfdFilesystemStorageClass* filesystem_storage_class)
 {
   GObjectClass* object_class;
-  InfdFilesystemStorageClass* filesystem_storage_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  filesystem_storage_class = INFD_FILESYSTEM_STORAGE_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfdFilesystemStoragePrivate));
+  object_class = G_OBJECT_CLASS(filesystem_storage_class);
 
   object_class->finalize = infd_filesystem_storage_finalize;
   object_class->set_property = infd_filesystem_storage_set_property;
@@ -819,12 +828,8 @@ infd_filesystem_storage_class_init(gpointer g_class,
 }
 
 static void
-infd_filesystem_storage_storage_init(gpointer g_iface,
-                                     gpointer iface_data)
+infd_filesystem_storage_storage_iface_init(InfdStorageInterface* iface)
 {
-  InfdStorageIface* iface;
-  iface = (InfdStorageIface*)g_iface;
-
   iface->read_subdirectory =
     infd_filesystem_storage_storage_read_subdirectory;
   iface->create_subdirectory =
@@ -837,58 +842,15 @@ infd_filesystem_storage_storage_init(gpointer g_iface,
     infd_filesystem_storage_storage_write_acl;
 }
 
-GType
-infd_filesystem_storage_get_type(void)
-{
-  static GType filesystem_storage_type = 0;
-
-  if(!filesystem_storage_type)
-  {
-    static const GTypeInfo filesystem_storage_type_info = {
-      sizeof(InfdFilesystemStorageClass),  /* class_size */
-      NULL,                                /* base_init */
-      NULL,                                /* base_finalize */
-      infd_filesystem_storage_class_init,  /* class_init */
-      NULL,                                /* class_finalize */
-      NULL,                                /* class_data */
-      sizeof(InfdFilesystemStorage),       /* instance_size */
-      0,                                   /* n_preallocs */
-      infd_filesystem_storage_init,        /* instance_init */
-      NULL                                 /* value_table */
-    };
-
-    static const GInterfaceInfo storage_info = {
-      infd_filesystem_storage_storage_init,
-      NULL,
-      NULL
-    };
-
-    filesystem_storage_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfdFilesystemStorage",
-      &filesystem_storage_type_info,
-      0
-    );
-
-    g_type_add_interface_static(
-      filesystem_storage_type,
-      INFD_TYPE_STORAGE,
-      &storage_info
-    );
-  }
-
-  return filesystem_storage_type;
-}
-
 /**
- * infd_filesystem_storage_new:
+ * infd_filesystem_storage_new: (constructor)
  * @root_directory: A directory name in UTF-8.
  *
  * Creates a new #InfdFilesystemStorage that stores its nodes in the
  * given directory on the file system. The directory is created if it does
  * not exist.
  *
- * Returns: A new #InfdFilesystemStorage.
+ * Returns: (transfer full): A new #InfdFilesystemStorage.
  **/
 InfdFilesystemStorage*
 infd_filesystem_storage_new(const gchar* root_directory)
@@ -920,7 +882,8 @@ infd_filesystem_storage_new(const gchar* root_directory)
  * identifiers can be used to store custom data in the filesystem, linked to
  * this #InfdFilesystemStorage object.
  *
- * Returns: An absolute filename path to be freed with g_free(), or %NULL.
+ * Returns: (type filename) (allow-none) (transfer full): An absolute
+ * filename path to be freed with g_free(), or %NULL.
  */
 gchar*
 infd_filesystem_storage_get_path(InfdFilesystemStorage* storage,
@@ -956,7 +919,8 @@ infd_filesystem_storage_get_path(InfdFilesystemStorage* storage,
  * @identifier: The type of node to open.
  * @path: The path to open, in UTF-8.
  * @mode: Either "r" for reading or "w" for writing.
- * @full_path: Return location of the full filename, or %NULL.
+ * @full_path: (out) (type filename) (transfer full): Return location
+ * of the full filename, or %NULL.
  * @error: Location to store error information, if any.
  *
  * Opens a file in the given path within the storage's root directory. If
@@ -971,7 +935,7 @@ infd_filesystem_storage_get_path(InfdFilesystemStorage* storage,
  * identifiers can be used to store custom data in the filesystem, linked to
  * this #InfdFilesystemStorage object.
  *
- * Returns: A stream for the open file. Close with
+ * Returns: (transfer full): A stream for the open file. Close with
  * infd_filesystem_storage_stream_close().
  **/
 FILE*
@@ -1031,7 +995,8 @@ infd_filesystem_storage_open(InfdFilesystemStorage* storage,
  * If @toplevel_tag is non-%NULL, then this function generates an error if
  * the XML document read has a toplevel tag with a different name.
  *
- * Returns: A new XML document, or %NULL on error. Free with xmlDocFree().
+ * Returns: (transfer full) (allow-none): A new XML document, or %NULL on
+ * error. Free with xmlDocFree().
  **/
 xmlDocPtr
 infd_filesystem_storage_read_xml_file(InfdFilesystemStorage* storage,
@@ -1141,7 +1106,8 @@ infd_filesystem_storage_stream_close(FILE* file)
 /**
  * infd_filesystem_storage_stream_read:
  * @file: A #FILE opened with infd_filesystem_storage_open().
- * @buffer: A buffer into which to read data.
+ * @buffer: (type guint8*) (array length=len) (out): A buffer
+ * into which to read data.
  * @len: Maximum number of bytes to read.
  *
  * This is a thin wrapper around fread(). Use this function instead of
