@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,12 +17,35 @@
  * MA 02110-1301, USA.
  */
 
-#include <libinfinity/server/infd-session-proxy.h>
+/**
+ * SECTION:infd-session-proxy
+ * @title: InfdSessionProxy
+ * @short_description: Server-side session management
+ * @include: libinfinity/server/infd-session-proxy.h
+ * @see_also: #InfSession
+ * @stability: Unstable
+ *
+ * #InfdSessionProxy is a manager for #InfSession that is specific for the
+ * server side. The #InfdSessionProxy manages exactly one #InfSession. It is
+ * usually created by #InfdDirectory when the content of a node is requested.
+ *
+ * #InfdSessionProxy allows to perform server-specific operations for the
+ * session it manages, such as joining a local user, or removing a connection
+ * from the list of subscribed connections. In addition it handles requests
+ * made by its counterpart, #InfcSessionProxy, for the client side, such as
+ * remote user joins.
+ *
+ * #InfdSessionProxy implements the #InfSessionProxy interface, which can be
+ * used to access the underlying #InfSession or to join a local user.
+ */
 
+#include <libinfinity/server/infd-session-proxy.h>
+#include <libinfinity/server/infd-request.h>
+#include <libinfinity/common/inf-session-proxy.h>
+#include <libinfinity/common/inf-request-result.h>
+#include <libinfinity/common/inf-io.h>
 #include <libinfinity/common/inf-xml-util.h>
 #include <libinfinity/common/inf-error.h>
-
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-i18n.h>
 #include <libinfinity/inf-signals.h>
 
@@ -38,6 +61,7 @@ struct _InfdSessionProxySubscription {
 
 typedef struct _InfdSessionProxyPrivate InfdSessionProxyPrivate;
 struct _InfdSessionProxyPrivate {
+  InfIo* io;
   InfSession* session;
   InfCommunicationHostedGroup* subscription_group;
 
@@ -46,7 +70,6 @@ struct _InfdSessionProxyPrivate {
 
   /* Local users that do not belong to a particular connection */
   GSList* local_users;
-
   /* Whether there are any subscriptions / synchronizations */
   gboolean idle;
 };
@@ -55,6 +78,7 @@ enum {
   PROP_0,
 
   /* construct/only */
+  PROP_IO,
   PROP_SESSION,
   PROP_SUBSCRIPTION_GROUP,
 
@@ -65,15 +89,22 @@ enum {
 enum {
   ADD_SUBSCRIPTION,
   REMOVE_SUBSCRIPTION,
+  REJECT_USER_JOIN,
 
   LAST_SIGNAL
 };
 
 #define INFD_SESSION_PROXY_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INFD_TYPE_SESSION_PROXY, InfdSessionProxyPrivate))
 
-static GObjectClass* parent_class;
 static guint session_proxy_signals[LAST_SIGNAL];
 
+static void infd_session_proxy_communication_object_iface_init(InfCommunicationObjectInterface* iface);
+static void infd_session_proxy_session_proxy_iface_init(InfSessionProxyInterface* iface);
+G_DEFINE_TYPE_WITH_CODE(InfdSessionProxy, infd_session_proxy, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfdSessionProxy)
+  G_IMPLEMENT_INTERFACE(INF_COMMUNICATION_TYPE_OBJECT, infd_session_proxy_communication_object_iface_init)
+  G_IMPLEMENT_INTERFACE(INF_TYPE_SESSION_PROXY, infd_session_proxy_session_proxy_iface_init))
+
 /*
  * SessionProxy subscriptions.
  */
@@ -128,6 +159,22 @@ infd_session_proxy_find_subscription(InfdSessionProxy* proxy,
   return (InfdSessionProxySubscription*)item->data;
 }
 
+static gboolean
+infd_session_proxy_check_idle(InfdSessionProxy* proxy)
+{
+  InfdSessionProxyPrivate* priv;
+  priv = INFD_SESSION_PROXY_PRIVATE(proxy);
+
+  if(priv->subscriptions == NULL &&
+     priv->local_users == NULL &&
+     !inf_session_has_synchronizations(priv->session))
+  {
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 static void
 infd_session_proxy_user_notify_status_cb(InfUser* user,
                                          const GParamSpec* pspec,
@@ -157,6 +204,12 @@ infd_session_proxy_user_notify_status_cb(InfUser* user,
     else
     {
       priv->local_users = g_slist_remove(priv->local_users, user);
+
+      if(priv->idle == FALSE && infd_session_proxy_check_idle(proxy) == TRUE)
+      {
+        priv->idle = TRUE;
+        g_object_notify(G_OBJECT(proxy), "idle");
+      }
     }
 
     inf_signal_handlers_disconnect_by_func(
@@ -239,12 +292,11 @@ infd_session_proxy_perform_user_join(InfdSessionProxy* proxy,
 
   if(name_param == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_request_error_quark(),
       INF_REQUEST_ERROR_NO_SUCH_ATTRIBUTE,
-      "%s",
-      _("Request does not contain required attribute 'name'")
+      _("Request does not contain required attribute \"name\"")
     );
 
     return NULL;
@@ -262,7 +314,7 @@ infd_session_proxy_perform_user_join(InfdSessionProxy* proxy,
       error,
       inf_user_error_quark(),
       INF_USER_ERROR_NAME_IN_USE,
-      _("Name '%s' already in use"),
+      _("Name \"%s\" already in use"),
       g_value_get_string(&name_param->value)
     );
 
@@ -274,11 +326,10 @@ infd_session_proxy_perform_user_join(InfdSessionProxy* proxy,
   param = inf_session_get_user_property(user_props, "id");
   if(G_IS_VALUE(&param->value))
   {
-    g_set_error(
+    g_set_error_literal(
       error,
-      inf_user_error_quark(),
-      INF_USER_ERROR_ID_PROVIDED,
-      "%s",
+      inf_request_error_quark(),
+      INF_REQUEST_ERROR_INVALID_ATTRIBUTE,
       inf_user_strerror(INF_USER_ERROR_ID_PROVIDED)
     );
 
@@ -301,12 +352,11 @@ infd_session_proxy_perform_user_join(InfdSessionProxy* proxy,
   {
     if(g_value_get_enum(&param->value) == INF_USER_UNAVAILABLE)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
-        inf_user_error_quark(),
-        INF_USER_ERROR_STATUS_UNAVAILABLE,
-        "%s",
-        _("'status' attribute is 'unavailable' in user join request")
+        inf_request_error_quark(),
+        INF_REQUEST_ERROR_INVALID_ATTRIBUTE,
+        _("\"status\" attribute is \"unavailable\" in user join request")
       );
 
       return NULL;
@@ -336,37 +386,55 @@ infd_session_proxy_perform_user_join(InfdSessionProxy* proxy,
   g_value_init(&param->value, INF_TYPE_XML_CONNECTION);
   g_value_set_object(&param->value, G_OBJECT(connection));
 
+  /* Validate properties, but exclude the rejoining user from the check.
+   * Otherwise, we would get conflicts because the name and the ID
+   * of the request and the rejoining user are the same. */
+  result = session_class->validate_user_props(
+    priv->session,
+    (const GParameter*)user_props->data,
+    user_props->len,
+    user,
+    error
+  );
+
+  if(result == FALSE)
+    return NULL;
+
+  g_signal_emit(
+    proxy,
+    session_proxy_signals[REJECT_USER_JOIN],
+    0,
+    connection,
+    user_props,
+    user,
+    &result
+  );
+
+  if(result == TRUE)
+  {
+    g_set_error_literal(
+      error,
+      inf_request_error_quark(),
+      INF_REQUEST_ERROR_NOT_AUTHORIZED,
+      _("Permission denied")
+    );
+
+    return NULL;
+  }
+
   if(user == NULL)
   {
-    /* This validates properties */
     user = inf_session_add_user(
       priv->session,
       (const GParameter*)user_props->data,
-      user_props->len,
-      error
+      user_props->len
     );
 
-    if(user == NULL)
-      return NULL;
-
+    g_assert(user != NULL);
     xml = xmlNewNode(NULL, (const xmlChar*)"user-join");
   }
   else
   {
-    /* Validate properties, but exclude the rejoining user from the check.
-     * Otherwise, we would get conflicts because the name and the ID
-     * of the request and the rejoining user are the same. */
-    result = session_class->validate_user_props(
-      priv->session,
-      (const GParameter*)user_props->data,
-      user_props->len,
-      user,
-      error
-    );
-
-    if(result == FALSE)
-      return NULL;
-
     g_object_freeze_notify(G_OBJECT(user));
 
     /* Set properties on already existing user object. */
@@ -410,6 +478,12 @@ infd_session_proxy_perform_user_join(InfdSessionProxy* proxy,
   else
   {
     priv->local_users = g_slist_prepend(priv->local_users, user);
+
+    if(priv->idle == TRUE)
+    {
+      priv->idle = FALSE;
+      g_object_notify(G_OBJECT(proxy), "idle");
+    }
   }
 
   return user;
@@ -557,6 +631,7 @@ infd_session_proxy_synchronization_complete_cb(InfSession* session,
 
   /* Set idle if no more synchronizations are running */
   if(!priv->idle && priv->subscriptions == NULL &&
+     priv->local_users == NULL &&
      !inf_session_has_synchronizations(session))
   {
     priv->idle = TRUE;
@@ -625,8 +700,6 @@ infd_session_proxy_session_close_cb(InfSession* session,
   InfdSessionProxy* proxy;
   InfdSessionProxyPrivate* priv;
   InfdSessionProxySubscription* subscription;
-  InfSessionSyncStatus status;
-  xmlNodePtr xml;
 
   proxy = INFD_SESSION_PROXY(user_data);
   priv = INFD_SESSION_PROXY_PRIVATE(proxy);
@@ -641,39 +714,12 @@ infd_session_proxy_session_close_cb(InfSession* session,
   {
     subscription = (InfdSessionProxySubscription*)priv->subscriptions->data;
 
-    status = inf_session_get_synchronization_status(
-      priv->session,
-      subscription->connection
-    );
-
-    /* If synchronization is still in progress, the default handler of
-     * InfSession will cancel the synchronization in which case we do
-     * not need to send an extra session-close message. */
-
-    /* We send session_close when we are in AWAITING_ACK status. In
-     * AWAITING_ACK status we cannot cancel the synchronization anymore
-     * because everything has already been sent out. Therefore the client
-     * will eventuelly get in RUNNING state when he receives this message,
-     * and process it correctly. */
-    if(status != INF_SESSION_SYNC_IN_PROGRESS)
-    {
-      xml = xmlNewNode(NULL, (const xmlChar*)"session-close");
-
-      inf_communication_group_send_message(
-        INF_COMMUNICATION_GROUP(priv->subscription_group),
-        subscription->connection,
-        xml
-      );
-    }
-
     /* Note that this does not call our signal handler because we already
      * disconnected it. This way, we make sure not to send user status updates
      * which would be pointless since we are closing the group anyway. */
-    inf_communication_hosted_group_remove_member(
-      priv->subscription_group,
-      subscription->connection
-    );
+    infd_session_proxy_unsubscribe(proxy, subscription->connection);
 
+    /* However, this means we need to emit the unsubscribe signal ourselves */
     g_signal_emit(
       proxy,
       session_proxy_signals[REMOVE_SUBSCRIPTION],
@@ -682,6 +728,16 @@ infd_session_proxy_session_close_cb(InfSession* session,
     );
   }
 
+  /* Set local users to unavailable */
+  while(priv->local_users != NULL)
+  {
+    g_object_set(
+      G_OBJECT(priv->local_users->data),
+      "status", INF_USER_UNAVAILABLE,
+      NULL
+    );
+  }
+
   g_object_unref(priv->subscription_group);
   priv->subscription_group = NULL;
 }
@@ -691,15 +747,12 @@ infd_session_proxy_session_close_cb(InfSession* session,
  */
 
 static void
-infd_session_proxy_init(GTypeInstance* instance,
-                        gpointer g_class)
+infd_session_proxy_init(InfdSessionProxy* session_proxy)
 {
-  InfdSessionProxy* session_proxy;
   InfdSessionProxyPrivate* priv;
-
-  session_proxy = INFD_SESSION_PROXY(instance);
   priv = INFD_SESSION_PROXY_PRIVATE(session_proxy);
 
+  priv->io = NULL;
   priv->subscriptions = NULL;
   priv->subscription_group = NULL;
   priv->user_id_counter = 1;
@@ -707,30 +760,26 @@ infd_session_proxy_init(GTypeInstance* instance,
   priv->idle = TRUE;
 }
 
-static GObject*
-infd_session_proxy_constructor(GType type,
-                               guint n_construct_properties,
-                               GObjectConstructParam* construct_properties)
+static void
+infd_session_proxy_constructed(GObject* object)
 {
-  GObject* object;
   InfdSessionProxy* session_proxy;
   InfdSessionProxyPrivate* priv;
 
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_construct_properties,
-    construct_properties
-  );
-
   session_proxy = INFD_SESSION_PROXY(object);
   priv = INFD_SESSION_PROXY_PRIVATE(session_proxy);
 
+  G_OBJECT_CLASS(infd_session_proxy_parent_class)->constructed(object);
+
   g_assert(priv->subscription_group != NULL);
   g_assert(priv->session != NULL);
 
   /* Set unidle when session is currently being synchronized */
-  if(inf_session_get_status(priv->session) == INF_SESSION_SYNCHRONIZING)
+  if(inf_session_get_status(priv->session) == INF_SESSION_SYNCHRONIZING ||
+     priv->local_users != NULL)
+  {
     priv->idle = FALSE;
+  }
 
   /* TODO: We could perhaps optimize by only setting the subscription
    * group when there are subscribed connections. */
@@ -738,8 +787,6 @@ infd_session_proxy_constructor(GType type,
     priv->session,
     INF_COMMUNICATION_GROUP(priv->subscription_group)
   );
-
-  return object;
 }
 
 static void
@@ -807,7 +854,12 @@ infd_session_proxy_dispose(GObject* object)
   g_assert(priv->subscription_group == NULL);
   g_assert(priv->subscriptions == NULL);
 
+  g_object_unref(priv->io);
+  priv->io = NULL;
+
   g_object_unref(manager);
+
+  G_OBJECT_CLASS(infd_session_proxy_parent_class)->dispose(object);
 }
 
 static void
@@ -838,6 +890,10 @@ infd_session_proxy_set_property(GObject* object,
 
   switch(prop_id)
   {
+  case PROP_IO:
+    g_assert(priv->io == NULL); /* construct only */
+    priv->io = INF_IO(g_value_dup_object(value));
+    break;
   case PROP_SESSION:
     g_assert(priv->session == NULL); /* construct only */
     priv->session = INF_SESSION(g_value_dup_object(value));
@@ -932,6 +988,9 @@ infd_session_proxy_get_property(GObject* object,
 
   switch(prop_id)
   {
+  case PROP_IO:
+    g_value_set_object(value, priv->io);
+    break;
   case PROP_SESSION:
     g_value_set_object(value, priv->session);
     break;
@@ -965,7 +1024,7 @@ infd_session_proxy_add_subscription(InfdSessionProxy* proxy,
   subscription = infd_session_proxy_subscription_new(connection, seq_id);
   priv->subscriptions = g_slist_prepend(priv->subscriptions, subscription);
 
-  if(priv->idle)
+  if(priv->idle == TRUE)
   {
     priv->idle = FALSE;
     g_object_notify(G_OBJECT(proxy), "idle");
@@ -1001,14 +1060,23 @@ infd_session_proxy_remove_subscription(InfdSessionProxy* proxy,
   priv->subscriptions = g_slist_remove(priv->subscriptions, subscr);
   infd_session_proxy_subscription_free(subscr);
 
-  if(!priv->idle && priv->subscriptions == NULL &&
-     !inf_session_has_synchronizations(priv->session))
+  if(priv->idle == FALSE && infd_session_proxy_check_idle(proxy) == TRUE)
   {
     priv->idle = TRUE;
     g_object_notify(G_OBJECT(proxy), "idle");
   }
 }
 
+static gboolean
+infd_session_proxy_reject_user_join(InfdSessionProxy* proxy,
+                                    InfXmlConnection* connection,
+                                    const GArray* user_properties,
+                                    InfUser* user_rejoin)
+{
+  /* Allow user join by default */
+  return FALSE;
+}
+
 /*
  * Message handling.
  */
@@ -1128,8 +1196,7 @@ infd_session_proxy_communication_object_enqueued(InfCommunicationObject* obj,
 static InfCommunicationScope
 infd_session_proxy_communication_object_received(InfCommunicationObject* obj,
                                                  InfXmlConnection* connection,
-                                                 xmlNodePtr node,
-                                                 GError** error)
+                                                 xmlNodePtr node)
 {
   InfdSessionProxy* proxy;
   InfdSessionProxyPrivate* priv;
@@ -1152,8 +1219,7 @@ infd_session_proxy_communication_object_received(InfCommunicationObject* obj,
     return inf_communication_object_received(
       INF_COMMUNICATION_OBJECT(priv->session),
       connection,
-      node,
-      error
+      node
     );
   }
   else
@@ -1182,15 +1248,14 @@ infd_session_proxy_communication_object_received(InfCommunicationObject* obj,
       return inf_communication_object_received(
         INF_COMMUNICATION_OBJECT(priv->session),
         connection,
-        node,
-        error
+        node
       );
     }
   }
 
   if(local_error != NULL)
   {
-    if(!infd_session_proxy_make_seq(proxy, connection, node, &seq, error))
+    if(!infd_session_proxy_make_seq(proxy, connection, node, &seq, NULL))
       seq = NULL;
 
     /* Only send request-failed when it was a proxy-related request */
@@ -1214,39 +1279,112 @@ infd_session_proxy_communication_object_received(InfCommunicationObject* obj,
   return INF_COMMUNICATION_SCOPE_PTP;
 }
 
+/*
+ * InfSessionProxy implementation
+ */
+
+static InfRequest*
+infd_session_proxy_session_proxy_join_user(InfSessionProxy* proxy,
+                                           guint n_params,
+                                           const GParameter* params,
+                                           InfRequestFunc func,
+                                           gpointer user_data)
+{
+  InfdRequest* request;
+  GArray* array;
+
+  guint i;
+  GParameter param;
+
+  GError* error;
+  InfUser* user;
+
+  g_return_val_if_fail(INFD_IS_SESSION_PROXY(proxy), NULL);
+
+  request = g_object_new(
+    INFD_TYPE_REQUEST,
+    "type", "user-join",
+    NULL
+  );
+
+  if(func != NULL)
+  {
+    g_signal_connect_after(
+      G_OBJECT(request),
+      "finished",
+      G_CALLBACK(func),
+      user_data
+    );
+  }
+
+  array = g_array_sized_new(
+    FALSE,
+    FALSE,
+    sizeof(GParameter),
+    n_params
+  );
+
+  g_array_append_vals(array, params, n_params);
+
+  error = NULL;
+  user = infd_session_proxy_perform_user_join(
+    INFD_SESSION_PROXY(proxy),
+    NULL,
+    NULL,
+    array,
+    &error
+  );
+
+  /* Remove only those parameters that were added by the
+   * infd_session_proxy_perform_user_join function */
+  for(i = n_params; i < array->len; ++i)
+    g_value_unset(&g_array_index(array, GParameter, i).value);
+  g_array_free(array, TRUE);
+
+  if(error != NULL)
+  {
+    inf_request_fail(INF_REQUEST(request), error);
+  }
+  else
+  {
+    inf_request_finish(
+      INF_REQUEST(request),
+      inf_request_result_make_join_user(proxy, user)
+    );
+  }
+
+  if(error) g_error_free(error);
+  g_object_unref(request);
+  return NULL;
+}
+
 /*
  * GType registration.
  */
 
 static void
-infd_session_proxy_class_init(gpointer g_class,
-                              gpointer class_data)
+infd_session_proxy_class_init(InfdSessionProxyClass* proxy_class)
 {
   GObjectClass* object_class;
-  InfdSessionProxyClass* proxy_class;
+  object_class = G_OBJECT_CLASS(proxy_class);
 
-  object_class = G_OBJECT_CLASS(g_class);
-  proxy_class = INFD_SESSION_PROXY_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfdSessionProxyPrivate));
-
-  object_class->constructor = infd_session_proxy_constructor;
+  object_class->constructed = infd_session_proxy_constructed;
   object_class->dispose = infd_session_proxy_dispose;
   object_class->set_property = infd_session_proxy_set_property;
   object_class->get_property = infd_session_proxy_get_property;
 
   proxy_class->add_subscription = infd_session_proxy_add_subscription;
   proxy_class->remove_subscription = infd_session_proxy_remove_subscription;
+  proxy_class->reject_user_join = infd_session_proxy_reject_user_join;
 
   g_object_class_install_property(
     object_class,
-    PROP_SESSION,
+    PROP_IO,
     g_param_spec_object(
-      "session",
-      "Session",
-      "The underlying session",
-      INF_TYPE_SESSION,
+      "io",
+      "Io",
+      "The InfIo object for scheduling events",
+      INF_TYPE_IO,
       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY
     )
   );
@@ -1276,6 +1414,8 @@ infd_session_proxy_class_init(gpointer g_class,
     )
   );
 
+  g_object_class_override_property(object_class, PROP_SESSION, "session");
+
   /**
    * InfdSessionProxy::add-subscription:
    * @proxy: The #InfdSessionProxy emitting the signal.
@@ -1291,7 +1431,7 @@ infd_session_proxy_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfdSessionProxyClass, add_subscription),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT_UINT,
+    NULL,
     G_TYPE_NONE,
     2,
     INF_TYPE_XML_CONNECTION,
@@ -1312,133 +1452,69 @@ infd_session_proxy_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfdSessionProxyClass, remove_subscription),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_XML_CONNECTION
   );
+
+  /**
+   * InfdSessionProxy::reject-user-join:
+   * @proxy: The #InfdSessionProxy emitting the signal.
+   * @connection: A subscribed #InfXmlConnection requesting the user join.
+   * @user_properties: An array with the properties for the new user.
+   * @rejoin_user: The existing unavailable user that is being rejoined, or
+   * %NULL.
+   *
+   * This signal is emitted before every remote user join. The signal handler
+   * can return %TRUE in which case the #InfdSessionProxy does not allow the
+   * user join with %INF_REQUEST_ERROR_NOT_AUTHORIZED error. If there is more
+   * than one signal handler, then if one of them returns %TRUE the user
+   * join is rejected.
+   *
+   * The @user_properties parameter is a #GArray of #GParameter values. It
+   * contains the construct properties for the #InfUser object that would be
+   * created if the user join is not rejected. It must not be modified, but
+   * it can be used to make the decision whether to reject the user join or
+   * not dependent on the parameters, such as allowing the user join only if
+   * the user has a predefined name. The function
+   * inf_session_lookup_user_property() can be used to look up a named
+   * parameter in the array.
+   */
+  session_proxy_signals[REJECT_USER_JOIN] = g_signal_new(
+    "reject-user-join",
+    G_OBJECT_CLASS_TYPE(object_class),
+    G_SIGNAL_RUN_LAST,
+    G_STRUCT_OFFSET(InfdSessionProxyClass, reject_user_join),
+    g_signal_accumulator_true_handled, NULL,
+    NULL,
+    G_TYPE_BOOLEAN,
+    3,
+    INF_TYPE_XML_CONNECTION,
+    G_TYPE_ARRAY,
+    INF_TYPE_USER
+  );
 }
 
 static void
-infd_session_proxy_communication_object_init(gpointer g_iface,
-                                             gpointer iface_data)
+infd_session_proxy_communication_object_iface_init(
+  InfCommunicationObjectInterface* iface)
 {
-  InfCommunicationObjectIface* iface;
-  iface = (InfCommunicationObjectIface*)g_iface;
-
   iface->sent = infd_session_proxy_communication_object_sent;
   iface->enqueued = infd_session_proxy_communication_object_enqueued;
   iface->received = infd_session_proxy_communication_object_received;
 }
 
-GType
-infd_session_proxy_get_type(void)
+static void
+infd_session_proxy_session_proxy_iface_init(InfSessionProxyInterface* iface)
 {
-  static GType session_proxy_type = 0;
-
-  if(!session_proxy_type)
-  {
-    static const GTypeInfo session_proxy_type_info = {
-      sizeof(InfdSessionProxyClass),    /* class_size */
-      NULL,                             /* base_init */
-      NULL,                             /* base_finalize */
-      infd_session_proxy_class_init,    /* class_init */
-      NULL,                             /* class_finalize */
-      NULL,                             /* class_data */
-      sizeof(InfdSessionProxy),         /* instance_size */
-      0,                                /* n_preallocs */
-      infd_session_proxy_init,          /* instance_init */
-      NULL                              /* value_table */
-    };
-
-    static const GInterfaceInfo communication_object_info = {
-      infd_session_proxy_communication_object_init,
-      NULL,
-      NULL
-    };
-
-    session_proxy_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfdSessionProxy",
-      &session_proxy_type_info,
-      0
-    );
-
-    g_type_add_interface_static(
-      session_proxy_type,
-      INF_COMMUNICATION_TYPE_OBJECT,
-      &communication_object_info
-    );
-  }
-
-  return session_proxy_type;
+  iface->join_user = infd_session_proxy_session_proxy_join_user;
 }
 
 /*
  * Public API.
  */
 
-/**
- * infd_session_proxy_get_session:
- * @proxy: A #InfdSessionProxy.
- *
- * Returns the session proxied by @proxy. Returns %NULL if the session was
- * closed.
- *
- * Return Value: A #InfSession, or %NULL.
- **/
-InfSession*
-infd_session_proxy_get_session(InfdSessionProxy* proxy)
-{
-  g_return_val_if_fail(INFD_IS_SESSION_PROXY(proxy), NULL);
-  return INFD_SESSION_PROXY_PRIVATE(proxy)->session;
-}
-
-/**
- * infd_session_proxy_add_user:
- * @proxy: A #InfdSessionProxy.
- * @params: Construction properties for the #InfUser (or derived) object.
- * @n_params: Number of parameters.
- * @error: Location to store error information.
- *
- * Adds a local user to @proxy's session. @params must not contain the
- * 'id' property because it will be choosen by the proxy. Also, if the 'name'
- * property is already in use by an existing, but unavailable user, this user 
- * will be re-used.
- *
- * Return Value: The #InfUser that has been added, or %NULL in case of an
- * error.
- **/
-InfUser*
-infd_session_proxy_add_user(InfdSessionProxy* proxy,
-                            const GParameter* params,
-                            guint n_params,
-                            GError** error)
-{
-  InfUser* user;
-  GArray* array;
-
-  g_return_val_if_fail(INFD_IS_SESSION_PROXY(proxy), NULL);
-
-  /* TODO: Make sure values added by infd_session_proxy_perform_user_join are
-   * released, for example by inserting copies into the array, and freeing
-   * the values after the call. */
-  array = g_array_sized_new(FALSE, FALSE, sizeof(GParameter), n_params + 2);
-  g_array_append_vals(array, params, n_params);
-
-  user = infd_session_proxy_perform_user_join(
-    proxy,
-    NULL,
-    NULL,
-    array,
-    error
-  );
-
-  g_array_free(array, TRUE);
-
-  return user;
-}
-
 /**
  * infd_session_proxy_subscribe_to:
  * @proxy: A #InfdSessionProxy.
@@ -1531,6 +1607,76 @@ infd_session_proxy_subscribe_to(InfdSessionProxy* proxy,
   }
 }
 
+/**
+ * infd_session_proxy_unsubscribe:
+ * @proxy: A #InfdSessionProxy.
+ * @connection: The #InfXmlConnection to unsubscribe.
+ *
+ * Unsubscribes a subscribed connection from @proxy's session. This will
+ * prevent all users joined via @connection to continue modifying the
+ * session's buffer, and it will cancel ongoing synchronization to
+ * @connection, if not yet finished.
+ */
+void
+infd_session_proxy_unsubscribe(InfdSessionProxy* proxy,
+                               InfXmlConnection* connection)
+{
+  InfdSessionProxyPrivate* priv;
+  InfdSessionProxySubscription* subscription;
+  InfSessionSyncStatus status;
+  xmlNodePtr xml;
+
+  g_return_if_fail(INFD_IS_SESSION_PROXY(proxy));
+  g_return_if_fail(INF_IS_XML_CONNECTION(connection));
+
+  priv = INFD_SESSION_PROXY_PRIVATE(proxy);
+
+  /* TODO: Can we support the SYNCHRONIZING case? In that case the session
+   * will probably end up closed... */
+  g_assert(inf_session_get_status(priv->session) == INF_SESSION_RUNNING);
+
+  subscription = infd_session_proxy_find_subscription(proxy, connection);
+  g_return_if_fail(subscription != NULL);
+
+  status = inf_session_get_synchronization_status(
+    priv->session,
+    subscription->connection
+  );
+
+  /* If synchronization is still in progress, the default handler of
+   * InfSession will cancel the synchronization in which case we do
+   * not need to send an extra session-close message. */
+
+  /* We send session_close when we are in AWAITING_ACK status. In
+   * AWAITING_ACK status we cannot cancel the synchronization anymore
+   * because everything has already been sent out. Therefore the client
+   * will eventuelly get in RUNNING state when it receives this message,
+   * and process it correctly. */
+  if(status != INF_SESSION_SYNC_IN_PROGRESS)
+  {
+    xml = xmlNewNode(NULL, (const xmlChar*)"session-close");
+
+    inf_communication_group_send_message(
+      INF_COMMUNICATION_GROUP(priv->subscription_group),
+      subscription->connection,
+      xml
+    );
+  }
+  else
+  {
+    /* In case we are synchronizing the client */
+    inf_session_cancel_synchronization(
+      priv->session,
+      subscription->connection
+    );
+  }
+
+  inf_communication_hosted_group_remove_member(
+    priv->subscription_group,
+    subscription->connection
+  );
+}
+
 /**
  * infd_session_proxy_has_subscriptions:
  * @proxy: A #InfdSessionProxy.
