@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,6 @@
 #include <libinfinity/server/infd-tcp-server.h>
 #include <libinfinity/server/infd-xml-server.h>
 #include <libinfinity/common/inf-xmpp-connection.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-signals.h>
 
 /* Some Windows header #defines ERROR for no good */
@@ -74,9 +73,13 @@ enum {
 
 #define INFD_XMPP_SERVER_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INFD_TYPE_XMPP_SERVER, InfdXmppServerPrivate))
 
-static GObjectClass* parent_class;
 static guint xmpp_server_signals[LAST_SIGNAL];
 
+static void infd_xmpp_server_xml_server_iface_init(InfdXmlServerInterface* iface);
+G_DEFINE_TYPE_WITH_CODE(InfdXmppServer, infd_xmpp_server, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfdXmppServer)
+  G_IMPLEMENT_INTERFACE(INFD_TYPE_XML_SERVER, infd_xmpp_server_xml_server_iface_init))
+
 static void
 infd_xmpp_server_new_connection_cb(InfdTcpServer* tcp_server,
                                    InfTcpConnection* tcp_connection,
@@ -232,7 +235,8 @@ infd_xmpp_server_setup_own_sasl_context(InfdXmppServer* xmpp)
       inf_sasl_context_set_callback(
         priv->sasl_context,
         infd_xmpp_server_sasl_cb,
-        xmpp
+        xmpp,
+        NULL
       );
 
       g_object_notify(G_OBJECT(xmpp), "sasl-context");
@@ -262,13 +266,13 @@ infd_xmpp_server_set_tcp(InfdXmppServer* xmpp,
     /* TODO: Make sure there are no connections with sasl_own_context out
      * there anymore because otherwise the SASL callback might access an
      * invalid InfdServer pointer once we get finalized. */
-#if 0
+    /* TODO: Or, give data associated with the SASL context its own
+     * life cycle. Should be easy given that it has a destroy notify. */
     if(priv->sasl_own_context != NULL)
     {
       inf_sasl_context_unref(priv->sasl_own_context);
       priv->sasl_own_context = NULL;
     }
-#endif
 
     inf_signal_handlers_disconnect_by_func(
       G_OBJECT(priv->tcp),
@@ -340,13 +344,9 @@ infd_xmpp_server_set_tcp(InfdXmppServer* xmpp,
 }
 
 static void
-infd_xmpp_server_init(GTypeInstance* instance,
-                      gpointer g_class)
+infd_xmpp_server_init(InfdXmppServer* xmpp)
 {
-  InfdXmppServer* xmpp;
   InfdXmppServerPrivate* priv;
-
-  xmpp = INFD_XMPP_SERVER(instance);
   priv = INFD_XMPP_SERVER_PRIVATE(xmpp);
 
   priv->tcp = NULL;
@@ -360,31 +360,6 @@ infd_xmpp_server_init(GTypeInstance* instance,
   priv->sasl_mechanisms = NULL;
 }
 
-static GObject*
-infd_xmpp_server_constructor(GType type,
-                             guint n_construct_properties,
-                             GObjectConstructParam* construct_properties)
-{
-  InfdXmppServerPrivate* priv;
-  GObject* obj;
-
-  obj = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_construct_properties,
-    construct_properties
-  );
-
-  priv = INFD_XMPP_SERVER_PRIVATE(obj);
-  g_assert(priv->tcp != NULL);
-
-  g_assert(
-    priv->security_policy == INF_XMPP_CONNECTION_SECURITY_ONLY_UNSECURED ||
-    priv->tls_creds != NULL
-  );
-
-  return obj;
-}
-
 static void
 infd_xmpp_server_dispose(GObject* object)
 {
@@ -417,7 +392,7 @@ infd_xmpp_server_dispose(GObject* object)
     priv->tls_creds = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(infd_xmpp_server_parent_class)->dispose(object);
 }
 
 static void
@@ -432,7 +407,7 @@ infd_xmpp_server_finalize(GObject* object)
   g_free(priv->local_hostname);
   g_free(priv->sasl_mechanisms);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(infd_xmpp_server_parent_class)->finalize(object);
 }
 
 static void
@@ -568,19 +543,11 @@ infd_xmpp_server_xml_server_close(InfdXmlServer* xml)
 }
 
 static void
-infd_xmpp_server_class_init(gpointer g_class,
-                            gpointer class_data)
+infd_xmpp_server_class_init(InfdXmppServerClass* xmpp_class)
 {
   GObjectClass* object_class;
-  InfdXmppServerClass* xmpp_class;
+  object_class = G_OBJECT_CLASS(xmpp_class);
 
-  object_class = G_OBJECT_CLASS(g_class);
-  xmpp_class = INFD_XMPP_SERVER_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfdXmppServerPrivate));
-
-  object_class->constructor = infd_xmpp_server_constructor;
   object_class->dispose = infd_xmpp_server_dispose;
   object_class->finalize = infd_xmpp_server_finalize;
   object_class->set_property = infd_xmpp_server_set_property;
@@ -669,68 +636,21 @@ infd_xmpp_server_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfdXmppServerClass, error),
     NULL, NULL,
-    inf_marshal_VOID__POINTER,
+    g_cclosure_marshal_VOID__BOXED,
     G_TYPE_NONE,
     1,
-    G_TYPE_POINTER /* actually a GError* */
+    G_TYPE_ERROR
   );
 }
 
 static void
-infd_xmpp_server_xml_server_init(gpointer g_iface,
-                                 gpointer iface_data)
+infd_xmpp_server_xml_server_iface_init(InfdXmlServerInterface* iface)
 {
-  InfdXmlServerIface* iface;
-  iface = (InfdXmlServerIface*)g_iface;
-
   iface->close = infd_xmpp_server_xml_server_close;
 }
 
-GType
-infd_xmpp_server_get_type(void)
-{
-  static GType xmpp_server_type = 0;
-
-  if(!xmpp_server_type)
-  {
-    static const GTypeInfo xmpp_server_type_info = {
-      sizeof(InfdXmppServerClass),   /* class_size */
-      NULL,                          /* base_init */
-      NULL,                          /* base_finalize */
-      infd_xmpp_server_class_init,   /* class_init */
-      NULL,                          /* class_finalize */
-      NULL,                          /* class_data */
-      sizeof(InfdXmppServer),        /* instance_size */
-      0,                             /* n_preallocs */
-      infd_xmpp_server_init,         /* instance_init */
-      NULL                           /* value_table */
-    };
-
-    static const GInterfaceInfo xml_server_info = {
-      infd_xmpp_server_xml_server_init,
-      NULL,
-      NULL
-    };
-
-    xmpp_server_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfdXmppServer",
-      &xmpp_server_type_info,
-      0
-    );
-
-    g_type_add_interface_static(
-      xmpp_server_type,
-      INFD_TYPE_XML_SERVER,
-      &xml_server_info
-    );
-  }
-
-  return xmpp_server_type;
-}
-
 /**
- * infd_xmpp_server_new:
+ * infd_xmpp_server_new: (constructor)
  * @tcp: A #InfdTcpServer.
  * @policy: The initial security policy.
  * @creds: Certificate credentials used to secure any communication.
@@ -756,7 +676,7 @@ infd_xmpp_server_get_type(void)
  * @sasl_mechanisms is %NULL, then all available mechanims will be offered.
  * If @sasl_context is %NULL, then this parameter is ignored.
  *
- * Return Value: A new #InfdXmppServer.
+ * Return Value: (transfer full): A new #InfdXmppServer.
  **/
 InfdXmppServer*
 infd_xmpp_server_new(InfdTcpServer* tcp,
