@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,18 +17,59 @@
  * MA 02110-1301, USA.
  */
 
+/**
+ * SECTION:inf-text-chunk
+ * @title: InfTextChunk
+ * @short_description: A chunk of text written by various authors
+ * @include: libinftext/inf-text-chunk.h
+ * @see_also: #InfTextBuffer
+ * @stability: Unstable
+ *
+ * #InfTextChunk represents a chunk of text where different parts of it can
+ * be written by different authors. The #InfTextChunk API can be used like a
+ * normal string API, except that apart from the text itself it also stores
+ * the author information.
+ *
+ * An #InfTextChunk is made up of segments, where each segment represents a
+ * contiguous piece of text which is written by the same user. The
+ * #InfTextChunkIter functionality can be used to iterate over the segments
+ * of a chunk.
+ *
+ * The #InfTextChunk API works with characters, not bytes, i.e. all offsets
+ * are given in number of characters. This ensures that unicode strings
+ * cannot be torn apart in the middle of a multibyte sequence. The encoding
+ * of an #InfTextChunk is not fixed, but it can be freely chosen.
+ * #InfTextChunk then uses iconv to convert between bytes and character
+ * offsets where necessary. For a small set of selected encodings which are
+ * very popular, most notably UTF-8, there exist more optimized code paths to
+ * do the conversion.
+ */
+
 #include <libinftext/inf-text-chunk.h>
 #include <libinfinity/common/inf-xml-util.h>
 
 #include <string.h>
 
+G_DEFINE_BOXED_TYPE(InfTextChunkIter, inf_text_chunk_iter, inf_text_chunk_iter_copy, inf_text_chunk_iter_free)
+G_DEFINE_BOXED_TYPE(InfTextChunk, inf_text_chunk, inf_text_chunk_copy, inf_text_chunk_free)
+
 /* Don't check integrity in stable releases */
 /*#define CHUNK_CHECK_INTEGRITY*/
 
+typedef struct _InfTextChunkPath InfTextChunkPath;
+struct _InfTextChunkPath {
+  gsize (*get_byte_index)(InfTextChunk* chunk,
+                          gchar* text,
+                          gsize bytes,
+                          guint offset);
+};
+
 struct _InfTextChunk {
   GSequence* segments;
   guint length; /* in characters */
   GQuark encoding;
+
+  const InfTextChunkPath* path;
 };
 
 typedef struct _InfTextChunkSegment InfTextChunkSegment;
@@ -42,6 +83,79 @@ struct _InfTextChunkSegment {
   guint offset; /* absolute to chunk begin in characters, sort criteria */
 };
 
+/*
+ * get_byte_index paths
+ */
+
+gsize inf_text_chunk_get_byte_index_utf8(InfTextChunk* self,
+                                         gchar* text,
+                                         gsize bytes,
+                                         guint offset)
+{
+#ifdef CHUNK_CHECK_INTEGRITY
+  g_assert(offset <= g_utf8_strlen(text, bytes));
+#endif
+
+  return g_utf8_offset_to_pointer(text, offset) - text;
+}
+
+gsize inf_text_chunk_get_byte_index_iconv(InfTextChunk* self,
+                                          gchar* text,
+                                          gsize bytes,
+                                          guint offset)
+{
+  /* We convert the segment's text into UCS-4, character by character.
+   * This assumes every UCS-4 character is 4 bytes in length */
+
+  /* This is quite inefficient, but a general-purpose solution. It looks like
+   * libicu has a function, UCharIteratorMove, which would allow us to move
+   * directly N characters ahead and get the new byte index.
+   * TODO: We could profile it, and if it brings a benefit, we could use it,
+   * maybe with an option to fall back to iconv at configure time */
+
+  GIConv cd;
+  gchar buffer[4];
+
+  gchar* inbuf;
+  gchar* outbuf;
+  gsize inlen;
+  gsize outlen;
+  guint count;
+  gsize result;
+
+  cd = g_iconv_open("UCS-4", g_quark_to_string(self->encoding));
+  g_assert(cd != (GIConv)-1);
+
+  inbuf = text;
+  inlen = bytes;
+
+  for(count = 0; count < offset; ++count)
+  {
+    g_assert(inlen > 0);
+
+    outbuf = buffer;
+    outlen = 4;
+
+    result = g_iconv(cd, &inbuf, &inlen, &outbuf, &outlen);
+    g_assert(result == (size_t)(-1)); /* errno == E2BIG */
+  }
+
+  g_iconv_close(cd);
+  return bytes - inlen;
+}
+
+const InfTextChunkPath INF_TEXT_CHUNK_PATH_UTF8 = {
+  inf_text_chunk_get_byte_index_utf8
+};
+
+const InfTextChunkPath INF_TEXT_CHUNK_PATH_ICONV = {
+  inf_text_chunk_get_byte_index_iconv
+};
+
+/*
+ * Helper functions
+ */
+
 static void
 inf_text_chunk_segment_free(InfTextChunkSegment* segment)
 {
@@ -70,6 +184,33 @@ inf_text_chunk_segment_cmp(gconstpointer first,
     return 1;
 }
 
+/* This is a special comparison function which is only used
+ * in inf_text_chunk_get_segment. It makes sure that when two segments
+ * have the same offset, the second one gets sorted behind. This ensures
+ * that the first segment in the sequence is never returned, since it
+ * always has offset 0, which is a precondition for the algorithm
+ * in inf_text_chunk_get_segment. See also libinfinity github issue #10. */
+static int
+inf_text_chunk_segment_cmp_for_get_segment(gconstpointer first,
+                                           gconstpointer second,
+                                           gpointer userdata)
+{
+  const InfTextChunkSegment* first_segment;
+  const InfTextChunkSegment* second_segment;
+
+  first_segment  = (const InfTextChunkSegment*)first;
+  second_segment = (const InfTextChunkSegment*)second;
+
+  g_return_val_if_fail(second != NULL && first != NULL, 0);
+
+  if (first_segment->offset < second_segment->offset)
+    return -1;
+  else if (first_segment->offset == second_segment->offset)
+    return -1;
+  else
+    return 1;
+}
+
 static guint
 inf_text_chunk_next_offset(InfTextChunk* self,
                            GSequenceIter* iter)
@@ -127,16 +268,6 @@ inf_text_chunk_get_segment(InfTextChunk* self,
   InfTextChunkSegment key;
   GSequenceIter* iter;
 
-  GIConv cd;
-  gchar buffer[4];
-
-  gchar* inbuf;
-  gchar* outbuf;
-  gsize inlen;
-  gsize outlen;
-  guint count;
-  gsize result;
-
   g_assert(pos <= self->length);
 
   /* only offset is relevant for lookup */
@@ -146,7 +277,7 @@ inf_text_chunk_get_segment(InfTextChunk* self,
   iter = g_sequence_search(
     self->segments,
     &key,
-    inf_text_chunk_segment_cmp,
+    inf_text_chunk_segment_cmp_for_get_segment,
     NULL
   );
 
@@ -174,67 +305,69 @@ inf_text_chunk_get_segment(InfTextChunk* self,
       }
       else
       {
-        /* We convert the segment's text into UCS-4, character by character.
-         * This assumes every UCS-4 character is 4 bytes in length */
-
-        /* TODO: Can we use some faster glib UTF-8 functions in case
-         * self->encoding is UTF-8? */
-        cd = g_iconv_open("UCS-4", g_quark_to_string(self->encoding));
-        g_assert(cd != (GIConv)-1);
-        
-        inbuf = found->text;
-        inlen = found->length;
-
-        for(count = 0; count < pos - found->offset; ++ count)
-        {
-          g_assert(inlen > 0);
-
-          outbuf = buffer;
-          outlen = 4;
-
-          result = g_iconv(cd, &inbuf, &inlen, &outbuf, &outlen);
-          g_assert(result == (size_t)(-1)); /* errno == E2BIG */
-        }
-
-        g_iconv_close(cd);
-        *index = found->length - inlen;
+        *index = self->path->get_byte_index(
+          self,
+          found->text,
+          found->length,
+          pos - found->offset
+        );
       }
     }
   }
   else
   {
-    if(*index)
+    if(index != NULL)
       *index = 0;
   }
   
   return iter;
 }
 
-GType
-inf_text_chunk_get_type(void)
+/*
+ * Public API
+ */
+
+/**
+ * inf_text_chunk_iter_copy:
+ * @iter: A #InfTextChunkIter.
+ *
+ * Makes a dynamically-allocated copy of @iter. This is typically not needed
+ * because #InfTextChunkIter can be copied by value, however it might be
+ * useful to language bindings.
+ *
+ * Returns: (transfer full): A copy of @iter. Free with
+ * inf_text_chunk_iter_free().
+ */
+InfTextChunkIter*
+inf_text_chunk_iter_copy(const InfTextChunkIter* iter)
 {
-  static GType chunk_type = 0;
-
-  if(!chunk_type)
-  {
-    chunk_type = g_boxed_type_register_static(
-      "InfTextChunk",
-      (GBoxedCopyFunc)inf_text_chunk_copy,
-      (GBoxedFreeFunc)inf_text_chunk_free
-    );
-  }
-
-  return chunk_type;
+  InfTextChunkIter* new_iter;
+  new_iter = g_slice_new(InfTextChunkIter);
+  *new_iter = *iter;
+  return new_iter;
 }
 
 /**
- * inf_text_chunk_new:
+ * inf_text_chunk_iter_free:
+ * @iter: The #InfTextChunkIter to free.
+ *
+ * Frees all resources allocated with inf_text_chunk_iter_copy(). Must not
+ * be used with stack-allocated #InfTextChunkIter<!-- -->s.
+ */
+void
+inf_text_chunk_iter_free(InfTextChunkIter* iter)
+{
+  g_slice_free(InfTextChunkIter, iter);
+}
+
+/**
+ * inf_text_chunk_new: (constructor)
  * @encoding: A content encoding, such as "UTF-8" or "LATIN1".
  *
  * Creates a new #InfTextChunk with no initial content that holds text
  * in the given encoding. TODO: Allow binary data with %NULL encoding.
  *
- * Return Value: A new #InfTextChunk.
+ * Returns: (transfer full): A new #InfTextChunk.
  **/
 InfTextChunk*
 inf_text_chunk_new(const gchar* encoding)
@@ -248,6 +381,11 @@ inf_text_chunk_new(const gchar* encoding)
   chunk->length = 0;
   chunk->encoding = g_quark_from_string(encoding);
 
+  if(chunk->encoding == g_quark_from_static_string("UTF-8"))
+    chunk->path = &INF_TEXT_CHUNK_PATH_UTF8;
+  else
+    chunk->path = &INF_TEXT_CHUNK_PATH_ICONV;
+
   return chunk;
 }
 
@@ -257,7 +395,7 @@ inf_text_chunk_new(const gchar* encoding)
  *
  * Returns a copy of @self.
  *
- * Return Value: A new #InfTextChunk.
+ * Returns: (transfer full): A new #InfTextChunk.
  **/
 InfTextChunk*
 inf_text_chunk_copy(InfTextChunk* self)
@@ -287,6 +425,7 @@ inf_text_chunk_copy(InfTextChunk* self)
 
   new_chunk->length = self->length;
   new_chunk->encoding = self->encoding;
+  new_chunk->path = self->path;
 
   return new_chunk;
 }
@@ -312,7 +451,7 @@ inf_text_chunk_free(InfTextChunk* self)
  *
  * Returns the character encoding in which the content of @self is encoded.
  *
- * Return Value: The encoding of @self.
+ * Returns: The encoding of @self.
  **/
 const gchar*
 inf_text_chunk_get_encoding(InfTextChunk* self)
@@ -327,7 +466,7 @@ inf_text_chunk_get_encoding(InfTextChunk* self)
  *
  * Returns the number of characters contained in @self.
  *
- * Return Value: The number of characters of @self.
+ * Returns: The number of characters of @self.
  **/
 guint
 inf_text_chunk_get_length(InfTextChunk* self)
@@ -345,7 +484,7 @@ inf_text_chunk_get_length(InfTextChunk* self)
  * Returns a new #InfTextChunk containing a substring of @self, beginning
  * at character offset @begin and @length characters long.
  *
- * Return Value: A new #InfTextChunk.
+ * Returns: (transfer full): A new #InfTextChunk.
  **/
 InfTextChunk*
 inf_text_chunk_substring(InfTextChunk* self,
@@ -424,6 +563,7 @@ inf_text_chunk_substring(InfTextChunk* self,
 
     result->length = length;
     result->encoding = self->encoding;
+    result->path = self->path;
   }
   else
   {
@@ -444,7 +584,8 @@ inf_text_chunk_substring(InfTextChunk* self,
  * inf_text_chunk_insert_text:
  * @self: A #InfTextChunk.
  * @offset: Character offset at which to insert text
- * @text: Text to insert.
+ * @text (type const guint8*) (array length=bytes) (transfer none): Text
+ * to insert.
  * @length: Number of characters contained in @text.
  * @bytes: Number of bytes of @text.
  * @author: User that wrote @text.
@@ -568,7 +709,7 @@ inf_text_chunk_insert_text(InfTextChunk* self,
  * inf_text_chunk_insert_chunk:
  * @self: A #InfTextChunk.
  * @offset: Character offset at which to insert text.
- * @text: Chunk to insert into @self.
+ * @text: (transfer none): Chunk to insert into @self.
  *
  * Inserts @text into @self at position @offset. @text and @self must
  * have the same encoding.
@@ -1034,13 +1175,14 @@ inf_text_chunk_erase(InfTextChunk* self,
 /**
  * inf_text_chunk_get_text:
  * @self: A #InfTextChunk.
- * @length: Location to write the number of bytes to, or %NULL.
+ * @length: (out): Location to write the number of bytes to, or %NULL.
  *
  * Returns the content of @self as an array. The text is encoded in
  * @self's encoding. @length is set to the number of bytes in the returned
  * buffer, if non-%NULL. The result is _not_ zero-terminated.
  *
- * Return Value: Content of @self. Free with g_free() if no longer in use.
+ * Returns: (type guint8*) (array length=length) (transfer full): Content of
+ * @self. Free with g_free() if no longer in use.
  **/
 gpointer
 inf_text_chunk_get_text(InfTextChunk* self,
@@ -1089,7 +1231,7 @@ inf_text_chunk_get_text(InfTextChunk* self,
  * Returns whether the two text chunks contain the same text and the same
  * segments were written by the same authors.
  *
- * Return Value: Whether the two chunks are equal.
+ * Returns: Whether the two chunks are equal.
  **/
 gboolean
 inf_text_chunk_equal(InfTextChunk* self,
@@ -1133,19 +1275,19 @@ inf_text_chunk_equal(InfTextChunk* self,
 }
 
 /**
- * inf_text_chunk_iter_init:
+ * inf_text_chunk_iter_init_begin:
  * @self: A #InfTextChunk.
- * @iter: A #InfTextChunkIter.
+ * @iter: (out): A #InfTextChunkIter.
  *
  * Sets @iter to point to the first segment of @self. If there are no
  * segments (i.e. @self is empty), @iter is left untouched and the function
  * returns %FALSE.
  *
- * Return Value: Whether @iter was set.
+ * Returns: Whether @iter was set.
  **/
 gboolean
-inf_text_chunk_iter_init(InfTextChunk* self,
-                         InfTextChunkIter* iter)
+inf_text_chunk_iter_init_begin(InfTextChunk* self,
+                               InfTextChunkIter* iter)
 {
   g_return_val_if_fail(self != NULL, FALSE);
   g_return_val_if_fail(iter != NULL, FALSE);
@@ -1163,6 +1305,37 @@ inf_text_chunk_iter_init(InfTextChunk* self,
   }
 }
 
+/**
+ * inf_text_chunk_iter_init_end:
+ * @self: A #InfTextChunk.
+ * @iter: (out): A #InfTextChunkIter.
+ *
+ * Sets @iter to point to the last segment of @self. If there are no
+ * segments (i.e. @self is empty), @iter is left untouched and the function
+ * returns %FALSE.
+ *
+ * Returns: Whether @iter was set.
+ **/
+gboolean
+inf_text_chunk_iter_init_end(InfTextChunk* self,
+                             InfTextChunkIter* iter)
+{
+  g_return_val_if_fail(self != NULL, FALSE);
+  g_return_val_if_fail(iter != NULL, FALSE);
+
+  if(self->length > 0)
+  {
+    iter->chunk = self;
+    iter->second = g_sequence_get_end_iter(self->segments);
+    iter->first = g_sequence_iter_prev(iter->second);
+    return TRUE;
+  }
+  else
+  {
+    return FALSE;
+  }
+}
+
 /**
  * inf_text_chunk_iter_next:
  * @iter: An initialized #InfTextChunkIter.
@@ -1170,7 +1343,7 @@ inf_text_chunk_iter_init(InfTextChunk* self,
  * Sets @iter to point to the next segment. If @iter already points to the
  * last segment, the function returns %FALSE.
  *
- * Return Value: Whether @iter was set.
+ * Returns: Whether @iter was set.
  **/
 gboolean
 inf_text_chunk_iter_next(InfTextChunkIter* iter)
@@ -1222,7 +1395,7 @@ inf_text_chunk_iter_prev(InfTextChunkIter* iter)
  * Returns the text of the segment @iter points to. The text is in the
  * underlaying #InfTextChunk's encoding.
  *
- * Return Value: The text of the segment @iter points to.
+ * Returns: (transfer none): The text of the segment @iter points to.
  **/
 gconstpointer
 inf_text_chunk_iter_get_text(InfTextChunkIter* iter)
@@ -1231,13 +1404,33 @@ inf_text_chunk_iter_get_text(InfTextChunkIter* iter)
   return ((InfTextChunkSegment*)g_sequence_get(iter->first))->text;
 }
 
+/**
+ * inf_text_chunk_iter_get_offset:
+ * @iter: An initialized #InfTextChunkIter.
+ *
+ * Returns the offset of the first character in the segment @iter points to.
+ *
+ * Returns: The offset of the first characters in the segment @iter
+ * points to.
+ **/
+guint
+inf_text_chunk_iter_get_offset(InfTextChunkIter* iter)
+{
+  InfTextChunkSegment* first;
+
+  g_return_val_if_fail(iter != NULL, 0);
+
+  first = (InfTextChunkSegment*)g_sequence_get(iter->first);
+  return first->offset;
+}
+
 /**
  * inf_text_chunk_iter_get_length:
  * @iter: An initialized #InfTextChunkIter.
  *
  * Returns the number of characters in the segment @iter points to.
  *
- * Return Value: The number of characters in the segment @iter points to.
+ * Returns: The number of characters in the segment @iter points to.
  **/
 guint
 inf_text_chunk_iter_get_length(InfTextChunkIter* iter)
@@ -1266,7 +1459,7 @@ inf_text_chunk_iter_get_length(InfTextChunkIter* iter)
  *
  * Returns the number of bytes in the segment @iter points to.
  *
- * Return Value: The number of bytes in the segment @iter points to.
+ * Returns: The number of bytes in the segment @iter points to.
  **/
 gsize
 inf_text_chunk_iter_get_bytes(InfTextChunkIter* iter)
@@ -1281,7 +1474,7 @@ inf_text_chunk_iter_get_bytes(InfTextChunkIter* iter)
  *
  * Returns the user ID of the author of the segment @iter points to.
  *
- * Return Value: The user ID of the author of the segment @iter points to.
+ * Returns: The user ID of the author of the segment @iter points to.
  **/
 guint
 inf_text_chunk_iter_get_author(InfTextChunkIter* iter)
