@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -78,22 +78,15 @@ struct _InfTextSessionEraseForeachData {
 
 #define INF_TEXT_SESSION_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TEXT_TYPE_SESSION, InfTextSessionPrivate))
 
-static InfAdoptedSessionClass* parent_class;
 static GQuark inf_text_session_error_quark;
 
+G_DEFINE_TYPE_WITH_CODE(InfTextSession, inf_text_session, INF_ADOPTED_TYPE_SESSION,
+  G_ADD_PRIVATE(InfTextSession))
+
 /*
  * Utility functions
  */
 
-static gint64
-inf_text_session_get_real_time()
-{
-  /* TODO: Replace by g_get_real_time() once we depend on glib >=2.28 */
-  GTimeVal timeval;
-  g_get_current_time(&timeval);
-  return (gint64)timeval.tv_sec * 1000000 + timeval.tv_usec;
-}
-
 /* Returns the difference between two GTimeVal, in milliseconds */
 static guint
 inf_text_session_timeval_diff(GTimeVal* first,
@@ -742,34 +735,23 @@ inf_text_session_init_text_handlers(InfTextSession* session)
  */
 
 static void
-inf_text_session_init(GTypeInstance* instance,
-                      gpointer g_class)
+inf_text_session_init(InfTextSession* session)
 {
-  InfTextSession* session;
   InfTextSessionPrivate* priv;
-
-  session = INF_TEXT_SESSION(instance);
   priv = INF_TEXT_SESSION_PRIVATE(session);
 
   priv->caret_update_interval = 500;
 }
 
-static GObject*
-inf_text_session_constructor(GType type,
-                             guint n_construct_properties,
-                             GObjectConstructParam* construct_properties)
+static void
+inf_text_session_constructed(GObject* object)
 {
-  GObject* object;
   InfTextSession* session;
   InfTextSessionPrivate* priv;
   InfTextBuffer* buffer;
   InfSessionStatus status;
 
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_construct_properties,
-    construct_properties
-  );
+  G_OBJECT_CLASS(inf_text_session_parent_class)->constructed(object);
 
   session = INF_TEXT_SESSION(object);
   priv = INF_TEXT_SESSION_PRIVATE(session);
@@ -790,8 +772,6 @@ inf_text_session_constructor(GType type,
 
   if(status == INF_SESSION_RUNNING)
     inf_text_session_init_text_handlers(session);
-
-  return object;
 }
 
 /*static void
@@ -859,7 +839,7 @@ inf_text_session_dispose(GObject* object)
     session
   );
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_text_session_parent_class)->dispose(object);
 }
 
 static void
@@ -871,7 +851,7 @@ inf_text_session_finalize(GObject* object)
   session = INF_TEXT_SESSION(object);
   priv = INF_TEXT_SESSION_PRIVATE(session);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_text_session_parent_class)->finalize(object);
 }
 
 static void
@@ -962,11 +942,10 @@ inf_text_session_handle_user_color_change(InfTextSession* session,
   if(inf_user_get_status(user) == INF_USER_UNAVAILABLE ||
      inf_user_get_connection(user) != connection)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_user_error_quark(),
       INF_USER_ERROR_NOT_JOINED,
-      "%s",
       _("User did not join from this connection")
     );
 
@@ -1010,8 +989,10 @@ inf_text_session_to_xml_sync(InfSession* session,
   gsize bytes_left;
   GIConv cd;
 
-  g_assert(INF_SESSION_CLASS(parent_class)->to_xml_sync != NULL);
-  INF_SESSION_CLASS(parent_class)->to_xml_sync(session, parent);
+  INF_SESSION_CLASS(inf_text_session_parent_class)->to_xml_sync(
+    session,
+    parent
+  );
 
   buffer = INF_TEXT_BUFFER(inf_session_get_buffer(session));
   cd = g_iconv_open("UTF-8", inf_text_buffer_get_encoding(buffer));
@@ -1123,9 +1104,7 @@ inf_text_session_process_xml_sync(InfSession* session,
   }
   else
   {
-    g_assert(INF_SESSION_CLASS(parent_class)->process_xml_sync != NULL);
-
-    return INF_SESSION_CLASS(parent_class)->process_xml_sync(
+    return INF_SESSION_CLASS(inf_text_session_parent_class)->process_xml_sync(
       session,
       connection,
       xml,
@@ -1140,8 +1119,6 @@ inf_text_session_process_xml_run(InfSession* session,
                                  const xmlNodePtr xml,
                                  GError** error)
 {
-  g_assert(INF_SESSION_CLASS(parent_class)->process_xml_run != NULL);
-
   if(strcmp((const char*)xml->name, "user-color-change") == 0)
   {
     return inf_text_session_handle_user_color_change(
@@ -1153,7 +1130,7 @@ inf_text_session_process_xml_run(InfSession* session,
   }
   else
   {
-    return INF_SESSION_CLASS(parent_class)->process_xml_run(
+    return INF_SESSION_CLASS(inf_text_session_parent_class)->process_xml_run(
       session,
       connection,
       xml,
@@ -1167,19 +1144,15 @@ inf_text_session_get_xml_user_props(InfSession* session,
                                     InfXmlConnection* connection,
                                     const xmlNodePtr xml)
 {
+  InfSessionClass* parent_class;
   GArray* array;
   GParameter* parameter;
   guint caret;
   gint selection;
   gdouble hue;
 
-  g_assert(INF_SESSION_CLASS(parent_class)->get_xml_user_props != NULL);
-
-  array = INF_SESSION_CLASS(parent_class)->get_xml_user_props(
-    session,
-    connection,
-    xml
-  );
+  parent_class = INF_SESSION_CLASS(inf_text_session_parent_class);
+  array = parent_class->get_xml_user_props(session, connection, xml);
 
   /* TODO: Error reporting for get_xml_user_props */
   if(inf_xml_util_get_attribute_uint(xml, "caret", &caret, NULL))
@@ -1214,14 +1187,11 @@ inf_text_session_set_xml_user_props(InfSession* session,
                                     guint n_params,
                                     xmlNodePtr xml)
 {
+  InfSessionClass* parent_class;
   const GParameter* param;
 
-  INF_SESSION_CLASS(parent_class)->set_xml_user_props(
-    session,
-    params,
-    n_params,
-    xml
-  );
+  parent_class = INF_SESSION_CLASS(inf_text_session_parent_class);
+  parent_class->set_xml_user_props(session, params, n_params, xml);
 
   param = inf_session_lookup_user_property(
     params,
@@ -1276,10 +1246,12 @@ inf_text_session_validate_user_props(InfSession* session,
                                      InfUser* exclude,
                                      GError** error)
 {
+  InfSessionClass* parent_class;
   const GParameter* caret;
   gboolean result;
 
-  result = INF_SESSION_CLASS(parent_class)->validate_user_props(
+  parent_class = INF_SESSION_CLASS(inf_text_session_parent_class);
+  result = parent_class->validate_user_props(
     session,
     params,
     n_params,
@@ -1297,12 +1269,11 @@ inf_text_session_validate_user_props(InfSession* session,
 
   if(caret == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_request_error_quark(),
       INF_REQUEST_ERROR_NO_SUCH_ATTRIBUTE,
-      "%s",
-      _("'caret' attribute in user message is missing")
+      _("\"caret\" attribute in user message is missing")
     );
 
     return FALSE;
@@ -1327,13 +1298,13 @@ static void
 inf_text_session_synchronization_complete(InfSession* session,
                                           InfXmlConnection* connection)
 {
+  InfSessionClass* parent_class;
   InfSessionStatus status;
+
+  parent_class = INF_SESSION_CLASS(inf_text_session_parent_class);
   status = inf_session_get_status(session);
 
-  INF_SESSION_CLASS(parent_class)->synchronization_complete(
-    session,
-    connection
-  );
+  parent_class->synchronization_complete(session, connection);
 
   /* init_text_handlers needs to access the algorithm which is created in the
    * parent class default signal handler which is why we call this afterwards.
@@ -1745,7 +1716,7 @@ inf_text_session_xml_to_request(InfAdoptedSession* session,
       vector,
       user_id,
       operation,
-      inf_text_session_get_real_time()
+      g_get_real_time()
     );
     g_object_unref(operation);
     break;
@@ -1753,14 +1724,14 @@ inf_text_session_xml_to_request(InfAdoptedSession* session,
     request = inf_adopted_request_new_undo(
       vector,
       user_id,
-      inf_text_session_get_real_time()
+      g_get_real_time()
     );
     break;
   case INF_ADOPTED_REQUEST_REDO:
     request = inf_adopted_request_new_redo(
       vector,
       user_id,
-      inf_text_session_get_real_time()
+      g_get_real_time()
     );
     break;
   default:
@@ -1781,21 +1752,17 @@ fail:
  */
 
 static void
-inf_text_session_class_init(gpointer g_class,
-                            gpointer class_data)
+inf_text_session_class_init(InfTextSessionClass* text_session_class)
 {
   GObjectClass* object_class;
   InfSessionClass* session_class;
   InfAdoptedSessionClass* adopted_session_class;
 
-  object_class = G_OBJECT_CLASS(g_class);
-  session_class = INF_SESSION_CLASS(g_class);
-  adopted_session_class = INF_ADOPTED_SESSION_CLASS(g_class);
+  object_class = G_OBJECT_CLASS(text_session_class);
+  session_class = INF_SESSION_CLASS(text_session_class);
+  adopted_session_class = INF_ADOPTED_SESSION_CLASS(text_session_class);
 
-  parent_class = INF_ADOPTED_SESSION_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfTextSessionPrivate));
-
-  object_class->constructor = inf_text_session_constructor;
+  object_class->constructed = inf_text_session_constructed;
   object_class->dispose = inf_text_session_dispose;
   object_class->finalize = inf_text_session_finalize;
   object_class->set_property = inf_text_session_set_property;
@@ -1833,51 +1800,20 @@ inf_text_session_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_text_session_get_type(void)
-{
-  static GType session_type = 0;
-
-  if(!session_type)
-  {
-    static const GTypeInfo session_type_info = {
-      sizeof(InfTextSessionClass),   /* class_size */
-      NULL,                          /* base_init */
-      NULL,                          /* base_finalize */
-      inf_text_session_class_init,   /* class_init */
-      NULL,                          /* class_finalize */
-      NULL,                          /* class_data */
-      sizeof(InfTextSession),        /* instance_size */
-      0,                             /* n_preallocs */
-      inf_text_session_init,         /* instance_init */
-      NULL                           /* value_table */
-    };
-
-    session_type = g_type_register_static(
-      INF_ADOPTED_TYPE_SESSION,
-      "InfTextSession",
-      &session_type_info,
-      0
-    );
-  }
-
-  return session_type;
-}
-
 /*
  * Public API.
  */
 
 /**
- * inf_text_session_new:
+ * inf_text_session_new: (constructor)
  * @manager: A #InfCommunicationManager.
  * @buffer: An initial #InfTextBuffer.
  * @io: A #InfIo object.
  * @status: The initial status of the session.
- * @sync_group: A group in which the session is synchronized. Ignored if
- * @status is %INF_SESSION_RUNNING.
- * @sync_connection: A connection to synchronize the session from. Ignored if
- * @status is %INF_SESSION_RUNNING.
+ * @sync_group: (allow-none): A group in which the session is synchronized.
+ * Ignored if @status is %INF_SESSION_RUNNING.
+ * @sync_connection: (allow-none): A connection to synchronize the session
+ * from. Ignored if @status is %INF_SESSION_RUNNING.
  *
  * Creates a new #InfTextSession. The communication manager is used to send
  * and receive requests from subscription and synchronization. @buffer will be
@@ -1889,7 +1825,7 @@ inf_text_session_get_type(void)
  * retrieved from @sync_connection. If you are subscribed to the session, set
  * the subscription group via inf_session_set_subscription_group().
  *
- * Return Value: A new #InfTextSession.
+ * Returns: (transfer full): A new #InfTextSession.
  **/
 InfTextSession*
 inf_text_session_new(InfCommunicationManager* manager,
@@ -1929,16 +1865,16 @@ inf_text_session_new(InfCommunicationManager* manager,
 }
 
 /**
- * inf_text_session_new_with_user_table:
+ * inf_text_session_new_with_user_table: (constructor)
  * @manager: A #InfCommunicationManager.
  * @buffer: An initial #InfTextBuffer.
  * @io: A #InfIo object.
  * @user_table: A #InfUserTable.
  * @status: The initial status for the session.
- * @sync_group: A group in which the session is synchronized. Ignored if
- * @status is %INF_SESSION_RUNNING.
- * @sync_connection: A connection to synchronize the session from. Ignored if
- * @status is %INF_SESSION_RUNNING.
+ * @sync_group: (allow-none): A group in which the session is synchronized.
+ * Ignored if @status is %INF_SESSION_RUNNING.
+ * @sync_connection: (allow-none): A connection to synchronize the session
+ * from. Ignored if @status is %INF_SESSION_RUNNING.
  *
  * Creates a new #InfTextSession. The connection manager is used to send and
  * receive requests from subscription and synchronization. @buffer will be
@@ -1956,7 +1892,7 @@ inf_text_session_new(InfCommunicationManager* manager,
  * to another session, but different sessions cannot share the same user
  * object.
  *
- * Return Value: A new #InfTextSession.
+ * Returns: (transfer full): A new #InfTextSession.
  **/
 InfTextSession*
 inf_text_session_new_with_user_table(InfCommunicationManager* manager,
@@ -2082,13 +2018,14 @@ inf_text_session_flush_requests_for_user(InfTextSession* session,
  * @hue: The user color of the user to join.
  * @caret_position: The initial position of the new user's cursor.
  * @selection_length: The initial length of the new user's selection.
- * @func: Function to call after completion of the request, or %NULL.
+ * @func: (scope async): Function to call after completion of the request,
+ * or %NULL.
  * @user_data: Additional data to pass to @func.
  *
  * This functions creates a user join request for an #InfTextSession. This is
  * a shortcut for inf_session_proxy_join_user().
  *
- * Returns: A #InfRequest, or %NULL on error.
+ * Returns: (transfer full): A #InfRequest, or %NULL.
  */
 InfRequest*
 inf_text_session_join_user(InfSessionProxy* proxy,
