@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,6 @@
  */
 
 #include <libinftext/inf-text-user.h>
-#include <libinfinity/inf-marshal.h>
 
 typedef struct _InfTextUserPrivate InfTextUserPrivate;
 struct _InfTextUserPrivate {
@@ -43,17 +42,15 @@ enum {
 
 #define INF_TEXT_USER_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TEXT_TYPE_USER, InfTextUserPrivate))
 
-static InfUserClass* parent_class;
 static guint user_signals[LAST_SIGNAL] = { 0 };
 
-static void
-inf_text_user_init(GTypeInstance* instance,
-                   gpointer g_class)
-{
-  InfTextUser* user;
-  InfTextUserPrivate* priv;
+G_DEFINE_TYPE_WITH_CODE(InfTextUser, inf_text_user, INF_ADOPTED_TYPE_USER,
+  G_ADD_PRIVATE(InfTextUser))
 
-  user = INF_TEXT_USER(instance);
+static void
+inf_text_user_init(InfTextUser* user)
+{
+  InfTextUserPrivate* priv;
   priv = INF_TEXT_USER_PRIVATE(user);
 
   priv->caret = 0;
@@ -136,17 +133,10 @@ inf_text_user_selection_changed(InfTextUser* user,
 }
 
 static void
-inf_text_user_class_init(gpointer g_class,
-                         gpointer class_data)
+inf_text_user_class_init(InfTextUserClass* user_class)
 {
   GObjectClass* object_class;
-  InfTextUserClass* user_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  user_class = INF_TEXT_USER_CLASS(g_class);
-
-  parent_class = INF_USER_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfTextUserPrivate));
+  object_class = G_OBJECT_CLASS(user_class);
 
   object_class->set_property = inf_text_user_set_property;
   object_class->get_property = inf_text_user_get_property;
@@ -202,7 +192,7 @@ inf_text_user_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfTextUserClass, selection_changed),
     NULL, NULL,
-    inf_marshal_VOID__UINT_INT_BOOLEAN,
+    NULL,
     G_TYPE_NONE,
     3,
     G_TYPE_UINT,
@@ -211,50 +201,19 @@ inf_text_user_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_text_user_get_type(void)
-{
-  static GType user_type = 0;
-
-  if(!user_type)
-  {
-    static const GTypeInfo user_type_info = {
-      sizeof(InfTextUserClass),  /* class_size */
-      NULL,                      /* base_init */
-      NULL,                      /* base_finalize */
-      inf_text_user_class_init,  /* class_init */
-      NULL,                      /* class_finalize */
-      NULL,                      /* class_data */
-      sizeof(InfTextUser),       /* instance_size */
-      0,                         /* n_preallocs */
-      inf_text_user_init,        /* instance_init */
-      NULL                       /* value_table */
-    };
-
-    user_type = g_type_register_static(
-      INF_ADOPTED_TYPE_USER,
-      "InfTextUser",
-      &user_type_info,
-      0
-    );
-  }
-
-  return user_type;
-}
-
 /**
- * inf_text_user_new:
+ * inf_text_user_new: (constructor)
  * @id: The user ID for this user.
  * @name: The user's name.
- * @vector: The state at which the user is at, or %NULL.
+ * @vector: (allow-none): The state at which the user is at, or %NULL.
  * @hue: The hue value of the user's color.
  *
  * Creates a new #InfTextUser. @id should be unique for all users working
  * together. #InfUserTable will refuse to add users with duplicate id. If
  * @vector is %NULL, then the vector with all components zero is used.
  *
- * Returns: A new #InfTextUser. Free with g_object_unref() when no longer
- * needed.
+ * Returns: (transfer full): A new #InfTextUser. Free with g_object_unref()
+ * when no longer needed.
  */
 InfTextUser*
 inf_text_user_new(guint id,
@@ -282,7 +241,7 @@ inf_text_user_new(guint id,
  *
  * Returns the position of @user's caret.
  *
- * Return Value: @user's caret position.
+ * Returns: @user's caret position.
  **/
 guint
 inf_text_user_get_caret_position(InfTextUser* user)
@@ -299,7 +258,7 @@ inf_text_user_get_caret_position(InfTextUser* user)
  * the caret position. Negative number mean selection towards the beginning
  * of the buffer.
  *
- * Return Value: @user's selection length in characters.
+ * Returns: @user's selection length in characters.
  **/
 gint
 inf_text_user_get_selection_length(InfTextUser* user)
