@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,6 +17,39 @@
  * MA 02110-1301, USA.
  */
 
+/**
+ * SECTION:inf-text-gtk-buffer
+ * @title: InfTextGtkBuffer
+ * @short_description: Synchronizing a text session with a #GtkTextBuffer
+ * @include: libinftextgtk/inf-text-gtk-buffer.h
+ * @see_also: #InfTextBuffer
+ * @stability: Unstable
+ *
+ * #InfTextGtkBuffer is an implementation of the #InfTextBuffer interface. It
+ * can therefore be used as a backend for #InfTextSession<!-- -->s to store
+ * their text. The buffer is implemented by using a #GtkTextBuffer as
+ * storage. This way the text document can be displayed using a #GtkTextView
+ * such that only one copy of the text is held in memory, which is used both
+ * by the user interface toolkit and the text synchronization functionality.
+ *
+ * If external changes are made to the #GtkTextBuffer, for example by a user
+ * typing into a #GtkTextView, then the text is synchronized to other
+ * participants of the session. For this purpose,
+ * inf_text_gtk_buffer_set_active_user() should be called with a user that
+ * was previously joined into the session using inf_session_proxy_join_user().
+ * If there is no local user in the session, no modifications to the buffer
+ * must be made because they cannot be synchronized to other participants.
+ *
+ * This class also takes care of setting background colors for the text to
+ * indicate which user wrote what text, by adding corresponding
+ * #GtkTextTag<!-- -->s to the document. The function
+ * inf_text_gtk_buffer_set_show_user_colors() to turn on or off the colored
+ * background. Even if background coloring is turned off, the text is still
+ * tagged according to the authorship, so that coloring can be turned on at a
+ * later point or so that the authorship can still be queried for other means,
+ * such as in a "blame" kind of functionality.
+ */
+
 #include <libinftextgtk/inf-text-gtk-buffer.h>
 #include <libinftext/inf-text-buffer.h>
 
@@ -71,6 +104,7 @@ struct _InfTextGtkBufferPrivate {
 
   gdouble saturation;
   gdouble value;
+  gdouble alpha;
 };
 
 enum {
@@ -84,6 +118,7 @@ enum {
 
   PROP_SATURATION,
   PROP_VALUE,
+  PROP_ALPHA,
 
   /* overriden */
   PROP_MODIFIED
@@ -91,84 +126,14 @@ enum {
 
 #define INF_TEXT_GTK_BUFFER_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TEXT_GTK_TYPE_BUFFER, InfTextGtkBufferPrivate))
 
-static GObjectClass* parent_class;
 static GQuark inf_text_gtk_buffer_tag_user_quark;
 
-/* This function is stolen from gtkhsv.c from GTK+ */
-/* TODO: Use gtk_hsv_to_rgb from GTK+ 2.14 instead */
-/* Converts from HSV to RGB */
-static void
-hsv_to_rgb (gdouble *h,
-            gdouble *s,
-            gdouble *v)
-{
-  gdouble hue, saturation, value;
-  gdouble f, p, q, t;
-
-  if (*s == 0.0)
-  {
-    *h = *v;
-    *s = *v;
-    *v = *v; /* heh */
-  }
-  else
-  {
-    hue = *h * 6.0;
-    saturation = *s;
-    value = *v;
-
-    if (hue == 6.0)
-      hue = 0.0;
-
-    f = hue - (int) hue;
-    p = value * (1.0 - saturation);
-    q = value * (1.0 - saturation * f);
-    t = value * (1.0 - saturation * (1.0 - f));
-
-    switch ((int) hue)
-    {
-    case 0:
-      *h = value;
-      *s = t;
-      *v = p;
-      break;
-
-    case 1:
-      *h = q;
-      *s = value;
-      *v = p;
-      break;
-
-    case 2:
-      *h = p;
-      *s = value;
-      *v = t;
-      break;
-
-    case 3:
-      *h = p;
-      *s = q;
-      *v = value;
-      break;
-
-    case 4:
-      *h = t;
-      *s = p;
-      *v = value;
-      break;
-
-    case 5:
-      *h = value;
-      *s = p;
-      *v = q;
-      break;
-
-    default:
-      g_assert_not_reached ();
-      break;
-    }
-  }
-}
+static void inf_text_gtk_buffer_buffer_iface_init(InfBufferInterface* iface);
+static void inf_text_gtk_buffer_text_buffer_iface_init(InfTextBufferInterface* iface);
+G_DEFINE_TYPE_WITH_CODE(InfTextGtkBuffer, inf_text_gtk_buffer, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfTextGtkBuffer)
+  G_IMPLEMENT_INTERFACE(INF_TYPE_BUFFER, inf_text_gtk_buffer_buffer_iface_init)
+  G_IMPLEMENT_INTERFACE(INF_TEXT_TYPE_BUFFER, inf_text_gtk_buffer_text_buffer_iface_init))
 
 static void
 inf_text_gtk_update_tag_color(InfTextGtkBuffer* buffer,
@@ -179,20 +144,18 @@ inf_text_gtk_update_tag_color(InfTextGtkBuffer* buffer,
   gdouble hue;
   gdouble saturation;
   gdouble value;
-  GdkColor color;
+  GdkRGBA rgba;
 
   priv = INF_TEXT_GTK_BUFFER_PRIVATE(buffer);
 
   hue = inf_text_user_get_hue(user);
   saturation = priv->saturation;
   value = priv->value;
-  hsv_to_rgb(&hue, &saturation, &value);
 
-  color.red = hue * 0xffff;
-  color.green = saturation * 0xffff;
-  color.blue = value * 0xffff;
+  gtk_hsv_to_rgb(hue, saturation, value, &rgba.red, &rgba.green, &rgba.blue);
+  rgba.alpha = priv->alpha;
 
-  g_object_set(G_OBJECT(tag), "background-gdk", &color, NULL);
+  g_object_set(G_OBJECT(tag), "background-rgba", &rgba, NULL);
 }
 
 static void
@@ -451,8 +414,8 @@ inf_text_gtk_buffer_ensure_author_tags_priority_foreach_func(GtkTextTag* tag,
 }
 
 static void
-inf_text_gtk_buffer_set_saturation_value_tag_table_foreach_func(GtkTextTag* t,
-                                                                gpointer data)
+inf_text_gtk_buffer_update_user_color_tag_table_foreach_func(GtkTextTag* tag,
+                                                             gpointer data)
 {
   InfTextGtkBuffer* buffer;
   InfTextGtkBufferPrivate* priv;
@@ -460,10 +423,10 @@ inf_text_gtk_buffer_set_saturation_value_tag_table_foreach_func(GtkTextTag* t,
 
   buffer = INF_TEXT_GTK_BUFFER(data);
   priv = INF_TEXT_GTK_BUFFER_PRIVATE(buffer);
-  author = inf_text_gtk_buffer_author_from_tag(t);
+  author = inf_text_gtk_buffer_author_from_tag(tag);
 
   if(author != NULL)
-    inf_text_gtk_update_tag_color(buffer, t, author);
+    inf_text_gtk_update_tag_color(buffer, tag, author);
 }
 
 /* Required by inf_text_gtk_buffer_record_signal() and
@@ -1355,13 +1318,9 @@ inf_text_gtk_buffer_set_buffer(InfTextGtkBuffer* buffer,
 }
 
 static void
-inf_text_gtk_buffer_init(GTypeInstance* instance,
-                         gpointer g_class)
+inf_text_gtk_buffer_init(InfTextGtkBuffer* buffer)
 {
-  InfTextGtkBuffer* buffer;
   InfTextGtkBufferPrivate* priv;
-
-  buffer = INF_TEXT_GTK_BUFFER(instance);
   priv = INF_TEXT_GTK_BUFFER_PRIVATE(buffer);
 
   priv->buffer = NULL;
@@ -1381,6 +1340,7 @@ inf_text_gtk_buffer_init(GTypeInstance* instance,
 
   priv->saturation = 0.35;
   priv->value = 1.0;
+  priv->alpha = 1.0;
 }
 
 static void
@@ -1398,7 +1358,7 @@ inf_text_gtk_buffer_dispose(GObject* object)
   inf_text_gtk_buffer_set_active_user(buffer, NULL);
   g_object_unref(priv->user_table);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_text_gtk_buffer_parent_class)->dispose(object);
 }
 
 static void
@@ -1412,7 +1372,7 @@ inf_text_gtk_buffer_finalize(GObject* object)
 
   g_hash_table_unref(priv->user_tags);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_text_gtk_buffer_parent_class)->finalize(object);
 }
 
 static void
@@ -1471,6 +1431,9 @@ inf_text_gtk_buffer_set_property(GObject* object,
       g_value_get_double(value)
     );
     break;
+  case PROP_ALPHA:
+    inf_text_gtk_buffer_set_fade(buffer, g_value_get_double(value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -1512,6 +1475,15 @@ inf_text_gtk_buffer_get_property(GObject* object,
     else
       g_value_set_boolean(value, FALSE);
 
+    break;
+  case PROP_SATURATION:
+    g_value_set_double(value, priv->saturation);
+    break;
+  case PROP_VALUE:
+    g_value_set_double(value, priv->value);
+    break;
+  case PROP_ALPHA:
+    g_value_set_double(value, priv->alpha);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -1655,7 +1627,7 @@ inf_text_gtk_buffer_buffer_insert_text(InfTextBuffer* buffer,
     buffer
   );
 
-  if(inf_text_chunk_iter_init(chunk, &chunk_iter))
+  if(inf_text_chunk_iter_init_begin(chunk, &chunk_iter))
   {
     gtk_text_buffer_get_iter_at_offset(
       priv->buffer,
@@ -1850,7 +1822,7 @@ inf_text_gtk_buffer_buffer_erase_text(InfTextBuffer* buffer,
 }
 
 static InfTextBufferIter*
-inf_text_gtk_buffer_buffer_create_iter(InfTextBuffer* buffer)
+inf_text_gtk_buffer_buffer_create_begin_iter(InfTextBuffer* buffer)
 {
   InfTextGtkBufferPrivate* priv;
   InfTextBufferIter* iter;
@@ -1877,6 +1849,34 @@ inf_text_gtk_buffer_buffer_create_iter(InfTextBuffer* buffer)
   }
 }
 
+static InfTextBufferIter*
+inf_text_gtk_buffer_buffer_create_end_iter(InfTextBuffer* buffer)
+{
+  InfTextGtkBufferPrivate* priv;
+  InfTextBufferIter* iter;
+
+  priv = INF_TEXT_GTK_BUFFER_PRIVATE(buffer);
+
+  if(gtk_text_buffer_get_char_count(priv->buffer) == 0)
+  {
+    return NULL;
+  }
+  else
+  {
+    iter = g_slice_new(InfTextBufferIter);
+    gtk_text_buffer_get_end_iter(priv->buffer, &iter->end);
+
+    iter->begin = iter->end;
+    inf_text_gtk_buffer_iter_prev_author_toggle(
+      &iter->begin,
+      &iter->user,
+      NULL
+    );
+
+    return iter;
+  }
+}
+
 static void
 inf_text_gtk_buffer_buffer_destroy_iter(InfTextBuffer* buffer,
                                         InfTextBufferIter* iter)
@@ -1928,12 +1928,19 @@ inf_text_gtk_buffer_buffer_iter_get_text(InfTextBuffer* buffer,
   );
 }
 
+static guint
+inf_text_gtk_buffer_buffer_iter_get_offset(InfTextBuffer* buffer,
+                                           InfTextBufferIter* iter)
+{
+  return gtk_text_iter_get_offset(&iter->begin);
+}
+
 static guint
 inf_text_gtk_buffer_buffer_iter_get_length(InfTextBuffer* buffer,
                                            InfTextBufferIter* iter)
 {
-  return gtk_text_iter_get_offset(&iter->begin) -
-    gtk_text_iter_get_offset(&iter->end);
+  return gtk_text_iter_get_offset(&iter->end) -
+    gtk_text_iter_get_offset(&iter->begin);
 }
 
 static gsize
@@ -1996,14 +2003,10 @@ inf_text_gtk_buffer_buffer_iter_get_author(InfTextBuffer* buffer,
 }
 
 static void
-inf_text_gtk_buffer_class_init(gpointer g_class,
-                               gpointer class_data)
+inf_text_gtk_buffer_class_init(InfTextGtkBufferClass* text_gtk_buffer_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfTextGtkBufferPrivate));
+  object_class = G_OBJECT_CLASS(text_gtk_buffer_class);
 
   object_class->dispose = inf_text_gtk_buffer_dispose;
   object_class->finalize = inf_text_gtk_buffer_finalize;
@@ -2103,37 +2106,45 @@ inf_text_gtk_buffer_class_init(gpointer g_class,
     )
   );
 
+  g_object_class_install_property(
+    object_class,
+    PROP_VALUE,
+    g_param_spec_double(
+      "alpha",
+      "Alpha",
+      "The translucency of the user color",
+      0.0,
+      1.0,
+      1.0,
+      G_PARAM_READWRITE
+    )
+  );
+
   g_object_class_override_property(object_class, PROP_MODIFIED, "modified");
 }
 
 static void
-inf_text_gtk_buffer_buffer_init(gpointer g_iface,
-                                gpointer iface_data)
+inf_text_gtk_buffer_buffer_iface_init(InfBufferInterface* iface)
 {
-  InfBufferIface* iface;
-  iface = (InfBufferIface*)g_iface;
-
   iface->get_modified = inf_text_gtk_buffer_buffer_get_modified;
   iface->set_modified = inf_text_gtk_buffer_buffer_set_modified;
 }
 
 static void
-inf_text_gtk_buffer_text_buffer_init(gpointer g_iface,
-                                     gpointer iface_data)
+inf_text_gtk_buffer_text_buffer_iface_init(InfTextBufferInterface* iface)
 {
-  InfTextBufferIface* iface;
-  iface = (InfTextBufferIface*)g_iface;
-
   iface->get_encoding = inf_text_gtk_buffer_buffer_get_encoding;
   iface->get_length = inf_text_gtk_buffer_get_length;
   iface->get_slice = inf_text_gtk_buffer_buffer_get_slice;
   iface->insert_text = inf_text_gtk_buffer_buffer_insert_text;
   iface->erase_text = inf_text_gtk_buffer_buffer_erase_text;
-  iface->create_iter = inf_text_gtk_buffer_buffer_create_iter;
+  iface->create_begin_iter = inf_text_gtk_buffer_buffer_create_begin_iter;
+  iface->create_end_iter = inf_text_gtk_buffer_buffer_create_end_iter;
   iface->destroy_iter = inf_text_gtk_buffer_buffer_destroy_iter;
   iface->iter_next = inf_text_gtk_buffer_buffer_iter_next;
   iface->iter_prev = inf_text_gtk_buffer_buffer_iter_prev;
   iface->iter_get_text = inf_text_gtk_buffer_buffer_iter_get_text;
+  iface->iter_get_offset = inf_text_gtk_buffer_buffer_iter_get_offset;
   iface->iter_get_length = inf_text_gtk_buffer_buffer_iter_get_length;
   iface->iter_get_bytes = inf_text_gtk_buffer_buffer_iter_get_bytes;
   iface->iter_get_author = inf_text_gtk_buffer_buffer_iter_get_author;
@@ -2141,63 +2152,8 @@ inf_text_gtk_buffer_text_buffer_init(gpointer g_iface,
   iface->text_erased = NULL;
 }
 
-GType
-inf_text_gtk_buffer_get_type(void)
-{
-  static GType buffer_type = 0;
-
-  if(!buffer_type)
-  {
-    static const GTypeInfo buffer_type_info = {
-      sizeof(InfTextGtkBufferClass),  /* class_size */
-      NULL,                           /* base_init */
-      NULL,                           /* base_finalize */
-      inf_text_gtk_buffer_class_init, /* class_init */
-      NULL,                           /* class_finalize */
-      NULL,                           /* class_data */
-      sizeof(InfTextGtkBuffer),       /* instance_size */
-      0,                              /* n_preallocs */
-      inf_text_gtk_buffer_init,       /* instance_init */
-      NULL                            /* value_table */
-    };
-
-    static const GInterfaceInfo buffer_info = {
-      inf_text_gtk_buffer_buffer_init,
-      NULL,
-      NULL
-    };
-
-    static const GInterfaceInfo text_buffer_info = {
-      inf_text_gtk_buffer_text_buffer_init,
-      NULL,
-      NULL
-    };
-
-    buffer_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfTextGtkBuffer",
-      &buffer_type_info,
-      0
-    );
-
-    g_type_add_interface_static(
-      buffer_type,
-      INF_TYPE_BUFFER,
-      &buffer_info
-    );
-
-    g_type_add_interface_static(
-      buffer_type,
-      INF_TEXT_TYPE_BUFFER,
-      &text_buffer_info
-    );
-  }
-
-  return buffer_type;
-}
-
 /**
- * inf_text_gtk_buffer_new:
+ * inf_text_gtk_buffer_new: (constructor)
  * @buffer: The underlaying #GtkTextBuffer.
  * @user_table: The #InfUserTable containing the participating users.
  *
@@ -2205,7 +2161,7 @@ inf_text_gtk_buffer_get_type(void)
  * #InfTextBuffer interface by using @buffer to store the text. User colors
  * are read from the users from @user_table.
  *
- * Return Value: A #InfTextGtkBuffer.
+ * Returns: (transfer full): A #InfTextGtkBuffer.
  **/
 InfTextGtkBuffer*
 inf_text_gtk_buffer_new(GtkTextBuffer* buffer,
@@ -2232,7 +2188,7 @@ inf_text_gtk_buffer_new(GtkTextBuffer* buffer,
  *
  * Returns the underlaying #GtkTextBuffer.
  *
- * Return Value: A #GtkTextBuffer.
+ * Returns: (transfer none): A #GtkTextBuffer.
  **/
 GtkTextBuffer*
 inf_text_gtk_buffer_get_text_buffer(InfTextGtkBuffer* buffer)
@@ -2244,7 +2200,7 @@ inf_text_gtk_buffer_get_text_buffer(InfTextGtkBuffer* buffer)
 /**
  * inf_text_gtk_buffer_set_active_user:
  * @buffer: A #InfTextGtkBuffer.
- * @user: A #InfTextUser, or %NULL.
+ * @user: (allow-none): A #InfTextUser, or %NULL.
  *
  * Sets the active user for @buffer. The active user is the user by which
  * edits not issued through the #InfTextBuffer interface are performed (for
@@ -2325,7 +2281,7 @@ inf_text_gtk_buffer_set_active_user(InfTextGtkBuffer* buffer,
  *
  * Returns the current active user for @buffer.
  *
- * Return Value: A #InfTextUser.
+ * Returns: (transfer none) (allow-none): A #InfTextUser.
  **/
 InfTextUser*
 inf_text_gtk_buffer_get_active_user(InfTextGtkBuffer* buffer)
@@ -2342,7 +2298,7 @@ inf_text_gtk_buffer_get_active_user(InfTextGtkBuffer* buffer)
  * Returns the #InfTextUser which wrote the character at @location. If there
  * is no such user, then %NULL is returned.
  *
- * Return Value: A #InfTextUser, or %NULL.
+ * Returns: (transfer none) (allow-none): A #InfTextUser, or %NULL.
  */
 InfTextUser*
 inf_text_gtk_buffer_get_author(InfTextGtkBuffer* buffer,
@@ -2371,7 +2327,7 @@ inf_text_gtk_buffer_get_author(InfTextGtkBuffer* buffer,
  * marked by @tag. If @tag is not an author tag then the function returns
  * %NULL.
  *
- * Returns: A #InfTextUser, or %NULL.
+ * Returns: (transfer none) (allow-none): A #InfTextUser, or %NULL.
  */
 InfTextUser*
 inf_text_gtk_buffer_get_user_for_tag(InfTextGtkBuffer* buffer,
@@ -2387,8 +2343,9 @@ inf_text_gtk_buffer_get_user_for_tag(InfTextGtkBuffer* buffer,
  * inf_text_gtk_buffer_is_author_toggle:
  * @buffer: A #InfTextGtkBuffer.
  * @iter: A #GtkTextIter pointing into @buffer's underlying #GtkTextBuffer.
- * @user_on: A location to store a #InfTextUser, or %NULL.
- * @user_off: Another location to store a #InfTextUser, or %NULL.
+ * @user_on: (out) (allow-none): A location to store a #InfTextUser, or %NULL.
+ * @user_off: (out) (allow-none): Another location to store a #InfTextUser,
+ * or %NULL.
  *
  * This function returns %TRUE if the author of the text in @buffer changes
  * at @iter, or %FALSE otherwise. If it returns %TRUE, then the user who
@@ -2420,8 +2377,9 @@ inf_text_gtk_buffer_is_author_toggle(InfTextGtkBuffer* buffer,
  * inf_text_gtk_buffer_forward_to_author_toggle:
  * @buffer: A #InfTextGtkBuffer.
  * @iter: A #GtkTextIter pointing into @buffer's underlying #GtkTextBuffer.
- * @user_on: A location to store a #InfTextUser, or %NULL.
- * @user_off: Another location to store a #InfTextUser, or %NULL.
+ * @user_on: (out) (allow-none): A location to store a #InfTextUser, or %NULL.
+ * @user_off: (out) (allow-none): Another location to store a #InfTextUser,
+ * or %NULL.
  *
  * Moves @iter to the next point in @buffer's underlying #GtkTextBuffer where
  * the text has been written by another user. If @iter points to the end of
@@ -2453,8 +2411,9 @@ inf_text_gtk_buffer_forward_to_author_toggle(InfTextGtkBuffer* buffer,
  * inf_text_gtk_buffer_backward_to_author_toggle:
  * @buffer: A #InfTextGtkBuffer.
  * @iter: A #GtkTextIter pointing into @buffer's underlying #GtkTextBuffer.
- * @user_on: A location to store a #InfTextUser, or %NULL.
- * @user_off: Another location to store a #InfTextUser, or %NULL.
+ * @user_on: (out) (allow-none): A location to store a #InfTextUser, or %NULL.
+ * @user_off: (out) (allow-none): Another location to store a #InfTextUser,
+ * or %NULL.
  *
  * Moves @iter to the previous point in @buffer's underlying #GtkTextBuffer
  * where the text has been written by another user. If @iter points to the
@@ -2490,11 +2449,11 @@ inf_text_gtk_buffer_backward_to_author_toggle(InfTextGtkBuffer* buffer,
  * This function spcecifies whether movement of the insertion point or
  * selection bound of the underlying text buffer causes the active user
  * (see inf_text_gtk_buffer_set_active_user()) to become active when its
- * status is %INF_USER_STATUS_INACTIVE.
+ * status is %INF_USER_INACTIVE.
  *
- * If @wake is %TRUE, then the user status changes to %INF_USER_STATUS_ACTIVE
+ * If @wake is %TRUE, then the user status changes to %INF_USER_ACTIVE
  * in that case. If @wake is %FALSE, then the user status stays
- * %INF_USER_STATUS_INACTIVE, and its caret-position and selection-length
+ * %INF_USER_INACTIVE, and its caret-position and selection-length
  * properties will be no longer be synchronized to the buffer marks until
  * the user is set active again.
  */
@@ -2515,7 +2474,7 @@ inf_text_gtk_buffer_set_wake_on_cursor_movement(InfTextGtkBuffer* buffer,
  * Returns whether movement of the insertion point or selection bound of the
  * underlying text buffer causes whether the active user (see
  * inf_text_gtk_buffer_set_active_user()) to become active when its status
- * is %INF_USER_STATUS_INACTIVE. See also
+ * is %INF_USER_INACTIVE. See also
  * inf_text_gtk_buffer_set_wake_on_cursor_movement().
  *
  * Returns: Whether to make inactive users active when the insertion mark
@@ -2601,12 +2560,57 @@ inf_text_gtk_buffer_set_saturation_value(InfTextGtkBuffer* buffer,
   tag_table = gtk_text_buffer_get_tag_table(priv->buffer);
   gtk_text_tag_table_foreach(
     tag_table,
-    inf_text_gtk_buffer_set_saturation_value_tag_table_foreach_func,
+    inf_text_gtk_buffer_update_user_color_tag_table_foreach_func,
     buffer
   );
   g_object_thaw_notify(G_OBJECT(buffer));
 }
 
+/**
+ * inf_text_gtk_buffer_set_fade:
+ * @buffer: A #InfTextGtkBuffer.
+ * @alpha: An alpha value between 0.0 and 1.0.
+ *
+ * This functions can be used to show the user background color with limited
+ * intensity, such that the background of the #GtkTextView showing the buffer
+ * partly shines through.
+ *
+ * An @alpha value of 1.0 means to fully show the user background color, a
+ * value of 0.0 means to show the given background color. Values inbetween
+ * interpolate linearly between the two colors in RGB color space.
+ *
+ * The default value for @alpha is 1.0.
+ */
+void
+inf_text_gtk_buffer_set_fade(InfTextGtkBuffer* buffer,
+                             gdouble alpha)
+{
+  InfTextGtkBufferPrivate* priv;
+  GtkTextTagTable* tag_table;
+
+  g_return_if_fail(INF_TEXT_GTK_IS_BUFFER(buffer));
+  g_return_if_fail(alpha >= 0.0 && alpha <= 1.0);
+
+  priv = INF_TEXT_GTK_BUFFER_PRIVATE(buffer);
+
+  g_object_freeze_notify(G_OBJECT(buffer));
+  if(alpha != priv->alpha)
+  {
+    priv->alpha = alpha;
+    g_object_notify(G_OBJECT(buffer), "alpha");
+  }
+
+  tag_table = gtk_text_buffer_get_tag_table(priv->buffer);
+
+  gtk_text_tag_table_foreach(
+    tag_table,
+    inf_text_gtk_buffer_update_user_color_tag_table_foreach_func,
+    buffer
+  );
+
+  g_object_thaw_notify(G_OBJECT(buffer));
+}
+
 /**
  * inf_text_gtk_buffer_get_saturation:
  * @buffer: A #InfTextGtkBuffer.
@@ -2680,8 +2684,10 @@ inf_text_gtk_buffer_get_show_user_colors(InfTextGtkBuffer* buffer)
  * inf_text_gtk_buffer_show_user_colors:
  * @buffer: A #InfTextGtkBuffer.
  * @show: Whether to show or hide user colors.
- * @start: Beginning of the range for which to show or hide user colors.
- * @end: End of the range for which to show or hide user colors.
+ * @start: (in) (transfer none): Beginning of the range for which to show
+ * or hide user colors.
+ * @end: (in) (transfer none): End of the range for which to show or hide
+ * user colors.
  *
  * If @show is %FALSE, then don't show user colors (which user wrote what
  * text) as the background of the text, in the range from @start to @end.
