@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,6 @@
  */
 
 #include <libinftextgtk/inf-text-gtk-hue-chooser.h>
-#include <libinfinity/inf-marshal.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -72,9 +71,11 @@ enum {
 
 #define INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TEXT_GTK_TYPE_HUE_CHOOSER, InfTextGtkHueChooserPrivate))
 
-static GtkWidgetClass* parent_class;
 static guint hue_chooser_signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE_WITH_CODE(InfTextGtkHueChooser, inf_text_gtk_hue_chooser, GTK_TYPE_WIDGET,
+  G_ADD_PRIVATE(InfTextGtkHueChooser))
+
 static const double INF_TEXT_GTK_HUE_CHOOSER_HUE_MOVE_DELTA = 0.002;
 
 /*
@@ -83,82 +84,6 @@ static const double INF_TEXT_GTK_HUE_CHOOSER_HUE_MOVE_DELTA = 0.002;
 
 #define INTENSITY(r, g, b) ((r) * 0.30 + (g) * 0.59 + (b) * 0.11)
 
-/* TODO: Use gtk_hsv_to_rgb from GTK+ 2.14 instead */
-static void
-inf_text_gtk_hue_chooser_hsv_to_rgb(gdouble* h,
-                                    gdouble* s,
-                                    gdouble* v)
-{
-  gdouble hue;
-  gdouble saturation;
-  gdouble value;
-  gdouble f, p, q, t;
-
-  if(*s == 0.0)
-  {
-    *h = *v;
-    *s = *v;
-    *v = *v; /* heh */
-  }
-  else
-  {
-    hue = *h * 6.0;
-    saturation = *s;
-    value = *v;
-
-    if(hue == 6.0)
-      hue = 0.0;
-
-    f = hue - (int)hue;
-    p = value * (1.0 - saturation);
-    q = value * (1.0 - saturation * f);
-    t = value * (1.0 - saturation * (1.0 - f));
-
-    switch((int)hue)
-    {
-    case 0:
-      *h = value;
-      *s = t;
-      *v = p;
-      break;
-
-    case 1:
-      *h = q;
-      *s = value;
-      *v = p;
-      break;
-
-    case 2:
-      *h = p;
-      *s = value;
-      *v = t;
-      break;
-
-    case 3:
-      *h = p;
-      *s = q;
-      *v = value;
-      break;
-
-    case 4:
-      *h = t;
-      *s = p;
-      *v = value;
-      break;
-
-    case 5:
-      *h = value;
-      *s = p;
-      *v = q;
-      break;
-
-    default:
-      g_assert_not_reached ();
-      break;
-    }
-  }
-}
-
 static gboolean
 inf_text_gtk_hue_chooser_is_in_ring(InfTextGtkHueChooser* chooser,
                                     gdouble x,
@@ -174,11 +99,7 @@ inf_text_gtk_hue_chooser_is_in_ring(InfTextGtkHueChooser* chooser,
   gdouble outer;
   gdouble dist_sqr;
 
-#if GTK_CHECK_VERSION(2,18,0)
   gtk_widget_get_allocation(GTK_WIDGET(chooser), &allocation);
-#else
-  allocation = GTK_WIDGET(chooser)->allocation;
-#endif
 
   priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(chooser);
   center_x = allocation.width / 2.0;
@@ -204,11 +125,7 @@ inf_text_gtk_hue_chooser_hue_by_coords(InfTextGtkHueChooser* chooser,
   double dy;
   double angle;
 
-#if GTK_CHECK_VERSION(2,18,0)
   gtk_widget_get_allocation(GTK_WIDGET(chooser), &allocation);
-#else
-  allocation = GTK_WIDGET(chooser)->allocation;
-#endif
 
   center_x = allocation.width / 2.0;
   center_y = allocation.height / 2.0;
@@ -259,11 +176,7 @@ inf_text_gtk_hue_chooser_paint(InfTextGtkHueChooser* chooser,
     NULL
   );
 
-#if GTK_CHECK_VERSION(2,18,0)
   gtk_widget_get_allocation(GTK_WIDGET(chooser), &allocation);
-#else
-  allocation = GTK_WIDGET(chooser)->allocation;
-#endif
 
   center_x = allocation.width / 2.0;
   center_y = allocation.height / 2.0;
@@ -298,10 +211,7 @@ inf_text_gtk_hue_chooser_paint(InfTextGtkHueChooser* chooser,
 
       hue = angle / (2.0 * G_PI);
 
-      r = hue;
-      g = 1.0;
-      b = 1.0;
-      inf_text_gtk_hue_chooser_hsv_to_rgb(&r, &g, &b);
+      gtk_hsv_to_rgb(hue, 1.0, 1.0, &r, &g, &b);
 
       r_ = floor (r * 255 + 0.5);
       g_ = floor (g * 255 + 0.5);
@@ -322,14 +232,10 @@ inf_text_gtk_hue_chooser_paint(InfTextGtkHueChooser* chooser,
   );
 
   /* Now draw the value marker onto the source image, so that it
-   * will get properly clipped at the edges of the ring
-   */
+   * will get properly clipped at the edges of the ring */
   source_cr = cairo_create(source);
 
-  r = priv->hue;
-  g = 1.0;
-  b = 1.0;
-  inf_text_gtk_hue_chooser_hsv_to_rgb(&r, &g, &b);
+  gtk_hsv_to_rgb(priv->hue, 1.0, 1.0, &r, &g, &b);
 
   if(INTENSITY(r, g, b) > 0.5)
     cairo_set_source_rgb(source_cr, 0.0, 0.0, 0.0);
@@ -376,22 +282,13 @@ inf_text_gtk_hue_chooser_paint(InfTextGtkHueChooser* chooser,
  */
 
 static void
-inf_text_gtk_hue_chooser_init(GTypeInstance* instance,
-                              gpointer g_class)
+inf_text_gtk_hue_chooser_init(InfTextGtkHueChooser* hue_chooser)
 {
-  InfTextGtkHueChooser* hue_chooser;
   InfTextGtkHueChooserPrivate* priv;
-
-  hue_chooser = INF_TEXT_GTK_HUE_CHOOSER(instance);
   priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(hue_chooser);
 
-#if GTK_CHECK_VERSION(2,18,0)
   gtk_widget_set_has_window(GTK_WIDGET(hue_chooser), FALSE);
   gtk_widget_set_can_focus(GTK_WIDGET(hue_chooser), TRUE);
-#else
-  GTK_WIDGET_SET_FLAGS(hue_chooser, GTK_NO_WINDOW);
-  GTK_WIDGET_SET_FLAGS(hue_chooser, GTK_CAN_FOCUS);
-#endif
 
   priv->hue = 0.0;
   priv->window = NULL;
@@ -409,7 +306,7 @@ inf_text_gtk_hue_chooser_dispose(GObject* object)
   hue_chooser = INF_TEXT_GTK_HUE_CHOOSER(object);
   priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(hue_chooser);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_text_gtk_hue_chooser_parent_class)->dispose(object);
 }
 
 static void
@@ -421,7 +318,7 @@ inf_text_gtk_hue_chooser_finalize(GObject* object)
   hue_chooser = INF_TEXT_GTK_HUE_CHOOSER(object);
   priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(hue_chooser);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_text_gtk_hue_chooser_parent_class)->finalize(object);
 }
 
 static void
@@ -483,7 +380,7 @@ inf_text_gtk_hue_chooser_map(GtkWidget* widget)
   chooser = INF_TEXT_GTK_HUE_CHOOSER(widget);
   priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(chooser);
 
-  GTK_WIDGET_CLASS(parent_class)->map(widget);
+  GTK_WIDGET_CLASS(inf_text_gtk_hue_chooser_parent_class)->map(widget);
   gdk_window_show(priv->window);
 }
 
@@ -497,7 +394,7 @@ inf_text_gtk_hue_chooser_unmap(GtkWidget* widget)
   priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(chooser);
 
   gdk_window_hide(priv->window);
-  GTK_WIDGET_CLASS(parent_class)->unmap(widget);
+  GTK_WIDGET_CLASS(inf_text_gtk_hue_chooser_parent_class)->unmap(widget);
 }
 
 static void
@@ -512,17 +409,8 @@ inf_text_gtk_hue_chooser_realize(GtkWidget* widget)
   chooser = INF_TEXT_GTK_HUE_CHOOSER(widget);
   priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(chooser);
 
-#if GTK_CHECK_VERSION(2,20,0)
   gtk_widget_set_realized(widget, TRUE);
-#else
-  GTK_WIDGET_SET_FLAGS(widget, GTK_REALIZED);
-#endif
-
-#if GTK_CHECK_VERSION(2,18,0)
   gtk_widget_get_allocation(widget, &allocation);
-#else
-  allocation = widget->allocation;
-#endif
 
   attr.window_type = GDK_WINDOW_CHILD;
   attr.x = allocation.x;
@@ -543,18 +431,11 @@ inf_text_gtk_hue_chooser_realize(GtkWidget* widget)
   parent_window = gtk_widget_get_parent_window(widget);
   g_object_ref(parent_window);
 
-#if GTK_CHECK_VERSION(2,18,0)
   gtk_widget_set_window(widget, parent_window);
-#else
-  widget->window = parent_window;
-#endif
 
   priv->window = gdk_window_new(parent_window, &attr, GDK_WA_X | GDK_WA_Y);
   gdk_window_set_user_data(priv->window, chooser);
-  gtk_widget_set_style(
-    widget,
-    gtk_style_attach(gtk_widget_get_style(widget), parent_window)
-  );
+  gdk_window_show(priv->window);
 }
 
 static void
@@ -570,7 +451,7 @@ inf_text_gtk_hue_chooser_unrealize(GtkWidget* widget)
   gdk_window_destroy(priv->window);
   priv->window = NULL;
 
-  GTK_WIDGET_CLASS(parent_class)->unrealize(widget);
+  GTK_WIDGET_CLASS(inf_text_gtk_hue_chooser_parent_class)->unrealize(widget);
 }
 
 static gint
@@ -594,7 +475,6 @@ inf_text_gtk_hue_chooser_calculate_width_and_height(GtkWidget *widget)
   return priv->size + 2 * (focus_width + focus_pad);
 }
 
-#if GTK_CHECK_VERSION(2,91,6)
 static void
 inf_text_gtk_hue_chooser_get_preferred_width(GtkWidget *widget,
                                              gint *minimum,
@@ -613,17 +493,6 @@ inf_text_gtk_hue_chooser_get_preferred_height(GtkWidget *widget,
     inf_text_gtk_hue_chooser_calculate_width_and_height(widget);
 }
 
-#else
-
-static void
-inf_text_gtk_hue_chooser_size_request(GtkWidget* widget,
-                                      GtkRequisition* requisition)
-{
-  requisition->width = requisition->height =
-    inf_text_gtk_hue_chooser_calculate_width_and_height(widget);
-}
-#endif
-
 static void
 inf_text_gtk_hue_chooser_size_allocate(GtkWidget* widget,
                                        GtkAllocation* allocation)
@@ -634,17 +503,9 @@ inf_text_gtk_hue_chooser_size_allocate(GtkWidget* widget,
   chooser = INF_TEXT_GTK_HUE_CHOOSER(widget);
   priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(chooser);
 
-#if GTK_CHECK_VERSION(2,18,0)
   gtk_widget_set_allocation(widget, allocation);
-#else
-  widget->allocation = *allocation;
-#endif
 
-#if GTK_CHECK_VERSION(2,20,0)
   if(gtk_widget_get_realized(widget))
-#else
-  if(GTK_WIDGET_REALIZED(widget))
-#endif
   {
     /* TODO: Keep 1:1 aspect ratio, center within allocation, set size
      * accordingly, always request only ring_width * 2. */
@@ -688,7 +549,6 @@ inf_text_gtk_hue_chooser_button_press_event(GtkWidget* widget,
       GDK_CROSSHAIR
     );
 
-#if GTK_CHECK_VERSION(2, 91, 0)
     gdk_device_grab(
       gdk_event_get_device((GdkEvent*)event),
       priv->window,
@@ -700,19 +560,8 @@ inf_text_gtk_hue_chooser_button_press_event(GtkWidget* widget,
       cursor,
       event->time
     );
-#else
-    gdk_pointer_grab(
-      priv->window,
-      FALSE,
-      GDK_POINTER_MOTION_MASK |
-      GDK_POINTER_MOTION_HINT_MASK |
-      GDK_BUTTON_RELEASE_MASK,
-      NULL,
-      cursor,
-      event->time
-    );
-#endif
-    gdk_cursor_unref(cursor);
+
+    g_object_unref(cursor);
     gtk_widget_grab_focus(widget);
   }
 
@@ -738,17 +587,11 @@ inf_text_gtk_hue_chooser_button_release_event(GtkWidget* widget,
   hue = inf_text_gtk_hue_chooser_hue_by_coords(chooser, event->x, event->y);
   inf_text_gtk_hue_chooser_set_hue(chooser, hue);
 
-#if GTK_CHECK_VERSION(2, 91, 0)
   gdk_device_ungrab(
     gdk_event_get_device((GdkEvent*)event),
     event->time
   );
-#else
-  gdk_display_pointer_ungrab(
-    gdk_drawable_get_display(event->window),
-    event->time
-  );
-#endif
+
   return TRUE;
 }
 
@@ -774,7 +617,6 @@ inf_text_gtk_hue_chooser_motion_notify_event(GtkWidget* widget,
   return TRUE;
 }
 
-#if GTK_CHECK_VERSION(2,91,0)
 static gboolean
 inf_text_gtk_hue_chooser_draw(GtkWidget* widget,
                               cairo_t* cr)
@@ -796,90 +638,6 @@ inf_text_gtk_hue_chooser_draw(GtkWidget* widget,
 
   return FALSE;
 }
-#else
-static gboolean
-inf_text_gtk_hue_chooser_expose_event(GtkWidget* widget,
-                                      GdkEventExpose* event)
-{
-  InfTextGtkHueChooser* chooser;
-  InfTextGtkHueChooserPrivate* priv;
-  GtkAllocation allocation;
-  GdkRectangle rect;
-  GdkRectangle dest;
-  cairo_t* cr;
-
-  chooser = INF_TEXT_GTK_HUE_CHOOSER(widget);
-  priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(chooser);
-
-#if GTK_CHECK_VERSION(2,18,0)
-  if(!(gtk_widget_is_drawable(widget) &&
-       event->window == gtk_widget_get_window(widget)))
-  {
-    return FALSE;
-  }
-#else
-  if(!(GTK_WIDGET_DRAWABLE(widget) && event->window == widget->window))
-    return FALSE;
-#endif
-
-#if GTK_CHECK_VERSION(2,18,0)
-  gtk_widget_get_allocation(widget, &allocation);
-#else
-  allocation = widget->allocation;
-#endif
-
-  rect.x = allocation.x;
-  rect.y = allocation.y;
-  rect.width = allocation.width;
-  rect.height = allocation.height;
-
-  if(!gdk_rectangle_intersect(&event->area, &rect, &dest))
-    return FALSE;
-
-#if GTK_CHECK_VERSION(2,14,0)
-  cr = gdk_cairo_create(gtk_widget_get_window(widget));
-#else
-  cr = gdk_cairo_create(widget->window);
-#endif
-  cairo_translate(cr, allocation.x, allocation.y);
-
-  inf_text_gtk_hue_chooser_paint(
-    chooser,
-    cr,
-    dest.x - allocation.x,
-    dest.y - allocation.y,
-    dest.width,
-    dest.height
-  );
-
-  cairo_destroy(cr);
-
-#if GTK_CHECK_VERSION(2,18,0)
-  if(gtk_widget_has_focus(widget))
-#else
-  if(GTK_WIDGET_HAS_FOCUS(widget))
-#endif
-  {
-    /* This looks irritating: */
-#if 0
-    gtk_paint_focus(
-      widget->style,
-      widget->window,
-      GTK_WIDGET_STATE(widget),
-      &event->area,
-      widget,
-      NULL,
-      widget->allocation.x,
-      widget->allocation.y,
-      widget->allocation.width,
-      widget->allocation.height
-    );
-#endif
-  }
-
-  return FALSE;
-}
-#endif
 
 static gboolean
 inf_text_gtk_hue_chooser_focus(GtkWidget* widget,
@@ -891,11 +649,7 @@ inf_text_gtk_hue_chooser_focus(GtkWidget* widget,
   chooser = INF_TEXT_GTK_HUE_CHOOSER(widget);
   priv = INF_TEXT_GTK_HUE_CHOOSER_PRIVATE(chooser);
 
-#if GTK_CHECK_VERSION(2,18,0)
   if(!gtk_widget_has_focus(widget))
-#else
-  if(!GTK_WIDGET_HAS_FOCUS(widget))
-#endif
   {
     gtk_widget_grab_focus(GTK_WIDGET(widget));
     return TRUE;
@@ -971,40 +725,29 @@ inf_text_gtk_hue_chooser_move(InfTextGtkHueChooser* chooser,
  */
 
 static void
-inf_text_gtk_hue_chooser_class_init(gpointer g_class,
-                                    gpointer class_data)
+inf_text_gtk_hue_chooser_class_init(
+  InfTextGtkHueChooserClass* hue_chooser_class)
 {
-  GObjectClass* gobject_class;
+  GObjectClass* object_class;
   GtkWidgetClass* widget_class;
-  InfTextGtkHueChooserClass* hue_chooser_class;
   GtkBindingSet* binding_set;
 
-  gobject_class = G_OBJECT_CLASS(g_class);
-  widget_class = GTK_WIDGET_CLASS(g_class);
-  hue_chooser_class = INF_TEXT_GTK_HUE_CHOOSER_CLASS(g_class);
+  object_class = G_OBJECT_CLASS(hue_chooser_class);
+  widget_class = GTK_WIDGET_CLASS(hue_chooser_class);
 
-  parent_class = GTK_WIDGET_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfTextGtkHueChooserPrivate));
-
-  gobject_class->dispose = inf_text_gtk_hue_chooser_dispose;
-  gobject_class->finalize = inf_text_gtk_hue_chooser_finalize;
-  gobject_class->set_property = inf_text_gtk_hue_chooser_set_property;
-  gobject_class->get_property = inf_text_gtk_hue_chooser_get_property;
-
-  /*object_class->destroy = inf_text_gtk_hue_chooser_destroy;*/
+  object_class->dispose = inf_text_gtk_hue_chooser_dispose;
+  object_class->finalize = inf_text_gtk_hue_chooser_finalize;
+  object_class->set_property = inf_text_gtk_hue_chooser_set_property;
+  object_class->get_property = inf_text_gtk_hue_chooser_get_property;
 
   widget_class->map = inf_text_gtk_hue_chooser_map;
   widget_class->unmap = inf_text_gtk_hue_chooser_unmap;
   widget_class->realize = inf_text_gtk_hue_chooser_realize;
   widget_class->unrealize = inf_text_gtk_hue_chooser_unrealize;
-#if GTK_CHECK_VERSION(2,91,6)
   widget_class->get_preferred_height =
     inf_text_gtk_hue_chooser_get_preferred_height;
   widget_class->get_preferred_width =
     inf_text_gtk_hue_chooser_get_preferred_width;
-#else
-  widget_class->size_request = inf_text_gtk_hue_chooser_size_request;
-#endif
   widget_class->size_allocate = inf_text_gtk_hue_chooser_size_allocate;
   widget_class->button_press_event =
     inf_text_gtk_hue_chooser_button_press_event;
@@ -1012,11 +755,7 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
     inf_text_gtk_hue_chooser_button_release_event;
   widget_class->motion_notify_event =
     inf_text_gtk_hue_chooser_motion_notify_event;
-#if GTK_CHECK_VERSION(2,91,0)
   widget_class->draw = inf_text_gtk_hue_chooser_draw;
-#else
-  widget_class->expose_event = inf_text_gtk_hue_chooser_expose_event;
-#endif
   widget_class->focus = inf_text_gtk_hue_chooser_focus;
   widget_class->grab_broken_event = inf_text_gtk_hue_chooser_grab_broken_event;
 
@@ -1032,11 +771,11 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
    */
   hue_chooser_signals[HUE_CHANGE] = g_signal_new(
     "hue-change",
-    G_OBJECT_CLASS_TYPE(gobject_class),
+    G_OBJECT_CLASS_TYPE(object_class),
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfTextGtkHueChooserClass, hue_change),
     NULL, NULL,
-    inf_marshal_VOID__DOUBLE,
+    g_cclosure_marshal_VOID__DOUBLE,
     G_TYPE_NONE,
     1,
     G_TYPE_DOUBLE
@@ -1051,18 +790,18 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
    */
   hue_chooser_signals[MOVE] = g_signal_new(
     "move",
-    G_OBJECT_CLASS_TYPE(gobject_class),
+    G_OBJECT_CLASS_TYPE(object_class),
     G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
     G_STRUCT_OFFSET(InfTextGtkHueChooserClass, move),
     NULL, NULL,
-    inf_marshal_VOID__ENUM,
+    g_cclosure_marshal_VOID__ENUM,
     G_TYPE_NONE,
     1,
     GTK_TYPE_DIRECTION_TYPE
   );
 
   g_object_class_install_property(
-    gobject_class,
+    object_class,
     PROP_HUE,
     g_param_spec_double(
       "hue",
@@ -1075,15 +814,11 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
     )
   );
 
-  binding_set = gtk_binding_set_by_class(g_class);
+  binding_set = gtk_binding_set_by_class(object_class);
 
   gtk_binding_entry_add_signal(
     binding_set,
-#if GTK_CHECK_VERSION(2,90,7)
     GDK_KEY_Up,
-#else
-    GDK_Up,
-#endif
     0,
     "move",
     1,
@@ -1093,11 +828,7 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
 
   gtk_binding_entry_add_signal(
     binding_set,
-#if GTK_CHECK_VERSION(2,90,7)
     GDK_KEY_KP_Up,
-#else
-    GDK_KP_Up,
-#endif
     0,
     "move",
     1,
@@ -1107,11 +838,7 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
 
   gtk_binding_entry_add_signal(
     binding_set,
-#if GTK_CHECK_VERSION(2,90,7)
     GDK_KEY_Down,
-#else
-    GDK_Down,
-#endif
     0,
     "move",
     1,
@@ -1121,11 +848,7 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
 
   gtk_binding_entry_add_signal(
     binding_set,
-#if GTK_CHECK_VERSION(2,90,7)
     GDK_KEY_KP_Down,
-#else
-    GDK_KP_Down,
-#endif
     0,
     "move",
     1,
@@ -1135,11 +858,7 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
 
   gtk_binding_entry_add_signal(
     binding_set,
-#if GTK_CHECK_VERSION(2,90,7)
     GDK_KEY_Right,
-#else
-    GDK_Right,
-#endif
     0,
     "move",
     1,
@@ -1149,11 +868,7 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
 
   gtk_binding_entry_add_signal(
     binding_set,
-#if GTK_CHECK_VERSION(2,90,7)
     GDK_KEY_KP_Right,
-#else
-    GDK_KP_Right,
-#endif
     0,
     "move",
     1,
@@ -1163,11 +878,7 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
 
   gtk_binding_entry_add_signal(
     binding_set,
-#if GTK_CHECK_VERSION(2,90,7)
     GDK_KEY_Left,
-#else
-    GDK_Left,
-#endif
     0,
     "move",
     1,
@@ -1177,11 +888,7 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
 
   gtk_binding_entry_add_signal(
     binding_set,
-#if GTK_CHECK_VERSION(2,90,7)
     GDK_KEY_KP_Left,
-#else
-    GDK_KP_Left,
-#endif
     0,
     "move",
     1,
@@ -1190,48 +897,17 @@ inf_text_gtk_hue_chooser_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_text_gtk_hue_chooser_get_type(void)
-{
-  static GType hue_chooser_type = 0;
-
-  if(!hue_chooser_type)
-  {
-    static const GTypeInfo hue_chooser_type_info = {
-      sizeof(InfTextGtkHueChooserClass),   /* class_size */
-      NULL,                                /* base_init */
-      NULL,                                /* base_finalize */
-      inf_text_gtk_hue_chooser_class_init, /* class_init */
-      NULL,                                /* class_finalize */
-      NULL,                                /* class_data */
-      sizeof(InfTextGtkHueChooser),        /* instance_size */
-      0,                                   /* n_preallocs */
-      inf_text_gtk_hue_chooser_init,       /* instance_init */
-      NULL                                 /* value_table */
-    };
-
-    hue_chooser_type = g_type_register_static(
-      GTK_TYPE_WIDGET,
-      "InfTextGtkHueChooser",
-      &hue_chooser_type_info,
-      0
-    );
-  }
-
-  return hue_chooser_type;
-}
-
 /*
  * Public API
  */
 
 /**
- * inf_text_gtk_hue_chooser_new:
+ * inf_text_gtk_hue_chooser_new: (constructor)
  *
  * Creates a new #InfTextGtkHueChooser widget with the initial hue set to 0.0
  * (red).
  *
- * Return Value: A newly created #InfTextGtkHueChooser.
+ * Returns: (transfer floating): A newly created #InfTextGtkHueChooser.
  **/
 GtkWidget*
 inf_text_gtk_hue_chooser_new(void)
@@ -1242,13 +918,13 @@ inf_text_gtk_hue_chooser_new(void)
 }
 
 /**
- * inf_text_gtk_hue_chooser_new_with_hue:
+ * inf_text_gtk_hue_chooser_new_with_hue: (constructor)
  * @hue: Initial hue value
  *
  * Creates a new #InfTextGtkHueChooser widget with the given hue as initial
  * value. @hue must be between 0.0 and 1.0.
  *
- * Return Value: A newly created #InfTextGtkHueChooser.
+ * Returns: (transfer floating): A newly created #InfTextGtkHueChooser.
  **/
 GtkWidget*
 inf_text_gtk_hue_chooser_new_with_hue(gdouble hue)
