@@ -1,5 +1,5 @@
 /* infinote - Collaborative notetaking application
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -115,157 +115,8 @@ enum {
 
 #define INF_TEXT_GTK_VIEW_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TEXT_GTK_TYPE_VIEW, InfTextGtkViewPrivate))
 
-static GObjectClass* parent_class;
-
-/* Converts from HSV to RGB */
-/* TODO: Use gtk_hsv_to_rgb from GTK+ 2.14 instead */
-static void
-hsv_to_rgb(gdouble *h,
-           gdouble *s,
-           gdouble *v)
-{
-  gdouble hue, saturation, value;
-  gdouble f, p, q, t;
-
-  if (*s == 0.0)
-  {
-    *h = *v;
-    *s = *v;
-    *v = *v; /* heh */
-  }
-  else
-  {
-    hue = *h * 6.0;
-    saturation = *s;
-    value = *v;
-
-    if (hue == 6.0)
-      hue = 0.0;
-
-    f = hue - (int) hue;
-    p = value * (1.0 - saturation);
-    q = value * (1.0 - saturation * f);
-    t = value * (1.0 - saturation * (1.0 - f));
-
-    switch ((int) hue)
-    {
-    case 0:
-      *h = value;
-      *s = t;
-      *v = p;
-      break;
-
-    case 1:
-      *h = q;
-      *s = value;
-      *v = p;
-      break;
-
-    case 2:
-      *h = p;
-      *s = value;
-      *v = t;
-      break;
-
-    case 3:
-      *h = p;
-      *s = q;
-      *v = value;
-      break;
-
-    case 4:
-      *h = t;
-      *s = p;
-      *v = value;
-      break;
-
-    case 5:
-      *h = value;
-      *s = p;
-      *v = q;
-      break;
-
-    default:
-      g_assert_not_reached ();
-    }
-  }
-}
-
-/* Converts from RGB to HSV */
-/* TODO: Use gtk_rgb_to_hsv from GTK+ 2.14 instead */
-static void
-rgb_to_hsv (gdouble *r,
-            gdouble *g,
-            gdouble *b)
-{
-  gdouble red, green, blue;
-  gdouble h, s, v;
-  gdouble min, max;
-  gdouble delta;
-
-  red = *r;
-  green = *g;
-  blue = *b;
-
-  h = 0.0;
-
-  if (red > green)
-  {
-    if (red > blue)
-      max = red;
-    else
-      max = blue;
-
-    if (green < blue)
-      min = green;
-    else
-      min = blue;
-  }
-  else
-  {
-    if (green > blue)
-      max = green;
-    else
-      max = blue;
-
-    if (red < blue)
-      min = red;
-    else
-      min = blue;
-  }
-
-  v = max;
-
-  if (max != 0.0)
-    s = (max - min) / max;
-  else
-    s = 0.0;
-
-  if (s == 0.0)
-    h = 0.0;
-  else
-  {
-    delta = max - min;
-
-    if (red == max)
-      h = (green - blue) / delta;
-    else if (green == max)
-      h = 2 + (blue - red) / delta;
-    else if (blue == max)
-      h = 4 + (red - green) / delta;
-
-    h /= 6.0;
-
-    if (h < 0.0)
-      h += 1.0;
-    else if (h > 1.0)
-      h -= 1.0;
-  }
-
-  *r = h;
-  *g = s;
-  *b = v;
-}
+G_DEFINE_TYPE_WITH_CODE(InfTextGtkView, inf_text_gtk_view, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfTextGtkView))
 
 static InfTextGtkViewUser*
 inf_text_gtk_view_find_user(InfTextGtkView* view,
@@ -355,11 +206,7 @@ inf_text_gtk_view_get_left_margin(GtkTextView* view)
   gint margin;
   gint hadj;
 
-#if GTK_CHECK_VERSION(2,22,0)
-  hadjustment = gtk_text_view_get_hadjustment(view);
-#else
-  hadjustment = view->hadjustment;
-#endif
+  hadjustment = gtk_scrollable_get_hadjustment(GTK_SCROLLABLE(view));
 
   margin = gtk_text_view_get_left_margin(view);
   if(!hadjustment) return margin;
@@ -379,11 +226,7 @@ inf_text_gtk_view_get_right_margin(GtkTextView* view)
   gdouble hupper;
   gdouble hpage;
 
-#if GTK_CHECK_VERSION(2,22,0)
-  hadjustment = gtk_text_view_get_hadjustment(view);
-#else
-  hadjustment = view->hadjustment;
-#endif
+  hadjustment = gtk_scrollable_get_hadjustment(GTK_SCROLLABLE(view));
 
   margin = gtk_text_view_get_right_margin(view);
   if(!hadjustment) return margin;
@@ -420,22 +263,14 @@ inf_text_gtk_view_user_invalidate_user_area(InfTextGtkViewUser* view_user)
 
   priv = INF_TEXT_GTK_VIEW_PRIVATE(view_user->view);
 
-#if GTK_CHECK_VERSION(2,20,0)
   if(gtk_widget_get_realized(GTK_WIDGET(priv->textview)))
-#else
-  if(GTK_WIDGET_REALIZED(priv->textview))
-#endif
   {
     /* Invalidate cursors/selections */
     if(priv->show_remote_cursors || priv->show_remote_selections ||
        priv->show_remote_current_lines)
     {
       window = gtk_text_view_get_window(priv->textview, GTK_TEXT_WINDOW_TEXT);
-#if GTK_CHECK_VERSION(2, 91, 0)
       window_width = gdk_window_get_width(window);
-#else
-      gdk_drawable_get_size(GDK_DRAWABLE(window), &window_width, NULL);
-#endif
 
       gtk_text_view_buffer_to_window_coords(
         priv->textview,
@@ -651,15 +486,9 @@ inf_text_gtk_view_add_user_toggle_pair(GSequence* sequence,
 }
 
 static gboolean
-#if GTK_CHECK_VERSION(2, 91, 0)
 inf_text_gtk_view_draw_before_cb(GtkWidget* widget,
                                  cairo_t* cr,
                                  gpointer user_data)
-#else
-inf_text_gtk_view_expose_event_before_cb(GtkWidget* widget,
-                                         GdkEventExpose* event,
-                                         gpointer user_data)
-#endif
 {
   InfTextGtkView* view;
   InfTextGtkViewPrivate* priv;
@@ -671,7 +500,8 @@ inf_text_gtk_view_expose_event_before_cb(GtkWidget* widget,
   GtkAdjustment* vadjustment;
   GdkWindow *text_window;
 
-  GdkColor* color;
+  GtkStyleContext* style;
+  GdkRGBA bg;
   double h, s, v;
   double r, g, b;
 
@@ -679,11 +509,7 @@ inf_text_gtk_view_expose_event_before_cb(GtkWidget* widget,
   GdkRectangle rect;
   gint window_width;
   gint rx, ry;
-#if GTK_CHECK_VERSION(2, 91, 0)
   GdkRectangle clip_area;
-#else
-  cairo_t* cr;
-#endif
   cairo_pattern_t* pattern;
   double n, n_users;
   cairo_matrix_t matrix;
@@ -693,39 +519,27 @@ inf_text_gtk_view_expose_event_before_cb(GtkWidget* widget,
 
   text_window = gtk_text_view_get_window(priv->textview, GTK_TEXT_WINDOW_TEXT);
 
-#if GTK_CHECK_VERSION(2, 91, 0)
   if(!gtk_cairo_should_draw_window(cr, text_window))
-#else
-  if(text_window != event->window)
-#endif
   {
     return FALSE;
   }
 
   if(priv->show_remote_current_lines)
   {
-#if GTK_CHECK_VERSION(2, 91, 0)
     gtk_cairo_transform_to_window(cr, GTK_WIDGET(priv->textview), text_window);
 
     gdk_cairo_get_clip_rectangle(cr, &clip_area);
-#else
-    cr = gdk_cairo_create(text_window);
-#endif
 
-#if GTK_CHECK_VERSION(2, 91, 0)
-      window_width = gdk_window_get_width(text_window);
-#else
-      gdk_drawable_get_size(GDK_DRAWABLE(text_window), &window_width, NULL);
-#endif
+    window_width = gdk_window_get_width(text_window);
 
-    /* Make selection color based on text color: If text is dark, selection
-     * is dark, if text is bright selection is bright. Note that we draw with
-     * 50% alpha only, so text remains readable. */
-    color = &gtk_widget_get_style(widget)->bg[GTK_STATE_NORMAL];
-    h = color->red / 65535.0;
-    s = color->green / 65535.0;
-    v = color->blue / 65535.0;
-    rgb_to_hsv(&h, &s, &v);
+    /* Make current line color depend on background. */
+    style = gtk_widget_get_style_context(GTK_WIDGET(priv->textview));
+    gtk_style_context_save(style);
+    gtk_style_context_add_class(style, GTK_STYLE_CLASS_VIEW);
+    gtk_style_context_get_background_color(style, GTK_STATE_FLAG_NORMAL, &bg);
+    gtk_style_context_restore(style);
+
+    gtk_rgb_to_hsv(bg.red, bg.green, bg.blue, &h, &s, &v);
     v = MAX(v, 0.3);
     s = MAX(s, 0.1 + 0.3*(1 - v));
 
@@ -754,23 +568,13 @@ inf_text_gtk_view_expose_event_before_cb(GtkWidget* widget,
         rect.width = window_width - rect.x;
         rect.height = prev_user->line_height;
 
-#if GTK_CHECK_VERSION(2, 91, 0)
         if(gdk_rectangle_intersect(&clip_area, &rect, NULL))
-#elif GTK_CHECK_VERSION(2,90,5)
-        if(cairo_region_contains_rectangle(event->region, &rect) !=
-           CAIRO_REGION_OVERLAP_OUT)
-#else
-        if(gdk_region_rect_in(event->region, &rect) !=
-           GDK_OVERLAP_RECTANGLE_OUT)
-#endif
         {
-#if GTK_CHECK_VERSION(2,22,0)
-          hadjustment = gtk_text_view_get_hadjustment(priv->textview);
-          vadjustment = gtk_text_view_get_vadjustment(priv->textview);
-#else
-          hadjustment = priv->textview->hadjustment;
-          vadjustment = priv->textview->vadjustment;
-#endif
+          hadjustment =
+            gtk_scrollable_get_hadjustment(GTK_SCROLLABLE(priv->textview));
+          vadjustment =
+            gtk_scrollable_get_vadjustment(GTK_SCROLLABLE(priv->textview));
+
           /* Construct pattern */
           rx = gtk_adjustment_get_value(vadjustment);
           ry = gtk_adjustment_get_value(hadjustment);
@@ -786,8 +590,7 @@ inf_text_gtk_view_expose_event_before_cb(GtkWidget* widget,
           {
             view_user = (InfTextGtkViewUser*)prev_item->data;
             h = inf_text_user_get_hue(view_user->user);
-            r = h; g = s; b = v;
-            hsv_to_rgb(&r, &g, &b);
+            gtk_hsv_to_rgb(h, s, v, &r, &g, &b);
 
             cairo_pattern_add_color_stop_rgb(
               pattern,
@@ -815,30 +618,22 @@ inf_text_gtk_view_expose_event_before_cb(GtkWidget* widget,
     }
 
     g_slist_free(sort_users);
-
-#if ! GTK_CHECK_VERSION(2, 91, 0)
-    cairo_destroy(cr);
-#endif
   }
 
   return FALSE;
 }
 
 static gboolean
-#if GTK_CHECK_VERSION(2, 91, 0)
 inf_text_gtk_view_draw_after_cb(GtkWidget* widget,
                                 cairo_t* cr,
                                 gpointer user_data)
-#else
-inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
-                                        GdkEventExpose* event,
-                                        gpointer user_data)
-#endif
 {
   InfTextGtkView* view;
   InfTextGtkViewPrivate* priv;
   gint window_width;
+  GtkStyleContext* style;
   GdkColor* cursor_color;
+  GdkRGBA fg;
   double hc,sc,vc;
   double hs,ss,vs;
   GSList* item;
@@ -846,11 +641,7 @@ inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
   double rc,gc,bc;
   double rs,gs,bs;
 
-#if GTK_CHECK_VERSION(2, 91, 0)
   GdkRectangle clip_area;
-#else
-  cairo_t* cr;
-#endif
 
   gint ax, ay;
   GtkTextIter begin_iter;
@@ -888,39 +679,28 @@ inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
 
   text_window = gtk_text_view_get_window(priv->textview, GTK_TEXT_WINDOW_TEXT);
 
-#if GTK_CHECK_VERSION(2, 91, 0)
   if(!gtk_cairo_should_draw_window(cr, text_window))
-#else
-  if(text_window != event->window)
-#endif
   {
     return FALSE;
   }
 
-#if GTK_CHECK_VERSION(2, 91, 0)
   gtk_cairo_transform_to_window(cr, GTK_WIDGET(priv->textview), text_window);
   gdk_cairo_get_clip_rectangle(cr, &clip_area);
-#else
-  cr = gdk_cairo_create(text_window);
-#endif
+
+  style = gtk_widget_get_style_context(GTK_WIDGET(priv->textview));
+  gtk_style_context_save(style);
+  gtk_style_context_add_class(style, GTK_STYLE_CLASS_VIEW);
+  gtk_style_context_get_color(style, GTK_STATE_FLAG_NORMAL, &fg);
+  gtk_style_context_restore(style);
 
   if(priv->show_remote_selections)
   {
-#if GTK_CHECK_VERSION(2, 91, 0)
-      window_width = gdk_window_get_width(text_window);
-#else
-      gdk_drawable_get_size(GDK_DRAWABLE(text_window), &window_width, NULL);
-#endif
+    window_width = gdk_window_get_width(text_window);
 
     /* Make selection color based on text color: If text is dark, selection
      * is dark, if text is bright selection is bright. Note that we draw with
      * 50% alpha only, so text remains readable. */
-    cursor_color = &gtk_widget_get_style(widget)->text[GTK_STATE_NORMAL];
-    hs = cursor_color->red / 65535.0;
-    ss = cursor_color->green / 65535.0;
-    vs = cursor_color->blue / 65535.0;
-
-    rgb_to_hsv(&hs, &ss, &vs);
+    gtk_rgb_to_hsv(fg.red, fg.green, fg.blue, &hs, &ss, &vs);
     vs = MAX(vs, 0.5);
     ss = 1.0 - 0.4*(vs);
 
@@ -928,11 +708,7 @@ inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
     gtk_text_view_window_to_buffer_coords(
       priv->textview,
       GTK_TEXT_WINDOW_TEXT,
-#if GTK_CHECK_VERSION(2, 91, 0)
       clip_area.x, clip_area.y,
-#else
-      event->area.x, event->area.y,
-#endif
       &ax, &ay
     );
 
@@ -946,13 +722,8 @@ inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
     gtk_text_view_get_iter_at_location(
       priv->textview,
       &end_iter,
-#if GTK_CHECK_VERSION(2, 91, 0)
       ax + clip_area.width,
       ay + clip_area.height
-#else
-      ax + event->area.width,
-      ay + event->area.height
-#endif
     );
 
     area_begin = gtk_text_iter_get_offset(&begin_iter);
@@ -1143,13 +914,10 @@ inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
         g_assert(prev_toggle != NULL);
         g_assert(n_users > 0);
 
-  #if GTK_CHECK_VERSION(2,22,0)
-        hadjustment = gtk_text_view_get_hadjustment(priv->textview);
-        vadjustment = gtk_text_view_get_vadjustment(priv->textview);
-  #else
-        hadjustment = priv->textview->hadjustment;
-        vadjustment = priv->textview->vadjustment;
-  #endif
+        hadjustment =
+          gtk_scrollable_get_hadjustment(GTK_SCROLLABLE(priv->textview));
+        vadjustment =
+          gtk_scrollable_get_vadjustment(GTK_SCROLLABLE(priv->textview));
 
         /* Construct pattern */
         rx = gtk_adjustment_get_value(hadjustment);
@@ -1163,11 +931,7 @@ inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
         {
           view_user = ((InfTextGtkViewUserToggle*)item->data)->user;
           hs = inf_text_user_get_hue(view_user->user);
-
-          rs = hs;
-          gs = ss;
-          bs = vs;
-          hsv_to_rgb(&rs, &gs, &bs);
+          gtk_hsv_to_rgb(hs, ss, vs, &rs, &gs, &bs);
 
           cairo_pattern_add_color_stop_rgba(
             pattern,
@@ -1284,20 +1048,19 @@ inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
     gtk_widget_style_get (widget, "cursor-color", &cursor_color, NULL);
     if(cursor_color != NULL)
     {
-      hc = cursor_color->red / 65535.0;
-      sc = cursor_color->green / 65535.0;
-      vc = cursor_color->blue / 65535.0;
+      rc = cursor_color->red / 65535.0;
+      bc = cursor_color->green / 65535.0;
+      gc = cursor_color->blue / 65535.0;
       gdk_color_free(cursor_color);
     }
     else
     {
-      cursor_color = &gtk_widget_get_style(widget)->text[GTK_STATE_NORMAL];
-      hc = cursor_color->red / 65535.0;
-      sc = cursor_color->green / 65535.0;
-      vc = cursor_color->blue / 65535.0;
+      rc = fg.red;
+      bc = fg.green;
+      gc = fg.blue;
     }
 
-    rgb_to_hsv(&hc, &sc, &vc);
+    gtk_rgb_to_hsv(rc, bc, gc, &hc, &sc, &vc);
     sc = MIN(MAX(sc, 0.3), 0.8);
     vc = MAX(vc, 0.7);
 
@@ -1316,22 +1079,10 @@ inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
         rct.width = view_user->cursor_rect.width;
         rct.height = view_user->cursor_rect.height;
 
-#if GTK_CHECK_VERSION(2, 91, 0)
         if(gdk_rectangle_intersect(&clip_area, &rct, NULL))
-#elif GTK_CHECK_VERSION(2,90,5)
-        if(cairo_region_contains_rectangle(event->region, &rct) !=
-           CAIRO_REGION_OVERLAP_OUT)
-#else
-        if(gdk_region_rect_in(event->region, &rct) !=
-           GDK_OVERLAP_RECTANGLE_OUT)
-#endif
         {
           hc = inf_text_user_get_hue(view_user->user);
-
-          rc = hc;
-          gc = sc;
-          bc = vc;
-          hsv_to_rgb(&rc, &gc, &bc);
+          gtk_hsv_to_rgb(hc, sc, vc, &rc, &gc, &bc);
 
           cairo_set_source_rgb(cr, rc, gc, bc);
           gdk_cairo_rectangle(cr, &rct);
@@ -1341,17 +1092,12 @@ inf_text_gtk_view_expose_event_after_cb(GtkWidget* widget,
     }
   }
 
-#if ! GTK_CHECK_VERSION(2, 91, 0)
-  cairo_destroy(cr);
-#endif
-
   return FALSE;
 }
 
 static void
-inf_text_gtk_view_style_set_cb(GtkWidget* widget,
-                               GtkStyle* previous_style,
-                               gpointer user_data)
+inf_text_gtk_view_style_updated_cb(GtkWidget* widget,
+                                   gpointer user_data)
 {
   InfTextGtkView* view;
   InfTextGtkViewPrivate* priv;
@@ -1765,27 +1511,19 @@ inf_text_gtk_view_set_view(InfTextGtkView* view,
   {
     inf_signal_handlers_disconnect_by_func(
       G_OBJECT(priv->textview),
-#if GTK_CHECK_VERSION(2, 91, 0)
       G_CALLBACK(inf_text_gtk_view_draw_before_cb),
-#else
-      G_CALLBACK(inf_text_gtk_view_expose_event_before_cb),
-#endif
       view
     );
 
     inf_signal_handlers_disconnect_by_func(
       G_OBJECT(priv->textview),
-#if GTK_CHECK_VERSION(2, 91, 0)
       G_CALLBACK(inf_text_gtk_view_draw_after_cb),
-#else
-      G_CALLBACK(inf_text_gtk_view_expose_event_after_cb),
-#endif
       view
     );
 
     inf_signal_handlers_disconnect_by_func(
       G_OBJECT(priv->textview),
-      G_CALLBACK(inf_text_gtk_view_style_set_cb),
+      G_CALLBACK(inf_text_gtk_view_style_updated_cb),
       view
     );
 
@@ -1806,38 +1544,28 @@ inf_text_gtk_view_set_view(InfTextGtkView* view,
 
     g_signal_connect(
       G_OBJECT(gtk_view),
-#if GTK_CHECK_VERSION(2, 91, 0)
       "draw",
       G_CALLBACK(inf_text_gtk_view_draw_before_cb),
-#else
-      "expose-event",
-      G_CALLBACK(inf_text_gtk_view_expose_event_before_cb),
-#endif
       view
     );
 
     g_signal_connect_after(
       G_OBJECT(gtk_view),
-#if GTK_CHECK_VERSION(2, 91, 0)
       "draw",
       G_CALLBACK(inf_text_gtk_view_draw_after_cb),
-#else
-      "expose-event",
-      G_CALLBACK(inf_text_gtk_view_expose_event_after_cb),
-#endif
       view
     );
 
     g_signal_connect_after(
       G_OBJECT(gtk_view),
-      "style-set",
-      G_CALLBACK(inf_text_gtk_view_style_set_cb),
+      "style-updated",
+      G_CALLBACK(inf_text_gtk_view_style_updated_cb),
       view
     );
 
     /* This is required for the remote cursors showing up at the correct
      * position initially. Maybe gtk_text_view_get_iter_location() seems to
-     * return junk before. Note that also style-set is not enough. */
+     * return junk before. Note that also style-updated is not enough. */
     g_signal_connect_after(
       G_OBJECT(gtk_view),
       "size-allocate",
@@ -1910,13 +1638,9 @@ inf_text_gtk_view_set_user_table(InfTextGtkView* view,
 }
 
 static void
-inf_text_gtk_view_init(GTypeInstance* instance,
-                         gpointer g_class)
+inf_text_gtk_view_init(InfTextGtkView* view)
 {
-  InfTextGtkView* view;
   InfTextGtkViewPrivate* priv;
-
-  view = INF_TEXT_GTK_VIEW(instance);
   priv = INF_TEXT_GTK_VIEW_PRIVATE(view);
 
   priv->io = NULL;
@@ -1951,7 +1675,7 @@ inf_text_gtk_view_dispose(GObject* object)
     priv->io = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_text_gtk_view_parent_class)->dispose(object);
 }
 
 static void
@@ -2066,14 +1790,10 @@ inf_text_gtk_view_get_property(GObject* object,
 }
 
 static void
-inf_text_gtk_view_class_init(gpointer g_class,
-                               gpointer class_data)
+inf_text_gtk_view_class_init(InfTextGtkViewClass* view_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfTextGtkViewPrivate));
+  object_class = G_OBJECT_CLASS(view_class);
 
   object_class->dispose = inf_text_gtk_view_dispose;
   object_class->set_property = inf_text_gtk_view_set_property;
@@ -2164,39 +1884,8 @@ inf_text_gtk_view_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_text_gtk_view_get_type(void)
-{
-  static GType view_type = 0;
-
-  if(!view_type)
-  {
-    static const GTypeInfo view_type_info = {
-      sizeof(InfTextGtkViewClass),    /* class_size */
-      NULL,                           /* base_init */
-      NULL,                           /* base_finalize */
-      inf_text_gtk_view_class_init,   /* class_init */
-      NULL,                           /* class_finalize */
-      NULL,                           /* class_data */
-      sizeof(InfTextGtkView),         /* instance_size */
-      0,                              /* n_preallocs */
-      inf_text_gtk_view_init,         /* instance_init */
-      NULL                            /* value_table */
-    };
-
-    view_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfTextGtkView",
-      &view_type_info,
-      0
-    );
-  }
-
-  return view_type;
-}
-
 /**
- * inf_text_gtk_view_new:
+ * inf_text_gtk_view_new: (constructor)
  * @io: A #InfIo.
  * @view: A #GtkTextView.
  * @user_table: The #InfUserTable for the text session displayed in @view.
@@ -2204,7 +1893,7 @@ inf_text_gtk_view_get_type(void)
  * Creates a new #InfTextGtkView for @view. This draws remote user's cursors
  * and selections into the text view.
  *
- * Returns: A new #InfTextGtkView.
+ * Returns: (transfer full): A new #InfTextGtkView.
  */
 InfTextGtkView*
 inf_text_gtk_view_new(InfIo* io,
@@ -2234,7 +1923,7 @@ inf_text_gtk_view_new(InfIo* io,
  *
  * Returns the underlying #GtkTextView.
  *
- * Return Value: The #InfTextGtkView's #GtkTextView.
+ * Returns: (transfer none): The #InfTextGtkView's #GtkTextView.
  **/
 GtkTextView*
 inf_text_gtk_view_get_text_view(InfTextGtkView* view)
@@ -2250,7 +1939,7 @@ inf_text_gtk_view_get_text_view(InfTextGtkView* view)
  * Returns the #InfUserTable containing the users of the session the
  * #InfTextGtkView's #GtkTextView is displaying.
  *
- * Returns: The #InfTextGtkView's #InfUserTable.
+ * Returns: (transfer none): The #InfTextGtkView's #InfUserTable.
  */
 InfUserTable*
 inf_text_gtk_view_get_user_table(InfTextGtkView* view)
@@ -2262,7 +1951,7 @@ inf_text_gtk_view_get_user_table(InfTextGtkView* view)
 /**
  * inf_text_gtk_view_set_active_user:
  * @view: A #InfTextGtkView.
- * @user: A user from @view's user table, or %NULL.
+ * @user: (allow-none): A user from @view's user table, or %NULL.
  *
  * Sets the user for which perspective to draw the view. The selection and
  * cursor position is not drawn for this user since it is assumed that the
@@ -2312,7 +2001,7 @@ inf_text_gtk_view_set_active_user(InfTextGtkView* view,
  *
  * Returns the active user of @view. See inf_text_gtk_view_set_active_user().
  *
- * Returns: The active user of @view.
+ * Returns: (transfer none) (allow-none): The active user of @view.
  */
 InfTextUser*
 inf_text_gtk_view_get_active_user(InfTextGtkView* view)
