@@ -1,5 +1,5 @@
 /* infinote - Collaborative notetaking application
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,6 +16,26 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/**
+ * SECTION:inf-text-gtk-viewport
+ * @title: InfTextGtkViewport
+ * @short_description: Drawing position of remote users into the scrollbar
+ * @include: libinftextgtk/inf-text-gtk-viewport.h
+ * @see_also: #InfTextGtkView
+ * @stability: Unstable
+ *
+ * #InfTextGtkViewport is a helper object which, as long as it is alive,
+ * draws the location of the cursor of remote users into the scrollbar of
+ * a #GtkScrolledWindow containing a #GtkTextView. 
+ *
+ * The function inf_text_gtk_viewport_set_active_user() can be used to skip
+ * drawing the location of the user working locally on the document, if there
+ * is any.
+ *
+ * See #InfTextGtkView for drawing the cursor and selected region of remote
+ * users into a #GtkTextView.
+ */	
+
 #include <libinftextgtk/inf-text-gtk-viewport.h>
 #include <libinfinity/inf-signals.h>
 
@@ -52,157 +72,8 @@ enum {
 
 #define INF_TEXT_GTK_VIEWPORT_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TEXT_GTK_TYPE_VIEWPORT, InfTextGtkViewportPrivate))
 
-static GObjectClass* parent_class;
-
-/* Converts from HSV to RGB */
-/* TODO: Use gtk_hsv_to_rgb from GTK+ 2.14 instead */
-static void
-hsv_to_rgb(gdouble *h,
-           gdouble *s,
-           gdouble *v)
-{
-  gdouble hue, saturation, value;
-  gdouble f, p, q, t;
-
-  if (*s == 0.0)
-  {
-    *h = *v;
-    *s = *v;
-    *v = *v; /* heh */
-  }
-  else
-  {
-    hue = *h * 6.0;
-    saturation = *s;
-    value = *v;
-
-    if (hue == 6.0)
-      hue = 0.0;
-
-    f = hue - (int) hue;
-    p = value * (1.0 - saturation);
-    q = value * (1.0 - saturation * f);
-    t = value * (1.0 - saturation * (1.0 - f));
-
-    switch ((int) hue)
-    {
-    case 0:
-      *h = value;
-      *s = t;
-      *v = p;
-      break;
-
-    case 1:
-      *h = q;
-      *s = value;
-      *v = p;
-      break;
-
-    case 2:
-      *h = p;
-      *s = value;
-      *v = t;
-      break;
-
-    case 3:
-      *h = p;
-      *s = q;
-      *v = value;
-      break;
-
-    case 4:
-      *h = t;
-      *s = p;
-      *v = value;
-      break;
-
-    case 5:
-      *h = value;
-      *s = p;
-      *v = q;
-      break;
-
-    default:
-      g_assert_not_reached ();
-    }
-  }
-}
-
-/* Converts from RGB to HSV */
-/* TODO: Use gtk_rgb_to_hsv from GTK+ 2.14 instead */
-static void
-rgb_to_hsv (gdouble *r,
-            gdouble *g,
-            gdouble *b)
-{
-  gdouble red, green, blue;
-  gdouble h, s, v;
-  gdouble min, max;
-  gdouble delta;
-
-  red = *r;
-  green = *g;
-  blue = *b;
-
-  h = 0.0;
-
-  if (red > green)
-  {
-    if (red > blue)
-      max = red;
-    else
-      max = blue;
-
-    if (green < blue)
-      min = green;
-    else
-      min = blue;
-  }
-  else
-  {
-    if (green > blue)
-      max = green;
-    else
-      max = blue;
-
-    if (red < blue)
-      min = red;
-    else
-      min = blue;
-  }
-
-  v = max;
-
-  if (max != 0.0)
-    s = (max - min) / max;
-  else
-    s = 0.0;
-
-  if (s == 0.0)
-    h = 0.0;
-  else
-  {
-    delta = max - min;
-
-    if (red == max)
-      h = (green - blue) / delta;
-    else if (green == max)
-      h = 2 + (blue - red) / delta;
-    else if (blue == max)
-      h = 4 + (red - green) / delta;
-
-    h /= 6.0;
-
-    if (h < 0.0)
-      h += 1.0;
-    else if (h > 1.0)
-      h -= 1.0;
-  }
-
-  *r = h;
-  *g = s;
-  *b = v;
-}
+G_DEFINE_TYPE_WITH_CODE(InfTextGtkViewport, inf_text_gtk_viewport, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfTextGtkViewport))
 
 static InfTextGtkViewportUser*
 inf_text_gtk_viewport_find_user(InfTextGtkViewport* viewport,
@@ -247,13 +118,8 @@ inf_text_gtk_viewport_user_compute_user_area(InfTextGtkViewportUser* user)
 
   textview = gtk_bin_get_child(GTK_BIN(priv->scroll));
   scrollbar = gtk_scrolled_window_get_vscrollbar(priv->scroll);
-#if GTK_CHECK_VERSION(2,20,0)
   if(GTK_IS_TEXT_VIEW(textview) && scrollbar != NULL &&
      gtk_widget_get_realized(textview))
-#else
-  if(GTK_IS_TEXT_VIEW(textview) && scrollbar != NULL &&
-     GTK_WIDGET_REALIZED(textview))
-#endif
   {
     gtk_text_buffer_get_iter_at_offset(
       gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview)),
@@ -283,11 +149,7 @@ inf_text_gtk_viewport_user_compute_user_area(InfTextGtkViewportUser* user)
       NULL
     );
 
-#if GTK_CHECK_VERSION(2,18,0)
     gtk_widget_get_allocation(scrollbar, &allocation);
-#else
-    allocation = scrollbar->allocation;
-#endif
 
     scroll_ox = border;
     scroll_oy = border + stepper_size + stepper_spacing;
@@ -326,72 +188,67 @@ static void
 inf_text_gtk_viewport_user_invalidate_user_area(InfTextGtkViewportUser* user)
 {
   InfTextGtkViewportPrivate* priv;
+  GtkWidget* scrollbar;
+
   priv = INF_TEXT_GTK_VIEWPORT_PRIVATE(user->viewport);
 
   if(priv->show_user_markers &&
      user->rectangle.width > 0 && user->rectangle.height > 0)
   {
-    gtk_widget_queue_draw_area(
-      gtk_scrolled_window_get_vscrollbar(priv->scroll),
-      user->rectangle.x,
-      user->rectangle.y,
-      user->rectangle.width,
-      user->rectangle.height
-    );
+    scrollbar = gtk_scrolled_window_get_vscrollbar(priv->scroll);
+
+    /* During destruction of the widget it can happen that there is no
+     * vertical scrollbar anymore, so check for it here. */
+    if(scrollbar != NULL)
+    {
+      gtk_widget_queue_draw_area(
+        scrollbar,
+        user->rectangle.x,
+        user->rectangle.y,
+        user->rectangle.width,
+        user->rectangle.height
+      );
+    }
   }
 }
 
 static gboolean
-#if GTK_CHECK_VERSION(2, 91, 0)
 inf_text_gtk_viewport_scrollbar_draw_cb(GtkWidget* scrollbar,
                                         cairo_t* cr,
                                         gpointer user_data)
-#else
-inf_text_gtk_viewport_scrollbar_expose_event_cb(GtkWidget* scrollbar,
-                                                GdkEventExpose* event,
-                                                gpointer user_data)
-#endif
 {
   InfTextGtkViewport* viewport;
   InfTextGtkViewportPrivate* priv;
   InfTextGtkViewportUser* viewport_user;
   GdkRectangle* rectangle;
-  GdkColor* color;
+  GtkStyleContext* style;
+  GdkRGBA bg;
   double h,s,v;
   double r,g,b;
   GSList* item;
   double line_width;
 
-#if GTK_CHECK_VERSION(2, 91, 0)
   GdkRectangle clip_area;
-#else
-  cairo_t* cr;
-#endif
 
   viewport = INF_TEXT_GTK_VIEWPORT(user_data);
   priv = INF_TEXT_GTK_VIEWPORT_PRIVATE(viewport);
 
   /* Can this happen? */
-#if GTK_CHECK_VERSION(2, 91, 0)
   if(!gtk_cairo_should_draw_window(cr, gtk_widget_get_window(scrollbar)))
-#elif GTK_CHECK_VERSION(2,14,0)
-  if(event->window != gtk_widget_get_window(scrollbar))
-#else
-  if(event->window != GTK_WIDGET(scrollbar)->window)
-#endif
     return FALSE;
 
   if(priv->show_user_markers)
   {
-    color = &gtk_widget_get_style(scrollbar)->bg[GTK_STATE_NORMAL];
-    h = color->red / 65535.0;
-    s = color->green / 65535.0;
-    v = color->blue / 65535.0;
-    rgb_to_hsv(&h, &s, &v);
+    style = gtk_widget_get_style_context(GTK_WIDGET(scrollbar));
+    gtk_style_context_save(style);
+    gtk_style_context_add_class(style, GTK_STYLE_CLASS_VIEW);
+    gtk_style_context_get_background_color(style, GTK_STATE_FLAG_NORMAL, &bg);
+    gtk_style_context_restore(style);
+
+    gtk_rgb_to_hsv(bg.red, bg.green, bg.blue, &h, &s, &v);
     s = MIN(MAX(s, 0.5), 0.8);
     v = MAX(v, 0.5);
 
-#if GTK_CHECK_VERSION(2, 91, 0)
     gtk_cairo_transform_to_window(
       cr,
       GTK_WIDGET(scrollbar),
@@ -399,9 +256,6 @@ inf_text_gtk_viewport_scrollbar_expose_event_cb(GtkWidget* scrollbar,
     );
 
     gdk_cairo_get_clip_rectangle(cr, &clip_area);
-#else
-    cr = gdk_cairo_create(event->window);
-#endif
 
     line_width = cairo_get_line_width(cr);
     for(item = priv->users; item != NULL; item = item->next)
@@ -409,15 +263,7 @@ inf_text_gtk_viewport_scrollbar_expose_event_cb(GtkWidget* scrollbar,
       viewport_user = (InfTextGtkViewportUser*)item->data;
       rectangle = &viewport_user->rectangle;
 
-#if GTK_CHECK_VERSION(2, 91, 0)
       if(gdk_rectangle_intersect(&clip_area, rectangle, NULL))
-#elif GTK_CHECK_VERSION(2,90,5)
-      if(cairo_region_contains_rectangle(event->region, rectangle) !=
-         CAIRO_REGION_OVERLAP_OUT)
-#else
-      if(gdk_region_rect_in(event->region, rectangle) !=
-         GDK_OVERLAP_RECTANGLE_OUT)
-#endif
       {
         h = inf_text_user_get_hue(viewport_user->user);
 
@@ -429,21 +275,15 @@ inf_text_gtk_viewport_scrollbar_expose_event_cb(GtkWidget* scrollbar,
           rectangle->height - line_width
         );
 
-        r = h; g = s; b = v/2.0;
-        hsv_to_rgb(&r, &g, &b);
+        gtk_hsv_to_rgb(h, s, v/2.0, &r, &g, &b);
         cairo_set_source_rgba(cr, r, g, b, 0.6);
         cairo_stroke_preserve(cr);
 
-        r = h; g = s; b = v;
-        hsv_to_rgb(&r, &g, &b);
+        gtk_hsv_to_rgb(h, s, v, &r, &g, &b);
         cairo_set_source_rgba(cr, r, g, b, 0.6);
         cairo_fill(cr);
       }
     }
-
-#if ! GTK_CHECK_VERSION(2, 91, 0)
-    cairo_destroy(cr);
-#endif
   }
 
   return FALSE;
@@ -493,9 +333,8 @@ inf_text_gtk_viewport_adjustment_changed_cb(GtkAdjustment* adjustment,
 }
 
 static void
-inf_text_gtk_viewport_scrollbar_style_set_cb(GtkWidget* scrollbar,
-                                             GtkStyle* prev_style,
-                                             gpointer user_data)
+inf_text_gtk_viewport_scrollbar_style_updated_cb(GtkWidget* scrollbar,
+                                                 gpointer user_data)
 {
   InfTextGtkViewport* viewport;
   InfTextGtkViewportPrivate* priv;
@@ -764,17 +603,13 @@ inf_text_gtk_viewport_set_scrolled_window(InfTextGtkViewport* viewport,
 
       inf_signal_handlers_disconnect_by_func(
         G_OBJECT(scrollbar),
-        G_CALLBACK(inf_text_gtk_viewport_scrollbar_style_set_cb),
+        G_CALLBACK(inf_text_gtk_viewport_scrollbar_style_updated_cb),
         viewport
       );
 
       inf_signal_handlers_disconnect_by_func(
         G_OBJECT(scrollbar),
-#if GTK_CHECK_VERSION(2, 91, 0)
         G_CALLBACK(inf_text_gtk_viewport_scrollbar_draw_cb),
-#else
-        G_CALLBACK(inf_text_gtk_viewport_scrollbar_expose_event_cb),
-#endif
         viewport
       );
     }
@@ -811,26 +646,17 @@ inf_text_gtk_viewport_set_scrolled_window(InfTextGtkViewport* viewport,
 
     g_signal_connect_after(
       G_OBJECT(scrollbar),
-      "style-set",
-      G_CALLBACK(inf_text_gtk_viewport_scrollbar_style_set_cb),
+      "style-updated",
+      G_CALLBACK(inf_text_gtk_viewport_scrollbar_style_updated_cb),
       viewport
     );
 
-#if GTK_CHECK_VERSION(2, 91, 0)
     g_signal_connect_after(
       G_OBJECT(scrollbar),
       "draw",
       G_CALLBACK(inf_text_gtk_viewport_scrollbar_draw_cb),
       viewport
     );
-#else
-    g_signal_connect_after(
-      G_OBJECT(scrollbar),
-      "expose-event",
-      G_CALLBACK(inf_text_gtk_viewport_scrollbar_expose_event_cb),
-      viewport
-    );
-#endif
   }
 
   g_object_notify(G_OBJECT(viewport), "scrolled-window");
@@ -897,13 +723,9 @@ inf_text_gtk_viewport_set_user_table(InfTextGtkViewport* viewport,
 }
 
 static void
-inf_text_gtk_viewport_init(GTypeInstance* instance,
-                           gpointer g_class)
+inf_text_gtk_viewport_init(InfTextGtkViewport* viewport)
 {
-  InfTextGtkViewport* viewport;
   InfTextGtkViewportPrivate* priv;
-
-  viewport = INF_TEXT_GTK_VIEWPORT(instance);
   priv = INF_TEXT_GTK_VIEWPORT_PRIVATE(viewport);
 
   priv->scroll = NULL;
@@ -923,13 +745,13 @@ inf_text_gtk_viewport_dispose(GObject* object)
   viewport = INF_TEXT_GTK_VIEWPORT(object);
   priv = INF_TEXT_GTK_VIEWPORT_PRIVATE(viewport);
 
-  inf_text_gtk_viewport_set_scrolled_window(viewport, NULL);
   inf_text_gtk_viewport_set_user_table(viewport, NULL);
+  inf_text_gtk_viewport_set_scrolled_window(viewport, NULL);
 
   g_assert(priv->active_user == NULL);
   g_assert(priv->users == NULL);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_text_gtk_viewport_parent_class)->dispose(object);
 }
 
 static void
@@ -1017,14 +839,10 @@ inf_text_gtk_viewport_get_property(GObject* object,
 }
 
 static void
-inf_text_gtk_viewport_class_init(gpointer g_class,
-                                 gpointer class_data)
+inf_text_gtk_viewport_class_init(InfTextGtkViewportClass* viewport_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfTextGtkViewportPrivate));
+  object_class = G_OBJECT_CLASS(viewport_class);
 
   object_class->dispose = inf_text_gtk_viewport_dispose;
   object_class->set_property = inf_text_gtk_viewport_set_property;
@@ -1081,46 +899,15 @@ inf_text_gtk_viewport_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_text_gtk_viewport_get_type(void)
-{
-  static GType viewport_type = 0;
-
-  if(!viewport_type)
-  {
-    static const GTypeInfo viewport_type_info = {
-      sizeof(InfTextGtkViewportClass),    /* class_size */
-      NULL,                               /* base_init */
-      NULL,                               /* base_finalize */
-      inf_text_gtk_viewport_class_init,   /* class_init */
-      NULL,                               /* class_finalize */
-      NULL,                               /* class_data */
-      sizeof(InfTextGtkViewport),         /* instance_size */
-      0,                                  /* n_preallocs */
-      inf_text_gtk_viewport_init,         /* instance_init */
-      NULL                                /* value_table */
-    };
-
-    viewport_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfTextGtkViewport",
-      &viewport_type_info,
-      0
-    );
-  }
-
-  return viewport_type;
-}
-
 /**
- * inf_text_gtk_viewport_new:
+ * inf_text_gtk_viewport_new: (constructor)
  * @scroll: A #GtkScrolledWindow.
  * @user_table: The #InfUserTable for the text session displayed in @viewport.
  *
  * Creates a new #InfTextGtkViewport for @scroll. This draws the position of
  * remote user's cursors into the scrollbars of @scroll.
  *
- * Returns: A new #InfTextGtkViewport.
+ * Returns: (transfer full): A new #InfTextGtkViewport.
  */
 InfTextGtkViewport*
 inf_text_gtk_viewport_new(GtkScrolledWindow* scroll,
@@ -1147,7 +934,7 @@ inf_text_gtk_viewport_new(GtkScrolledWindow* scroll,
  *
  * Returns the underlying #GtkScrolledWindow.
  *
- * Return Value: The #InfTextGtkViewport's #GtkScrolledWindow.
+ * Returns: (transfer none): The #InfTextGtkViewport's #GtkScrolledWindow.
  **/
 GtkScrolledWindow*
 inf_text_gtk_viewport_get_scrolled_window(InfTextGtkViewport* viewport)
@@ -1163,7 +950,7 @@ inf_text_gtk_viewport_get_scrolled_window(InfTextGtkViewport* viewport)
  * Returns the #InfUserTable containing the users of the session the
  * #InfTextGtkViewport's #GtkScrolledWindow is displaying.
  *
- * Returns: The #InfGtkTextViewport's #InfUserTable.
+ * Returns: (transfer none): The #InfTextGtkViewport's #InfUserTable.
  */
 InfUserTable*
 inf_text_gtk_viewport_get_user_table(InfTextGtkViewport* viewport)
@@ -1175,7 +962,7 @@ inf_text_gtk_viewport_get_user_table(InfTextGtkViewport* viewport)
 /**
  * inf_text_gtk_viewport_set_active_user:
  * @viewport: A #InfTextGtkViewport.
- * @user: A user from @viewport's user table, or %NULL.
+ * @user: (allow-none): A user from @viewport's user table, or %NULL.
  *
  * Sets the user for which perspective to draw the viewport. The cursor
  * position for teh active user is not draws since it is assumed that the
@@ -1224,7 +1011,7 @@ inf_text_gtk_viewport_set_active_user(InfTextGtkViewport* viewport,
  * Returns the active user of @viewport. See
  * inf_text_gtk_viewport_set_active_user().
  *
- * Returns: The active user of @viewport.
+ * Returns: (transfer none) (allow-none): The active user of @viewport.
  */
 InfTextUser*
 inf_text_gtk_viewport_get_active_user(InfTextGtkViewport* viewport)
